/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.EventObject;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse stellt den passenden Event zum OberflaechenBerechtigungListener
 * zur Verfgung. Mit ihm knnen "Oberflchenelement" darber informiert werden,
 * ob sie nun gesperrt oder freigegeben sind. Elemente, die diese Information
 * verarbeiten, mssen sich dem entsprechend ein- oder ausgrauen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: BerechtigungEreignis.java 45393 2013-08-07 21:32:57Z peuker $
 */
public class BerechtigungEreignis extends EventObject {

	private static final long serialVersionUID = 1L;

	/**
	 * bei <code>true</code> erfolgt die Freigabe, bei <code>false</code> nicht.
	 */
	private final boolean freigabe;

	/** die Berechtigungsklasse. */
	private final SystemObject berechtigungsKlasse;

	/** die betroffenen Funktionen. */
	private final List<FunktionMitBerechtigung> funktionen = new ArrayList<FunktionMitBerechtigung>();

	/**
	 * Konstruktor des Events. Im Objekt <code>source</code> wird das Objekt,
	 * auf welches sich der Event bezieht, bergeben.
	 * 
	 * @param source
	 *            Quelle des Events.
	 * @param freigabe
	 *            Freigabe erfolgt wenn <code>true</code>, bei
	 *            <code>false</code> erfolgt die Sperrung.
	 * @param berechtigungsKlasse
	 *            die betroffene Beerchtigungsklasse
	 * @param funktionen
	 *            die betroffenen Funktionen
	 */
	public BerechtigungEreignis(final Object source, final boolean freigabe,
			final SystemObject berechtigungsKlasse,
			final Collection<FunktionMitBerechtigung> funktionen) {
		super(source);
		this.freigabe = freigabe;
		this.berechtigungsKlasse = berechtigungsKlasse;
		this.funktionen.addAll(funktionen);
	}

	/**
	 * ermittelt, ob eine Freigabe gemeldet wird.
	 * 
	 * @return der Zustand
	 */
	public boolean isFreigabe() {
		return freigabe;
	}

	/**
	 * liefert die betroffene Berechtigungsklasse.
	 * 
	 * @return die Berechtigungsklasse
	 */
	public SystemObject getBerechtigungsKlasse() {
		return berechtigungsKlasse;
	}

	/**
	 * liefert die betroffenen Funktionen.
	 * 
	 * @return eine Liste der Funktionen
	 */
	public List<FunktionMitBerechtigung> getFunktionen() {
		return Collections.unmodifiableList(funktionen);
	}
}
