/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.berechtigung;

import java.io.Serializable;

import org.eclipse.core.runtime.Assert;

/**
 * Klasse FunktionMitBerechtigung.
 * 
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: FunktionMitBerechtigung.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class FunktionMitBerechtigung implements Serializable {

	/** ID zur Serialisierung. */
	private static final long serialVersionUID = 1L;

	/** Standardstring fr Funktionen, denen keine Kategorie zugewiesen wurde. */
	public static final String NO_CATEGORY = "Keine Kategorie";

	/** die Plug-in-ID des Ursprungs-Plugins. */
	private final String pluginId;

	/** die Kategorie der Funktion. */
	private final String kategorie;

	/** die ID der Funktion. */
	private final String id;

	/** die Bezeichnung der Funktion. */
	private final String bezeichnung;

	/** die Beschreibung der Funktion. */
	private final String beschreibung;

	/**
	 * Konsruktor.
	 * 
	 * @param pluginId
	 *            die ID des definierenden Plugins
	 * @param kategorie
	 *            die Kategorie (optional)
	 * @param id
	 *            die ID der Funktioen
	 * @param bezeichnung
	 *            die Bezeichnung der Funktion
	 * @param beschreibung
	 *            eine optionale Beschreibung der Funktion
	 */
	public FunktionMitBerechtigung(final String pluginId,
			final String kategorie, final String id, final String bezeichnung,
			final String beschreibung) {

		Assert.isNotNull(id, "id");
		this.pluginId = pluginId;

		if (kategorie == null) {
			this.kategorie = FunktionMitBerechtigung.NO_CATEGORY;
		} else {
			this.kategorie = kategorie;
		}

		if (pluginId == null) {
			this.id = id;
		} else {
			this.id = pluginId + "." + id;
		}

		if (bezeichnung == null) {
			this.bezeichnung = id;
		} else {
			this.bezeichnung = bezeichnung;
		}

		if (bezeichnung == null) {
			this.beschreibung = "";
		} else {
			this.beschreibung = beschreibung;
		}
	}

	/**
	 * liefert die der Funktion.
	 * 
	 * @return die ID
	 */
	public String getId() {
		return id;
	}

	/**
	 * liefert die Bezeichnung der Funktion.
	 * 
	 * @return die Bezeichnung
	 */
	public String getBezeichnung() {
		return bezeichnung;
	}

	/**
	 * liefert die Beschreibung der Funktion.
	 * 
	 * @return die Beschreibung
	 */
	public String getBeschreibung() {
		return beschreibung;
	}

	/**
	 * liefert die definierte Kategorie oder {@value #NO_CATEGORY}, wenn keine
	 * festgelegt wurde.
	 * 
	 * @return die Kategorie oder {@value #NO_CATEGORY}
	 */
	public String getKategorie() {
		return kategorie;
	}

	/**
	 * die ID des definierenden Plug-ins.
	 * 
	 * @return die ID
	 */
	public String getPluginId() {
		return pluginId;
	}

	@Override
	public String toString() {
		return bezeichnung;
	}
}
