/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.berechtigung;

/**
 * Interface fr die Listener der Oberflaechenberechtigung.
 * 
 * Durch die Implementierung dieses Listeners knnen sich Klassen ber die
 * Freigabe oder Sperrung des entsprechenden Oberflchenelementes informieren.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: IBerechtigungListener.java 45807 2013-09-03 08:37:06Z peuker $
 */
public interface IBerechtigungListener {
	/**
	 * Dieser Listener wird aktiv, sobald die Sperrung erfolgt ist.
	 * 
	 * Fr die Implementierung des Listeners muss die Funktion
	 * <code>e.hasBerechtigung</code> == false liefern.
	 * 
	 * Dann kann mittels <code>e.getSystemObjekte</code> die Liste der
	 * Systemobjekte abgerufen werden (z.B. fr eine Ausgabe) abgerufen werden,
	 * wegen denen die Sperrung erfolgt.
	 * 
	 * @param e
	 *            OberflaechenBerechtigungEvent, der ausgelst wurde.
	 */
	void sperrung(BerechtigungEreignis e);

	/**
	 * Dieser Listener wird aktiv, sobald die Freigabe erfolgt ist.
	 * 
	 * Fr die Implementierung des Listeners muss die Funktion
	 * <code>e.hasBerechtigung</code> == true liefern.
	 * 
	 * @param e
	 *            OberflaechenBerechtigungEvent, der ausgelst wurde.
	 */
	void freigabe(BerechtigungEreignis e);
}
