/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.einstellungen;

import java.util.EventObject;

/**
 * Event das ber die {@link EinstellungChangeListener} verbreitet wird.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: EinstellungsEvent.java 45393 2013-08-07 21:32:57Z peuker $
 */
public class EinstellungsEvent extends EventObject {

	private static final long serialVersionUID = 1L;

	private final EinstellungsAdresse adresse;
	private final Object einstellung;

	/**
	 * Konstruktor.
	 * 
	 * @param source
	 *            die Quelle des Events
	 * @param adresse
	 *            die Einstellungadresse
	 * @param einstellung
	 *            das Einstellungsobjekt
	 */
	public EinstellungsEvent(final Object source,
			final EinstellungsAdresse adresse, final Object einstellung) {
		super(source);
		this.adresse = adresse;
		this.einstellung = einstellung;
	}

	/**
	 * liefert die Einstellungsadresse.
	 * 
	 * @return die Adresse
	 */
	public EinstellungsAdresse getAdresse() {
		return adresse;
	}

	/**
	 * liefert das Einstellungsobjekt.
	 * 
	 * @return das Objekt
	 */
	public Object getEinstellung() {
		return einstellung;
	}
}
