/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.util.Date;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.kalender.DatumZeit.Eingabetyp;

/**
 * Composite zur Darstellung von Zeitbereichen mit Plausibilittsprfung. Ist
 * Endzeitpunkt < Startzeitpunkt, wird eine entsprechende Meldung dargestellt.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: CompositeZeitBereich.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class CompositeZeitBereich extends Composite {

	private DatumZeit von;
	private DatumZeit bis;
	private Label labelInfo;

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *            Eltern {@link Composite}.
	 * @param style
	 *            SWT-Stil.
	 */
	public CompositeZeitBereich(final Composite parent, final int style) {
		super(parent, style);
		initialize();
	}

	/**
	 * Fhrt die Initialisierugn des GUI dieses Composites durch.
	 */
	private void initialize() {
		final GridData gridDataBis = new GridData(150, SWT.DEFAULT);
		gridDataBis.horizontalAlignment = GridData.BEGINNING;
		gridDataBis.grabExcessHorizontalSpace = false;
		gridDataBis.verticalAlignment = GridData.CENTER;
		final GridData gridDataVon = new GridData(150, SWT.DEFAULT);
		gridDataVon.horizontalAlignment = GridData.BEGINNING;
		gridDataVon.grabExcessHorizontalSpace = false;
		gridDataVon.verticalAlignment = GridData.CENTER;
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 5;
		final Label labelVon = new Label(this, SWT.NONE);
		labelVon.setText("Von:");
		von = new DatumZeit(this, SWT.NONE,
				DatumZeit.Eingabetyp.datumuhrEinfachMS, false, false);
		von.setLayoutData(gridDataVon);

		final Label labelBis = new Label(this, SWT.NONE);
		labelBis.setText("Bis:");

		bis = new DatumZeit(this, SWT.NONE, Eingabetyp.datumuhrEinfachMS,
				false, false);
		bis.setLayoutData(gridDataBis);

		labelInfo = new Label(this, SWT.NONE);
		labelInfo.setForeground(new Color(PlatformUI.getWorkbench()
				.getDisplay(), 255, 0, 0));
		labelInfo.setText("Startzeitpunkt liegt vor dem Endzeitpunkt!");
		labelInfo.setVisible(false);

		von.hinzufuegenSelektionsZuhoerer(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(final SelectionEvent e) {
				gemeinsam(e);
			}

			@Override
			public void widgetSelected(final SelectionEvent e) {
				gemeinsam(e);
			}

			private void gemeinsam(final SelectionEvent e) {
				if (e.data instanceof Date) {
					final Date startDatum = (Date) e.data;
					final Date endDatum = bis.getDatum();
					labelSetzen(startDatum, endDatum);

				}
			}

		});

		bis.hinzufuegenSelektionsZuhoerer(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(final SelectionEvent e) {
				gemeinsam(e);
			}

			@Override
			public void widgetSelected(final SelectionEvent e) {
				gemeinsam(e);
			}

			private void gemeinsam(final SelectionEvent e) {
				if (e.data instanceof Date) {
					final Date startDatum = von.getDatum();
					final Date endDatum = (Date) e.data;
					labelSetzen(startDatum, endDatum);

				}
			}

		});

		this.setLayout(gridLayout);
	}

	/**
	 * Setzt den Startzeitpunkt.
	 * 
	 * @param start
	 *            Startzeitpunkt.
	 */
	public void setzeStartZeitPunkt(final Date start) {
		von.setDatum(start);
		labelSetzen(von.getDatum(), bis.getDatum());
	}

	/**
	 * Liefert den Startzeitpunkt.
	 * 
	 * @return Startzeitpunkt.
	 */
	public Date getStartZeitPunkt() {
		return von.getDatum();
	}

	/**
	 * Setzt den Endzeitpunkt.
	 * 
	 * @param ende
	 *            Endzeitpunkt.
	 */
	public void setzeEndZeitPunkt(final Date ende) {
		bis.setDatum(ende);
		labelSetzen(von.getDatum(), bis.getDatum());
	}

	/**
	 * Liefert den Endzeitpunkt.
	 * 
	 * @return Endzeitpunkt.
	 */
	public Date getEndZeitPunkt() {
		return bis.getDatum();
	}

	/**
	 * Zeigt oder versteckt das Label zur Darstellung, dass die
	 * Plausibilittsprfung gescheitert ist.
	 * 
	 * @param start
	 *            Startzeitpunkt.
	 * @param ende
	 *            Endzeitpunkt.
	 */
	private void labelSetzen(final Date start, final Date ende) {
		if (pruefePlausibilitaet(start, ende)) {
			labelInfo.setVisible(false);
		} else {
			labelInfo.setVisible(true);
		}
	}

	/**
	 * Prft die Plausibilitt von Start- und Endzeitpunkt.
	 * 
	 * @param start
	 *            Startzeitpunkt.
	 * @param ende
	 *            Endzeitpunkt.
	 * @return Liefert <code>true</code> wenn plausiebel, sonst
	 *         <code>false</code> .
	 */
	private boolean pruefePlausibilitaet(final Date start, final Date ende) {
		boolean re = false;

		if ((start != null) && (ende != null) && (!(start.compareTo(ende) > 0))) {
			re = true;
		}

		return re;
	}

}
