/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.util.Date;
import java.util.List;

import org.eclipse.nebula.widgets.cdatetime.CDT;
import org.eclipse.nebula.widgets.cdatetime.CDateTime;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;

import de.bsvrz.buv.rw.basislib.kalender.internal.IFelderFuellen;
import de.bsvrz.buv.rw.basislib.kalender.internal.SchaltflaechenFabrikEreignisSystemKalenderDialog;

/**
 * Eingabefeld fr Datum + Zeit. <br>
 * <br>
 * In der Klasse KalenderDatum wird ein Eingabefeld des Datums zur Verfgung
 * gestellt. Auer durch die Tastatureingabe gibt es noch die Mglichkeit der
 * Eingabe ber einen Kalenderauswahldialog oder ber einen
 * Ereignissauswahldialog. Die Eingabemglichkeiten knnen per Konstuktor
 * eingestellt werden. Ebenso kann die Granularitt der Eingabe eingesetllt
 * werden, d.h. dass die kleinste einstellbare Zeiteinheit vorgegeben werden
 * kann. Wird als Eingabemglichkeit die Ereignisauswahl ermglicht, so wird
 * eine Verbindung zum Datenverteiler bentigt, da die Ereignisse aus dem
 * Ereignisskalender kommen. <br>
 * Die Klasse erfllt folgende Technische Anforderungen des Segments Bedienung
 * und Visualisierung (BUV)<br>
 * - TBuV-36
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: DatumZeit.java 51000 2014-07-17 12:33:42Z peuker $
 */
public class DatumZeit extends Composite {
	/**
	 * Spezifiziert den Eingabetyp.
	 */
	private final Eingabetyp eingabetyp;

	/**
	 * VorgabeDatum.
	 */
	private Date vorgabeDatum;

	/**
	 * CDateTime widget.
	 */
	private CDateTime cdt;

	/**
	 * Flag Anzeigen Ereigniskalender.
	 */
	private final boolean ereigniskalender;

	/**
	 * Flag Anzeigen Systemkalender.
	 */
	private final boolean systemkalender;

	/**
	 * Gibt an, welches Feld eines Bereichs bercksichtigt werden soll (von oder
	 * bis).
	 */
	private BereichsFeld bereichsFeld;

	/**
	 * Konstruktor.
	 *
	 * @param parent
	 *            Eltern-Composite, auf dessen Basis die Darstellung erfolgen
	 *            soll.
	 * @param style
	 *            SWT-Stil.
	 * @param eingabetyp
	 *            Eingabetyp, bestimmt die Gestalt des KalenderZeit-Elementes an
	 *            Hand des Enumerators {@link Eingabetyp}.
	 * @param ereigniskalender
	 *            Wenn <code>true</code>, dann werden die
	 *            Ereigniskalendereintrge fr den aktuellen Monat dargestellt,
	 *            bei <code>false</code> nicht. Funktioniert nur, wenn ein
	 *            Eingabetyp gewhlt ist, der die Datumseingabe beinhaltet.
	 * @param systemkalender
	 *            Wenn <code>true</code>, dann werden die Systemkalendereintrge
	 *            fr den aktuellen Monat dargestellt, bei <code>false</code>
	 *            nicht. Funktioniert nur, wenn ein Eingabetyp gewhlt ist, der
	 *            die Datumseingabe beinhaltet.
	 */
	public DatumZeit(final Composite parent, final int style,
			final Eingabetyp eingabetyp, final boolean ereigniskalender,
			final boolean systemkalender) {
		super(parent, style);
		this.eingabetyp = eingabetyp;
		this.ereigniskalender = ereigniskalender;
		this.systemkalender = systemkalender;
		init();
	}

	/**
	 * Konstruktor.
	 *
	 * @param parent
	 *            Eltern-Composite, auf dessen Basis die Darstellung erfolgen
	 *            soll.
	 * @param style
	 *            SWT-Stil.
	 * @param eingabetyp
	 *            Eingabetyp, bestimmt die Gestalt des KalenderZeit-Elementes an
	 *            Hand des Enumerators {@link Eingabetyp}.
	 * @param ereigniskalender
	 *            Wenn <code>true</code>, dann werden die
	 *            Ereigniskalendereintrge fr den aktuellen Monat dargestellt,
	 *            bei <code>false</code> nicht. Funktioniert nur, wenn ein
	 *            Eingabetyp gewhlt ist, der die Datumseingabe beinhaltet.
	 * @param systemkalender
	 *            Wenn <code>true</code>, dann werden die Systemkalendereintrge
	 *            fr den aktuellen Monat dargestellt, bei <code>false</code>
	 *            nicht. Funktioniert nur, wenn ein Eingabetyp gewhlt ist, der
	 *            die Datumseingabe beinhaltet.
	 * @param bereichsFeld
	 *            definiert, welche Seite eines Zeitbereichs bearbeitet wird
	 */
	public DatumZeit(final Composite parent, final int style,
			final Eingabetyp eingabetyp, final boolean ereigniskalender,
			final boolean systemkalender, final BereichsFeld bereichsFeld) {
		super(parent, style);
		this.eingabetyp = eingabetyp;
		this.ereigniskalender = ereigniskalender;
		this.systemkalender = systemkalender;
		this.bereichsFeld = bereichsFeld;
		init();
	}

	/**
	 * Konstruktor.
	 *
	 * @param parent
	 *            Eltern-Composite, auf dessen Basis die Darstellung erfolgen
	 *            soll.
	 * @param style
	 *            SWT-Stil
	 * @param eingabetyp
	 *            Eingabetyp, bestimmt die Gestalt des KalenderZeit-Elementes an
	 *            Hand des Enumerators {@link Eingabetyp}.
	 * @param vorgabeDatum
	 *            Vorgabe Datum/Uhrzeit als Instanz der Klasse {@link Date}.
	 *            Wenn <code>null</code>, dann wird vom aktuellen Datum
	 *            ausgegangen.
	 * @param ereigniskalender
	 *            Wenn <code>true</code>, dann werden die
	 *            Ereigniskalendereintrge fr den aktuellen Monat dargestellt,
	 *            bei <code>false</code> nicht. Funktioniert nur, wenn ein
	 *            Eingabetyp gewhlt ist, der die Datumseingabe beinhaltet.
	 * @param systemkalender
	 *            Wenn <code>true</code>, dann werden die Systemkalendereintrge
	 *            fr den aktuellen Monat dargestellt, bei <code>false</code>
	 *            nicht. Funktioniert nur, wenn ein Eingabetyp gewhlt ist, der
	 *            die Datumseingabe beinhaltet.
	 */
	public DatumZeit(final Composite parent, final int style,
			final Eingabetyp eingabetyp, final Date vorgabeDatum,
			final boolean ereigniskalender, final boolean systemkalender) {
		super(parent, style);
		this.eingabetyp = eingabetyp;
		this.vorgabeDatum = new Date(vorgabeDatum.getTime());
		this.ereigniskalender = ereigniskalender;
		this.systemkalender = systemkalender;
		init();
	}

	/**
	 * Konstruktor.
	 *
	 * @param parent
	 *            Eltern-Composite, auf dessen Basis die Darstellung erfolgen
	 *            soll.
	 * @param style
	 *            SWT-Stil
	 * @param eingabetyp
	 *            Eingabetyp, bestimmt die Gestalt des KalenderZeit-Elementes an
	 *            Hand des Enumerators {@link Eingabetyp}.
	 * @param vorgabeDatum
	 *            Vorgabe Datum/Uhrzeit als Instanz der Klasse {@link Date}.
	 *            Wenn <code>null</code>, dann wird vom aktuellen Datum
	 *            ausgegangen.
	 * @param ereigniskalender
	 *            Wenn <code>true</code>, dann werden die
	 *            Ereigniskalendereintrge fr den aktuellen Monat dargestellt,
	 *            bei <code>false</code> nicht. Funktioniert nur, wenn ein
	 *            Eingabetyp gewhlt ist, der die Datumseingabe beinhaltet.
	 * @param systemkalender
	 *            Wenn <code>true</code>, dann werden die Systemkalendereintrge
	 *            fr den aktuellen Monat dargestellt, bei <code>false</code>
	 *            nicht. Funktioniert nur, wenn ein Eingabetyp gewhlt ist, der
	 *            die Datumseingabe beinhaltet.
	 * @param bereichsFeld
	 *            definiert, welche Seite eines Bereiches definiert wird
	 */
	public DatumZeit(final Composite parent, final int style,
			final Eingabetyp eingabetyp, final Date vorgabeDatum,
			final boolean ereigniskalender, final boolean systemkalender,
			final BereichsFeld bereichsFeld) {
		super(parent, style);
		this.eingabetyp = eingabetyp;
		this.vorgabeDatum = new Date(vorgabeDatum.getTime());
		this.ereigniskalender = ereigniskalender;
		this.systemkalender = systemkalender;
		this.bereichsFeld = bereichsFeld;
		init();
	}

	private void init() {
		erzeugeDarstellung();

		if (vorgabeDatum != null) {
			cdt.setSelection(vorgabeDatum);
		}
	}

	/**
	 * Setzt das Datum (mit Uhrzeit).
	 *
	 * @param datum
	 *            Datum
	 */
	public void setDatum(final Date datum) {
		cdt.setSelection(datum);
	}

	/**
	 * Liefert das selektierte Datum (mit Uhrzeit).
	 *
	 * @return Datum.
	 */
	public Date getDatum() {
		return cdt.getSelection();
	}

	/**
	 * Hinzufuegen eines {@link SelectionListener}. Im Data-Feld des Events wird
	 * das aktuelle, selektierte Datum bertragen!
	 *
	 * @param zuhoerer
	 *            {@link SelectionListener}.
	 */
	public void hinzufuegenSelektionsZuhoerer(final SelectionListener zuhoerer) {
		if (cdt != null) {
			cdt.addSelectionListener(zuhoerer);
		}
	}

	/**
	 * entfernt einen Zuhrer, der bei nderungen der Selekiton benachrichtigt
	 * wurde.
	 *
	 * @param zuhoerer
	 *            der Zuhrer
	 */
	public void entferneSelektionsZuhoerer(final SelectionListener zuhoerer) {
		if (cdt != null) {
			cdt.removeSelectionListener(zuhoerer);
		}
	}

	private void erzeugeDarstellung() {
		int width = SWT.DEFAULT;

		switch (eingabetyp) {
		case datum:
			erzeugeDatum();
			break;
		case datumuhr:
			erzeugeDatumUhr();
			width = 130;
			break;
		case uhr:
			erzeugeUhr();
			break;
		case datumuhrMS:
			erzeugeDatumUhrMS();
			width = 150;
			break;
		case datumOhneJahr:
			erzeugeDatumOhneJahr();
			break;
		case uhrMS:
			erzeugeUhrMS();
			break;
		case datumEinfach:
			erzeugeDatumEinfach();
			break;
		case datumuhrEinfach:
			erzeugeDatumUhrEinfach();
			width = 130;
			break;
		case uhrEinfach:
			erzeugeUhrEinfach();
			break;
		case datumuhrEinfachMS:
			erzeugeDatumUhrEinfachMitMS();
			width = 150;
			break;
		case uhrEinfachMS:
		default:
			erzeugeUhrEinfachMitMS();
			break;
		}

		final GridData gridData = new GridData(width, SWT.DEFAULT);
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		cdt.setLayoutData(gridData);

		if (ereigniskalender || systemkalender) {
			erzeugeEreignisSystemkalender();
		}

		final GridLayout gridLayout = new GridLayout();

		if (ereigniskalender || systemkalender) {
			gridLayout.numColumns = 2;
		} else {
			gridLayout.numColumns = 1;
		}

		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 0;
		gridLayout.marginLeft = 0;
		gridLayout.marginRight = 0;
		gridLayout.marginTop = 0;
		gridLayout.marginBottom = 0;
		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;

		this.setLayout(gridLayout);

		this.update();
	}

	private void erzeugeEreignisSystemkalender() {

		final IFelderFuellen felderFuellen = new IFelderFuellen() {

			@Override
			public void fuellen(final List<IEintragBereich> bereiche) {
				if (bereiche.isEmpty()) {
					aktionFuellen(null);
				} else {
					aktionFuellen(bereiche.get(0));
				}
			}

		};
		final Button button = SchaltflaechenFabrikEreignisSystemKalenderDialog
				.erzeugeButton(this, felderFuellen, null, null);
		final GridData gd = new GridData();
		gd.grabExcessHorizontalSpace = false;
		gd.grabExcessVerticalSpace = false;
		gd.horizontalAlignment = GridData.BEGINNING;
		gd.verticalAlignment = GridData.CENTER;
		button.setLayoutData(gd);

	}

	private void aktionFuellen(final IEintragBereich bereich) {
		if (bereich != null) {
			if (bereichsFeld == null) {
				setDatum(new Date(bereich.getVon()));
			} else {
				if (bereichsFeld.equals(BereichsFeld.vonFeld)) {
					setDatum(new Date(bereich.getVon()));
				} else if (bereichsFeld.equals(BereichsFeld.bisFeld)) {
					setDatum(new Date(bereich.getBis()));
				}
			}
		}
	}

	private void erzeugeDatum() {
		cdt = new CDateTime(this, CDT.SPINNER | CDT.DROP_DOWN | CDT.DATE_MEDIUM);
		cdt.setEnabled(isEnabled());
	}

	private void erzeugeDatumUhr() {
		cdt = new CDateTime(this, CDT.DROP_DOWN | CDT.DATE_MEDIUM
				| CDT.TIME_MEDIUM | CDT.CLOCK_24_HOUR);
		cdt.setEnabled(isEnabled());
	}

	private void erzeugeUhr() {
		cdt = new CDateTime(this, CDT.SPINNER | CDT.DROP_DOWN | CDT.TIME_MEDIUM
				| CDT.CLOCK_24_HOUR);
		cdt.setEnabled(isEnabled());
	}

	private void erzeugeDatumUhrMS() {
		cdt = new CDateTime(this, CDT.SPINNER | CDT.DROP_DOWN | CDT.DATE_MEDIUM
				| CDT.TIME_MEDIUM | CDT.CLOCK_24_HOUR);
		cdt.setEnabled(isEnabled());

		/*
		 * Pattern
		 */
		cdt.setPattern("dd.MM.yyyy HH:mm:ss:SSS");

	}

	private void erzeugeUhrMS() {
		cdt = new CDateTime(this, CDT.SPINNER | CDT.DROP_DOWN
				| CDT.CLOCK_24_HOUR);
		cdt.setEnabled(isEnabled());

		/*
		 * Pattern
		 */
		cdt.setPattern("HH:mm:ss:SSS");
	}

	private void erzeugeDatumEinfach() {
		cdt = new CDateTime(this, CDT.SPINNER | CDT.DATE_MEDIUM);
		cdt.setEnabled(isEnabled());
	}

	private void erzeugeDatumUhrEinfach() {
		cdt = new CDateTime(this, CDT.SPINNER | CDT.DATE_MEDIUM
				| CDT.TIME_MEDIUM);
		cdt.setEnabled(isEnabled());
	}

	private void erzeugeUhrEinfach() {
		cdt = new CDateTime(this, CDT.SPINNER | CDT.TIME_MEDIUM);
		cdt.setEnabled(isEnabled());
	}

	/**
	 * Erzeugt ein Datum+Uhrzeitelement, mit dessen Hilfe auch Millisekunden
	 * erfasst werden knnen.
	 *
	 */
	private void erzeugeDatumUhrEinfachMitMS() {
		cdt = new CDateTime(this, CDT.SPINNER | CDT.DATE_MEDIUM
				| CDT.TIME_MEDIUM);
		cdt.setEnabled(isEnabled());

		/*
		 * Pattern
		 */
		cdt.setPattern("dd.MM.yyyy HH:mm:ss:SSS");

		/*
		 * Elementdimension neu bestimmen.
		 */
		final Point size = cdt.getSize();
		final int breite = size.x;
		final int hoehe = size.y;
		cdt.setSize(breite + 15, hoehe);

	}

	/**
	 * Erzeugt ein Datum+Uhrzeitelement, mit dessen Hilfe auch Millisekunden
	 * erfasst werden knnen.
	 *
	 */
	private void erzeugeDatumOhneJahr() {
		cdt = new CDateTime(this, CDT.SPINNER | CDT.DATE_MEDIUM
				| CDT.TIME_MEDIUM);
		cdt.setEnabled(isEnabled());

		/*
		 * Pattern
		 */
		cdt.setPattern("dd.MM.");

		/*
		 * Elementdimension neu bestimmen.
		 */
		final Point size = cdt.getSize();
		final int breite = size.x;
		final int hoehe = size.y;
		cdt.setSize(breite + 15, hoehe);

	}

	/**
	 * Erzeugt ein Uhrzeitelement, mit dessen Hife auch Millisekunden erfasst
	 * werden knnen.
	 *
	 */
	private void erzeugeUhrEinfachMitMS() {
		cdt = new CDateTime(this, CDT.SPINNER | CDT.TIME_MEDIUM);
		cdt.setEnabled(isEnabled());

		/*
		 * Pattern
		 */
		cdt.setPattern("HH:mm:ss:SSS");
		final Point size = cdt.getSize();
		final int breite = size.x;
		final int hoehe = size.y;
		cdt.setSize(breite + 15, hoehe);
	}

	/**
	 * Enumerator zur Angabe welches Feld des {@link IEintragBereich}es
	 * bercksichtigt werden soll.
	 *
	 * @author HofmannJ
	 *
	 */
	public enum BereichsFeld {
		/**
		 * Von Feld des {@link IEintragBereich}es.
		 */
		vonFeld,

		/**
		 * Bis Feld des {@link IEintragBereich}es.
		 */
		bisFeld
	}

	/**
	 * Enumerator fr die Auswahl von Datum und Uhrzeit.
	 *
	 * @author HofmannJ
	 *
	 */
	public enum Eingabetyp {
		/**
		 * Nur Datum.
		 */
		datum,
		/**
		 * Datum und Uhr.
		 */
		datumuhr,
		/**
		 * Nur Uhr.
		 */
		uhr,
		/**
		 * Datum und Uhr.
		 */
		datumuhrMS,
		/**
		 * Nur Uhr.
		 */
		uhrMS,
		/**
		 * Datum einfach.
		 */
		datumEinfach,
		/**
		 * Datum + Uhr einfach.
		 */
		datumuhrEinfach,
		/**
		 * Uhr einfach.
		 */
		uhrEinfach,
		/**
		 * Datum + Uhr einfach, mit Millisekunden.
		 */
		datumuhrEinfachMS,
		/**
		 * Datum ohne Jahr.
		 */
		datumOhneJahr,
		/**
		 * Datum + Uhr einfach mit Millisekunden.
		 */
		uhrEinfachMS
	}

	@Override
	public void notifyListeners(final int eventType, final Event event) {
		// TODO Auto-generated method stub
		cdt.notifyListeners(eventType, event);
	}

	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		if (cdt != null) {
			cdt.setEnabled(enabled);
		}
	}
}
