/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.internal;

import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.basislib.kalender.IEintragBereich;

/**
 * Kapsel fr Ereigniskalendereintrge.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: ErkaKapsel.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class ErkaKapsel implements Comparable<ErkaKapsel>, IEintragKapsel {

	/** Verweis auf einen {@link EintragEreigniskalender}. */
	private final EintragEreigniskalender eintragEreigniskalender;

	/** Speichert die Liste der {@link IEintragBereich}e. */
	private final List<IEintragBereich> bereiche = new LinkedList<IEintragBereich>();

	/**
	 * Konstruktor. bernimmt einen {@link EintragEreigniskalender}.
	 * 
	 * @param eintrag
	 *            {@link EintragEreigniskalender}
	 * @param sysKalAnfrageArbeiter
	 *            Schnittstelle zum Systemkalender
	 * @param startAuswahlDialog
	 *            Startzeitpunkt des zulssigen Bereichs
	 * @param endeAuswahlDialog
	 *            Endzeitpunkt des zulssigen Bereichs
	 */
	public ErkaKapsel(final EintragEreigniskalender eintrag,
			final SysKalAnfrageArbeiter sysKalAnfrageArbeiter,
			final Long startAuswahlDialog, final Long endeAuswahlDialog) {
		this.eintragEreigniskalender = eintrag;

		final Long anfang = eintragEreigniskalender.getAnfangsZeitpunkt();
		final Long ende = eintragEreigniskalender.getEndZeitpunkt();

		if (anfang.equals(ende)) {
			if (eintragEreigniskalender.getSystemObject() == null) {
				final ErkaBereich bereich = new ErkaBereich(anfang, ende);
				bereiche.add(bereich);
			} else {
				final IEintragKapsel kapsel = sysKalAnfrageArbeiter
						.berechneGueltigVonBisS(eintragEreigniskalender
								.getSystemObject().getPid(),
								startAuswahlDialog, endeAuswahlDialog);

				bereiche.addAll(kapsel.getBereiche());
			}
		} else {
			final ErkaBereich bereich = new ErkaBereich(anfang, ende);
			bereiche.add(bereich);
		}

	}

	@Override
	public void addBereich(final Long von, final Long bis) {
		// FIXME Wieso passiert hier nichts ?
	}

	@Override
	public void addBereich(final IEintragBereich bereich) {
		this.bereiche.add(bereich);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result)
				+ ((getPid() == null) ? 0 : getPid().hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final ErkaKapsel other = (ErkaKapsel) obj;
		if (getPid() == null) {
			if (other.getPid() != null) {
				return false;
			}
			return true;
		} else if (!getPid().equals(other.getPid())) {
			return false;
		}
		return true;
	}

	@Override
	public int compareTo(final ErkaKapsel o) {
		return this.getPid().compareTo(o.getPid());
	}

	@Override
	public List<IEintragBereich> getBereiche() {
		return bereiche;
	}

	@Override
	public int getBereicheAnzahl() {
		int re = 0;

		if (bereiche != null) {
			re = bereiche.size();
		}

		return re;
	}

	@Override
	public String getPid() {
		String re = null;
		if (eintragEreigniskalender != null) {
			re = eintragEreigniskalender.getPid();
		}
		return re;
	}

	@Override
	public String toString() {
		final String re = eintragEreigniskalender.getName() + " (Pid: "
				+ eintragEreigniskalender.getPid() + ")";
		return re;
	}

	@Override
	public String toStringLong() {
		return null;
	}
}
