/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.internal;

import java.util.List;

import de.bsvrz.buv.rw.basislib.kalender.IEintragBereich;

/**
 * Schnittstelle fr die Kapselung eines System- oder Ereigniskalendereintrags.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: IEintragKapsel.java 47112 2013-11-12 15:25:14Z peuker $
 */
public interface IEintragKapsel {

	/**
	 * Liefert die Pid des Systemkalendereintrags.
	 * 
	 * @return die PID
	 */
	String getPid();

	/**
	 * Fgt einen neuen {@link IEintragBereich}, fr den dieser
	 * SystemkalenderEintrag Gltigkeit hat, zur Liste der Bereiche hinzu. Der
	 * {@link IEintragBereich} wird aus den Von- und Bis-Zeitpunkten des
	 * Bereiches erstellt.
	 * 
	 * @param von
	 *            Von-Zeitpunkt
	 * @param bis
	 *            Bis-Zeitpunkt
	 */
	void addBereich(Long von, Long bis);

	/**
	 * Fgt einen {@link IEintragBereich} zur Liste der Bereiche hinzu.
	 * 
	 * @param bereich
	 *            Hinzuzufgender {@link IEintragBereich}.
	 */
	void addBereich(IEintragBereich bereich);

	/**
	 * Liefert eine Liste von {@link IEintragBereich}en, fr den dieser
	 * Systemkalendereintrag Gltigkeit hat.
	 * 
	 * @return Liste gltiger {@link IEintragBereich}e.
	 */
	List<IEintragBereich> getBereiche();

	/**
	 * Liefert die Anzahl der Bereiche zu einem Eintrag.
	 * 
	 * @return Anzahl der Bereiche.
	 */
	int getBereicheAnzahl();

	@Override
	String toString();

	/**
	 * liefert eine spezielle Stringreprsentation des gekapselten Eintrags?
	 * 
	 * @return die Zeichenkette
	 */
	String toStringLong();
}
