/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.internal;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.SortedMap;
import java.util.TreeMap;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderArbeiter;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderEintrag;

/**
 * Schnittstelle zur Kommunikation mit dem Systemkalender.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: SysKalAnfrageArbeiter.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class SysKalAnfrageArbeiter {

	private SystemkalenderArbeiter syskalArbeiter;

	/**
	 * Konstruktor.
	 * 
	 * @param connection
	 *            die Datenverteilerverbindung
	 * @param kalender
	 *            der Kalender
	 */
	public SysKalAnfrageArbeiter(final ClientDavInterface connection,
			final String kalender) {
		syskalArbeiter = SystemkalenderArbeiter.getInstance(connection,
				kalender);
	}

	/**
	 * ermittelt die Systemkalendereintrge fr den bergebenen Zeitbereich.
	 * 
	 * @param von
	 *            Anfangszeitpunkt fr die Abfrage
	 * @param bis
	 *            Endzeitpunkt fr die Abfrage
	 * @return die Liste der ermittelten Eintrge
	 */
	public List<SkeKapsel> berechneGueltigVonBisS(final Long von, final Long bis) {
		final LinkedList<SkeKapsel> listeKapseln = new LinkedList<SkeKapsel>();

		// Sortierte Map mit den zu sendenden Daten wird erstellt
		final SortedMap<String, String[]> sendMap = new TreeMap<String, String[]>();

		// Eintragen der Ereignisse und Berechnen der Anzahl
		for (final Map.Entry<String, SystemkalenderEintrag> me : SystemkalenderArbeiter
				.getSkeList().entrySet()) {

			final SystemkalenderEintrag ske = me.getValue();

			final SkeKapsel kapsel = new SkeKapsel(ske);
			listeKapseln.add(kapsel);

			final SortedMap<Long, Boolean> sm = ske
					.berecheneZustandsWechselVonBis(von, bis);

			Long wertA = null;
			Long wertB = null;
			final Iterator<Entry<Long, Boolean>> itEn = sm.entrySet()
					.iterator();
			while (itEn.hasNext()) {
				final Entry<Long, Boolean> en = itEn.next();

				if (wertA == null) {
					wertA = en.getKey();
				} else if (wertB == null) {
					wertB = en.getKey();
				}

				if ((wertA != null) && (wertB != null)) {
					/*
					 * Befindet sich das Ergebnis auch im geforderten
					 * Zeitbereich?
					 */
					if ((wertA >= von) && (wertB <= bis)) {
						kapsel.addBereich(wertA, wertB);
					}
					wertA = null;
					wertB = null;
				}
			}

			for (final Map.Entry<Long, Boolean> meAend : sm.entrySet()) {

				final Long time = meAend.getKey();

				final Boolean gueltig = meAend.getValue();

				final String[] sf = new String[2];

				sf[0] = me.getKey();

				if (gueltig) {
					sf[1] = "gltig";
				} else {
					sf[1] = "nicht gltig";
				}

				// Es wird ein eindeutiger Schluessel erzeugt
				final String uniqueKey = time + "_" + me.getKey();

				if ((time >= von) && (time <= bis)) {
					sendMap.put(uniqueKey, sf);
				}
			}

		}

		for (final Map.Entry<String, String[]> entry : sendMap.entrySet()) {

			final String tmp = entry.getKey();

			// Zeitinfo wird aus dem Schluessel extrahiert
			final String millis = tmp.substring(0, tmp.indexOf('_'));

			final Long l = Long.decode(millis);

			final Date d = new Date();

			d.setTime(l);
		}

		/*
		 * Prfen, ob sich in der Kapsel-Liste Eintrge ohne Bereich befinden.
		 * Diese Eintrge werden gelscht.
		 */
		final LinkedList<SkeKapsel> loeschListe = new LinkedList<SkeKapsel>();
		final Iterator<SkeKapsel> itLeereBereiche = listeKapseln.iterator();
		while (itLeereBereiche.hasNext()) {
			final SkeKapsel skeKapsel = itLeereBereiche.next();
			if (skeKapsel.getBereicheAnzahl() == 0) {
				loeschListe.add(skeKapsel);
			}
		}
		final Iterator<SkeKapsel> itLoeschen = loeschListe.iterator();
		while (itLoeschen.hasNext()) {
			final IEintragKapsel skeKapselLoeschen = itLoeschen.next();
			if (listeKapseln.contains(skeKapselLoeschen)) {
				listeKapseln.remove(skeKapselLoeschen);
			}
		}

		return listeKapseln;
	}

	/**
	 * ermittelt die Systemkalendereintrge fr den bergebenen Zeitbereich und
	 * eine definierte PID.
	 * 
	 * @param searchedPid
	 *            die PID
	 * @param von
	 *            Anfangszeitpunkt fr die Abfrage
	 * @param bis
	 *            Endzeitpunkt fr die Abfrage
	 * @return die Liste der ermittelten Eintrge
	 */
	public IEintragKapsel berechneGueltigVonBisS(final String searchedPid,
			final Long von, final Long bis) {

		final String pid = searchedPid.toLowerCase();

		// Sortierte Map mit den zu sendenden Daten wird erstellt
		final SortedMap<String, String[]> sendMap = new TreeMap<String, String[]>();

		/**
		 * Zurckzugebende Kapsel, beinhaltet die Daten des
		 * Systemkalendereintrags
		 */
		IEintragKapsel skeKapsel = null;

		if (SystemkalenderArbeiter.getSkeList().containsKey(pid)) {

			final Calendar cal1 = new GregorianCalendar();
			final Calendar cal2 = new GregorianCalendar();
			cal1.setTimeInMillis(von);
			cal2.setTimeInMillis(bis);

			final SystemkalenderEintrag ske = SystemkalenderArbeiter
					.getSkeList().get(pid);

			/*
			 * Kapsel Instanziieren
			 */
			skeKapsel = new SkeKapsel(ske);

			final SortedMap<Long, Boolean> sm = ske
					.berecheneZustandsWechselVonBis(von, bis);

			if (sm != null) {
				Long wertA = null;
				Long wertB = null;
				for (final Map.Entry<Long, Boolean> meAend : sm.entrySet()) {
					if (wertA == null) {
						wertA = meAend.getKey();
					} else if (wertB == null) {
						wertB = meAend.getKey();
					}

					if ((wertA != null) && (wertB != null)) {
						/*
						 * Befindet sich das Ergebnis auch im geforderten
						 * Zeitbereich?
						 */
						if ((wertA >= von) && (wertB <= bis)) {
							skeKapsel.addBereich(wertA, wertB);
						}
						wertA = null;
						wertB = null;
					}

					final Long time = meAend.getKey();

					final Boolean gueltig = meAend.getValue();

					final String[] sf = new String[2];

					// sf[0] = ske.getPid();
					sf[0] = pid;

					if (gueltig) {
						sf[1] = "gltig";
					} else {
						sf[1] = "noch gltig";
					}

					/**
					 * *****************************************************
					 * **********
					 */
					// Calendar cal = new GregorianCalendar();
					// cal.setTimeInMillis(time);
					// if ((cal.get(Calendar.MILLISECOND) == 999) &&
					// !meAend.getValue())
					// {
					// time += 1;
					// }
					/**
					 * *****************************************************
					 * **********
					 */

					// Es wird ein eindeutiger Schluessel erzeugt
					final String uniqueKey = time + "_" + ske.getPid();

					if ((time >= von) && (time <= bis)) {
						sendMap.put(uniqueKey, sf);
					}

				}

			}
		}

		for (final Map.Entry<String, String[]> entry : sendMap.entrySet()) {

			final String tmp = entry.getKey();

			// Zeitinfo wird aus dem Schluessel extrahiert
			final String millis = tmp.substring(0, tmp.indexOf('_'));

			final Long l = Long.decode(millis);

			final Date d = new Date();

			d.setTime(l);
		}

		return skeKapsel;
	}

	/**
	 * liefert die Klasse zur Auswertung der Systemkalenderdaten.
	 * 
	 * @return die Instanz
	 */
	public SystemkalenderArbeiter getSyskalArbeiter() {
		return syskalArbeiter;
	}

	/**
	 * setzt die Klasse zur Auswertung der Systemkalenderdaten.
	 * 
	 * @param arbeiter
	 *            die Instanz
	 */
	public void setSyskalArbeiter(final SystemkalenderArbeiter arbeiter) {
		syskalArbeiter = arbeiter;
	}

}
