/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.kalender.internal;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Informationen bei der Validierung von Zeiteingabe-Elementen?
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: ValidierungsInfo.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class ValidierungsInfo {

	private final List<FeldInfo> feldInfos = new LinkedList<FeldInfo>();
	private final List<String> nichtBestimmbareFelder;
	private String sonstigeInfos = "";

	/**
	 * Konstruktor.
	 * 
	 * @param feldInfos
	 *            die Liste der zu validierenden Felder
	 * @param nichtBestimmbareFelder
	 *            die Liste der nicht bestimmbaren Felder
	 */
	public ValidierungsInfo(final List<FeldInfo> feldInfos,
			final List<String> nichtBestimmbareFelder) {
		if (feldInfos != null) {
			this.feldInfos.addAll(feldInfos);
		}
		this.nichtBestimmbareFelder = nichtBestimmbareFelder;
	}

	/**
	 * liefert den Text fr die Felder.
	 * 
	 * @return den Text
	 */
	public String getFeldText() {
		final StringBuilder re = new StringBuilder();

		final Iterator<FeldInfo> it = feldInfos.iterator();
		while (it.hasNext()) {
			final FeldInfo feldInfo = it.next();

			if (re.length() > 0) {
				re.append(' ');
			}

			re.append(feldInfo.getFeldDarstellungsString());

		}

		return re.toString();
	}

	/**
	 * liefert die Liste der zu validierenden Felder.
	 * 
	 * @return die Liste
	 */
	public List<FeldInfo> getFeldInfos() {
		return feldInfos;
	}

	/**
	 * liefert die Liste der nicht bestimmbaren Felder.
	 * 
	 * @return die Liste
	 */
	public List<String> getNichtBestimmbareFelder() {
		return nichtBestimmbareFelder;
	}

	/**
	 * Liefert die sonstigen Informationen zu der Feldvalidierung.
	 * 
	 * @return Sonstige Infos zur Feldvalidierung
	 */
	public String getSonstigeInfos() {
		return sonstigeInfos;
	}

	/**
	 * Setzt die sonstigen Informationen zur Feldvalidierung.
	 * 
	 * @param sonstigeInfos
	 *            sonstige Informationen zur Feldvalidierung.
	 */
	public void setSonstigeInfos(final String sonstigeInfos) {
		this.sonstigeInfos = sonstigeInfos;
	}
}
