/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.internal;

/**
 * Liefert die Auswahl fr die verschiedenen Granularitten der Eingabe der
 * Zeitdauer.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: ZeitDauerGranularitaet.java 45393 2013-08-07 21:32:57Z peuker $
 */
public enum ZeitDauerGranularitaet {

	/** Erfassen aller ZeitdauerFelder. */
	jahr("Jahr", "Erfassen aller ZeitdauerFelder"),

	/** Erfassen aller ZeitdauerFelder ohne Jahre. */
	monat("Monat", "Erfassen aller ZeitdauerFelder ohne Jahre"),

	/** Erfassen aller ZeitdauerFelder ohne Jahre, Monate. */
	tag("Tag", "Erfassen aller ZeitdauerFelder ohne Jahre, Monate."),

	/** Erfassen aller ZeitdauerFelder ohne Jahre, Monate, Tage. */
	stunde("Stunde", "Erfassen aller ZeitdauerFelder ohne Jahre, Monate, Tage."),

	/** Erfassen aller ZeitdauerFelder ohne Jahre, Monate, Tage, Stunden. */
	minute("Minute",
			"Erfassen aller ZeitdauerFelder ohne Jahre, Monate, Tage, Stunden."),

	/**
	 * Erfassen aller ZeitdauerFelder ohne Jahre, Monate, Tage, Stunden,
	 * Minuten.
	 */
	sekunde("Sekunde",
			"Erfassen aller ZeitdauerFelder ohne Jahre, Monate, Tage, Stunden, Minuten."),

	/**
	 * Erfassen aller ZeitdauerFelder ohne Jahre, Monate, Tage, Stunden,
	 * Minuten, Sekunden.
	 */
	tausendstel(
			"Tausendstell",
			"Erfassen aller ZeitdauerFelder ohne Jahre, Monate, Tage, Stunden, Minuten, Sekunden.");

	/** Bezeichnung der Granularitt. */
	private String bezeichnung;

	/** Beschreibung der Granularitt. */
	private String beschreibung;

	private ZeitDauerGranularitaet(final String bezeichnung,
			final String beschreibung) {
		this.bezeichnung = bezeichnung;
		this.beschreibung = beschreibung;
	}

	/**
	 * Liefert die Bezeichnung der {@link ZeitDauerGranularitaet}.
	 * 
	 * @return Bezeichnung
	 */
	public String getBezeichnung() {
		return bezeichnung;
	}

	/**
	 * Liefert die Beschreibung der {@link ZeitDauerGranularitaet}.
	 * 
	 * @return Beschreibung
	 */
	public String getBeschreibung() {
		return beschreibung;
	}

	@Override
	public String toString() {
		return bezeichnung;
	}

	/**
	 * ermittelt, ob eine Granularitt in einer anderen enthalten ist.
	 * 
	 * @param other
	 *            die zu vergleichende Grnaularitt
	 * @return der Zustand
	 */
	public boolean istEnthaltenIn(final ZeitDauerGranularitaet other) {
		return this.ordinal() >= other.ordinal();
	}
}
