/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.internal;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Widget;

import de.bsvrz.buv.rw.basislib.kalender.CompositeZeitDauer;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Liefert das eigentliche Composite, das hinter den Zeitdialog steht. Bei den
 * Stunden, Minuten & Sekunden Elementen dieses Dialogs handelt es sich
 * letztendlich um Labels, die mit einem MouseListener ausgestattet wurden.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: ZeitDauerWaehler.java 49030 2014-02-28 15:19:54Z peuker $
 */
public class ZeitDauerWaehler extends Composite implements FocusListener {

	private final MouseListener mouseHandler = new MouseAdapter() {
		/**
		 * mouseDown
		 */
		@Override
		public void mouseDown(final MouseEvent event) {
			if (event.button == 1) {
				setFocus();

				// JahreKontrolle
				if (event.widget instanceof JahreKontrolle) {
					final int index = findeJahr(event.widget);

					verwalteJahre(index);

					// waehleJahr(index);
					waehleJahrLaufzeit(index);
				}

				// MonateKontrolle
				if (event.widget instanceof MonateKontrolle) {
					final int index = findeMonat(event.widget);

					verwalteMonate(index);

					// waehleMonat(index);
					waehleMonatLaufzeit(index);
				}

				// TageKontrolle
				if (event.widget instanceof TageKontrolle) {
					final int index = findeTag(event.widget);

					verwalteTage(index);

					// waehleTag(index);
					waehleTagLaufzeit(index);
				}

				// StundenKontrolle
				if (event.widget instanceof StundenKontrolle) {
					final int index = findeStunde(event.widget);

					verwalteStunden(index);

					// waehleStunde(index);
					waehleStundeLaufzeit(index);
				}

				// MinutenKontrolle
				if (event.widget instanceof MinutenKontrolle) {
					final int index = findeMinute(event.widget);

					verwalteMinuten(index);

					// waehleMinute(index);
					waehleMinuteLaufzeit(index);

				}

				// Sekundenkontrolle
				if (event.widget instanceof SekundenKontrolle) {
					final int index = findeSekunde(event.widget);

					verwalteSekunden(index);

					// waehleSekunde(index);
					waehleSekundeLaufzeit(index);
				}

			}
		}
	};

	/** Der Debugger. */
	private static final Debug LOGGER = Debug.getLogger();

	private Color aktiveAuswahlHintergrund;

	private Color inaktiveAuswahlHintergrund;

	private Color aktiveAuswahlVordergrund;

	private Color inaktiveAuswahlVordergrund;

	private Color textVordergrundFarbe;

	private Color textHintergrundFarbe;

	// Composites
	private Composite compositeJahreInfo;

	private Composite compositeJahre;

	private Composite compositeMonateInfo;

	private Composite compositeMonate;

	private Composite compositeTageInfo;

	private Composite compositeTage;

	private Composite compositeStundenInfo;

	private Composite compositeStunden;

	private Composite compositeMinutenInfo;

	private Composite compositeMinuten;

	// Elemente
	private JahreKontrolle[] jahreControls;

	private MonateKontrolle[] monateControls;

	private TageKontrolle[] tageControls;

	private StundenKontrolle[] stundenControls;

	private MinutenKontrolle[] minutenControls;

	private SekundenKontrolle[] sekundenControls;

	// Alte Selektionen
	private int altesSelektiertesJahr = -1;

	private int neuesSelektiertesJahr = -1;

	private int alterSelektierterMonat = -1;

	private int neuerSelektierterMonat = -1;

	private int alterSelektierterTag = -1;

	private int neuerSelektierterTag = -1;

	private int alteSelektierteStunde = -1;

	private int neueSelektierteStunde = -1;

	private int alteSelektierteMinute = -1;

	private int neueSelektierteMinute = -1;

	private int alteSelektierteSekunde = -1;

	private int neueSelektierteSekunde = -1;

	/**
	 * Beinhaltet die Datenstruktur der Darstellung.
	 */
	private final CompositeZeitDauer dataSource;

	/**
	 * Granularitt.
	 */
	private ZeitDauerGranularitaet granularitaet = ZeitDauerGranularitaet.jahr;

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *            der umgebende Container
	 * @param style
	 *            der Style fr das Widget
	 * @param dauer
	 *            das Zeitdauer-Composite aus dem der {@link ZeitDauerWaehler}
	 *            aufgerufen wurde
	 */
	public ZeitDauerWaehler(final Composite parent, final int style,
			final CompositeZeitDauer dauer) {
		super(parent, style);

		this.dataSource = dauer;
		this.granularitaet = dauer.getGranularitaet();

		erzeugeDarstellung(getParent());

		final int jahre = dataSource.getJahre();
		final int monate = dataSource.getMonate();
		final int tage = dataSource.getTage();
		final int stunden = dataSource.getStunden();
		final int minuten = dataSource.getMinuten();
		final int sekunden = dataSource.getSekunden();

		setzeZeit(jahre, monate, tage, stunden, minuten, sekunden);
		jahrZuruecksetzen(0);
		monatZuruecksetzen(0);
		tagZuruecksetzen(0);
		stundeZuruecksetzen(0);
		minuteZuruecksetzen(0);
		sekundeZuruecksetzen(0);

		initKalender();

	}

	private void erzeugeDarstellung(final Composite parent) {

		setBackground(getDisplay().getSystemColor(SWT.COLOR_WHITE));

		aktiveAuswahlHintergrund = getDisplay().getSystemColor(
				SWT.COLOR_LIST_SELECTION);
		inaktiveAuswahlHintergrund = getDisplay()
				.getSystemColor(SWT.COLOR_GRAY);
		aktiveAuswahlVordergrund = getDisplay().getSystemColor(
				SWT.COLOR_LIST_SELECTION_TEXT);
		inaktiveAuswahlVordergrund = getForeground();

		textVordergrundFarbe = getDisplay().getSystemColor(
				SWT.COLOR_LIST_FOREGROUND);
		textHintergrundFarbe = getDisplay().getSystemColor(
				SWT.COLOR_LIST_BACKGROUND);

		// 3 Zeilen fr die Darstellung der Composites von Stunden, Minuten und
		// Sekunden
		final GridLayout gridLayout = new GridLayout();
		gridLayout.makeColumnsEqualWidth = true;
		gridLayout.numColumns = 1;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		gridLayout.horizontalSpacing = 0;
		gridLayout.verticalSpacing = 0;
		setLayout(gridLayout);

		// ////
		// Composites definieren

		// composite JahreInfo
		compositeJahreInfo = new Composite(this, SWT.NONE);
		final Label separatorJahreInfo = new Label(compositeJahreInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		separatorJahreInfo.setLayoutData(gridData);
		final Label labelJahre = new Label(compositeJahreInfo, SWT.NONE);
		labelJahre.setText("Jahre");
		compositeJahreInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// composite Jahre
		compositeJahre = new Composite(this, SWT.NONE);
		compositeJahre.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeMonateInfo
		compositeMonateInfo = new Composite(this, SWT.NONE);
		final Label separatorMonateInfo = new Label(compositeMonateInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		// final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		separatorMonateInfo.setLayoutData(gridData);
		final Label labelMonate = new Label(compositeMonateInfo, SWT.NONE);
		labelMonate.setText("Monate");
		compositeMonateInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeMonate
		compositeMonate = new Composite(this, SWT.NONE);
		compositeMonate.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeTageInfo
		compositeTageInfo = new Composite(this, SWT.NONE);
		final Label separatorTageInfo = new Label(compositeTageInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		// final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		separatorTageInfo.setLayoutData(gridData);
		final Label labelTage = new Label(compositeTageInfo, SWT.NONE);
		labelTage.setText("Tage");
		compositeTageInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeTage
		compositeTage = new Composite(this, SWT.NONE);
		compositeTage.setBackground(getDisplay()
				.getSystemColor(SWT.COLOR_WHITE));

		// compositeStundenInfo
		compositeStundenInfo = new Composite(this, SWT.NONE);
		final Label separatorStundenInfo = new Label(compositeStundenInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		// final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		separatorStundenInfo.setLayoutData(gridData);
		final Label labelStunden = new Label(compositeStundenInfo, SWT.NONE);
		labelStunden.setText("Stunden");
		compositeStundenInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeStunden
		compositeStunden = new Composite(this, SWT.NONE);
		compositeStunden.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeMinutenInfo
		compositeMinutenInfo = new Composite(this, SWT.NONE);
		final Label separatorMinutenInfo = new Label(compositeMinutenInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		// final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		separatorMinutenInfo.setLayoutData(gridData);
		final Label labelMinuten = new Label(compositeMinutenInfo, SWT.NONE);
		labelMinuten.setText("Minuten");
		compositeMinutenInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeMinuten
		compositeMinuten = new Composite(this, SWT.NONE);
		compositeMinuten.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeSekundeniNFO
		final Composite compositeSekundenInfo = new Composite(this, SWT.NONE);
		final Label separatorSekundenInfo = new Label(compositeSekundenInfo,
				SWT.HORIZONTAL | SWT.SEPARATOR);
		// final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 7;
		separatorSekundenInfo.setLayoutData(gridData);
		final Label labelSekunden = new Label(compositeSekundenInfo, SWT.NONE);
		labelSekunden.setText("Sekunden");
		compositeSekundenInfo.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// compositeSekunden
		final Composite compositeSekunden = new Composite(this, SWT.NONE);
		compositeSekunden.setBackground(getDisplay().getSystemColor(
				SWT.COLOR_WHITE));

		// 1 Spalte fr JahrInfo
		final GridData gridDataJahreInfo = new GridData(
				GridData.FILL_HORIZONTAL);
		compositeJahreInfo.setLayoutData(gridDataJahreInfo);
		GridLayout gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 1;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeJahreInfo.setLayout(gridLayoutTopHeader);

		// 10 Spalten fr die Jahre
		final GridData gridDataJahre = new GridData(GridData.FILL_HORIZONTAL);
		compositeJahre.setLayoutData(gridDataJahre);
		gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 10;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeJahre.setLayout(gridLayoutTopHeader);

		// 1 Spalte fr MonateInfo
		final GridData gridDataMonateInfo = new GridData(
				GridData.FILL_HORIZONTAL);
		compositeMonateInfo.setLayoutData(gridDataMonateInfo);
		gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 1;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeMonateInfo.setLayout(gridLayoutTopHeader);

		// 12 Spalten fr die Monate
		final GridData gridDataMonate = new GridData(GridData.FILL_HORIZONTAL);
		compositeMonate.setLayoutData(gridDataMonate);
		gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 10;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeMonate.setLayout(gridLayoutTopHeader);

		// 1 Spalte fr TageInfo

		final GridData gridDataTageInfo = new GridData(GridData.FILL_HORIZONTAL);
		compositeTageInfo.setLayoutData(gridDataTageInfo);
		gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 1;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeTageInfo.setLayout(gridLayoutTopHeader);

		// 10 Spalten fr die Tage

		final GridData gridDataTage = new GridData(GridData.FILL_HORIZONTAL);
		compositeTage.setLayoutData(gridDataTage);
		gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 10;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeTage.setLayout(gridLayoutTopHeader);

		// 1 Spalte fr StundenInfo

		final GridData gridDataStundenInfo = new GridData(
				GridData.FILL_HORIZONTAL);
		compositeStundenInfo.setLayoutData(gridDataStundenInfo);
		gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 1;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeStundenInfo.setLayout(gridLayoutTopHeader);

		// 10 Spalten fr die Stunden

		final GridData gridDataStunden = new GridData(GridData.FILL_HORIZONTAL);
		compositeStunden.setLayoutData(gridDataStunden);
		gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 10;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeStunden.setLayout(gridLayoutTopHeader);

		// 2 Spalten fr die Minuteninfo

		final GridData gridDataMinutenInfo = new GridData(
				GridData.FILL_HORIZONTAL);
		compositeMinutenInfo.setLayoutData(gridDataMinutenInfo);
		gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 1;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeMinutenInfo.setLayout(gridLayoutTopHeader);

		// 10 Spalten fr die Minuten zu 6 Zeilen

		final GridData gridDataMinuten = new GridData(GridData.FILL_HORIZONTAL);
		compositeMinuten.setLayoutData(gridDataMinuten);
		gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 10;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeMinuten.setLayout(gridLayoutTopHeader);

		// 2 Spalten fr die SekundenInfo
		final GridData gridDataSekundenInfo = new GridData(
				GridData.FILL_HORIZONTAL);
		compositeSekundenInfo.setLayoutData(gridDataSekundenInfo);
		gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 1;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeSekundenInfo.setLayout(gridLayoutTopHeader);

		// 10 Spalten fr die Sekunden zu 6 Zeilen

		final GridData gridDataSekunden = new GridData(GridData.FILL_HORIZONTAL);
		compositeSekunden.setLayoutData(gridDataSekunden);
		gridLayoutTopHeader = new GridLayout();
		gridLayoutTopHeader.numColumns = 10;
		gridLayoutTopHeader.marginWidth = 0;
		gridLayoutTopHeader.marginHeight = 0;
		compositeSekunden.setLayout(gridLayoutTopHeader);

		// Jahrekontrolle anlegen
		jahreControls = new JahreKontrolle[20];
		for (int i = 0; i < jahreControls.length; i++) {
			final JahreKontrolle jahr = new JahreKontrolle(compositeJahre);
			jahreControls[i] = jahr;
			jahr.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			jahr.addMouseListener(mouseHandler);
		}

		// Monatekontrolle anlegen
		monateControls = new MonateKontrolle[13];
		for (int i = 0; i < monateControls.length; i++) {
			final MonateKontrolle monat = new MonateKontrolle(compositeMonate);
			monateControls[i] = monat;
			monat.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			monat.addMouseListener(mouseHandler);
		}

		// Tagekontrolle anlegen
		tageControls = new TageKontrolle[32];
		for (int i = 0; i < tageControls.length; i++) {
			final TageKontrolle tag = new TageKontrolle(compositeTage);
			tageControls[i] = tag;
			tag.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			tag.addMouseListener(mouseHandler);
		}

		// Stundenkontrolle anlegen
		stundenControls = new StundenKontrolle[24];
		for (int i = 0; i < stundenControls.length; i++) {
			final StundenKontrolle stunde = new StundenKontrolle(
					compositeStunden);
			stundenControls[i] = stunde;
			stunde.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			stunde.addMouseListener(mouseHandler);
		}

		// Minutenkontrolle anlegen
		minutenControls = new MinutenKontrolle[60];
		for (int i = 0; i < minutenControls.length; i++) {
			final MinutenKontrolle minute = new MinutenKontrolle(
					compositeMinuten);
			minutenControls[i] = minute;
			minute.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			minute.addMouseListener(mouseHandler);
		}

		// Sekundenkontrolle anlegen
		sekundenControls = new SekundenKontrolle[60];
		for (int i = 0; i < sekundenControls.length; i++) {
			final SekundenKontrolle sekunde = new SekundenKontrolle(
					compositeSekunden);
			sekundenControls[i] = sekunde;
			sekunde.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL
					| GridData.VERTICAL_ALIGN_FILL));
			sekunde.addMouseListener(mouseHandler);
		}

		setFont(parent.getFont());
		init();

		addMouseListener(mouseHandler);
		addFocusListener(this);

	}

	/** Initialisiert die verwendeten Kalender. */
	private void initKalender() {
		verwalteJahre(dataSource.getJahre());
		waehleJahrLaufzeit(dataSource.getJahre());
		verwalteMonate(dataSource.getMonate());
		waehleMonatLaufzeit(dataSource.getMonate());
		verwalteTage(dataSource.getTage());
		waehleTagLaufzeit(dataSource.getTage());
		verwalteStunden(dataSource.getStunden());
		waehleStundeLaufzeit(dataSource.getStunden());
		verwalteMinuten(dataSource.getMinuten());
		waehleMinuteLaufzeit(dataSource.getMinuten());
		verwalteSekunden(dataSource.getSekunden());
		waehleSekundeLaufzeit(dataSource.getSekunden());
	}

	void passeDarstellungAnGranularitaetAn() {

		compositeMinutenInfo.setVisible(ZeitDauerGranularitaet.minute
				.istEnthaltenIn(granularitaet));
		compositeMinuten.setVisible(ZeitDauerGranularitaet.minute
				.istEnthaltenIn(granularitaet));

		compositeStundenInfo.setVisible(ZeitDauerGranularitaet.stunde
				.istEnthaltenIn(granularitaet));
		compositeStunden.setVisible(ZeitDauerGranularitaet.stunde
				.istEnthaltenIn(granularitaet));

		compositeTageInfo.setVisible(ZeitDauerGranularitaet.tag
				.istEnthaltenIn(granularitaet));
		compositeTage.setVisible(ZeitDauerGranularitaet.tag
				.istEnthaltenIn(granularitaet));

		compositeMonateInfo.setVisible(ZeitDauerGranularitaet.monat
				.istEnthaltenIn(granularitaet));
		compositeMonate.setVisible(ZeitDauerGranularitaet.monat
				.istEnthaltenIn(granularitaet));

		compositeJahreInfo.setVisible(ZeitDauerGranularitaet.jahr
				.istEnthaltenIn(granularitaet));
		compositeJahre.setVisible(ZeitDauerGranularitaet.jahr
				.istEnthaltenIn(granularitaet));
	}

	private void init() {
		zeichneJahre();
		zeichneMonate();
		zeichneTage();
		zeichneStunden();
		zeichneMinuten();
		zeichneSekunden();
	}

	/**
	 * Zeichnet die Jahre aus dem Array <code>jahre</code>.
	 * 
	 */
	private void zeichneJahre() {
		for (int i = 0; i < jahreControls.length; i++) {
			final JahreKontrolle jahrKontrolle = jahreControls[i];
			jahrKontrolle.setText(String.valueOf(i));
		}
	}

	/**
	 * Zeichnet die Monate aus dem Array <code>monate</code>.
	 * 
	 */
	private void zeichneMonate() {
		for (int i = 0; i < monateControls.length; i++) {
			final MonateKontrolle monatKontrolle = monateControls[i];
			monatKontrolle.setText(String.valueOf(i));
		}
	}

	/**
	 * Zeichnet die Tage aus dem Array <code>tage</code>.
	 * 
	 */
	private void zeichneTage() {
		for (int i = 0; i < tageControls.length; i++) {
			final TageKontrolle tagKontrolle = tageControls[i];
			tagKontrolle.setText(String.valueOf(i));
		}
	}

	/**
	 * Zeichnet die Stunden aus dem Array <code>stunden</code>.
	 * 
	 */
	private void zeichneStunden() {
		for (int i = 0; i < stundenControls.length; i++) {
			final StundenKontrolle stuKontrolle = stundenControls[i];
			stuKontrolle.setText(String.valueOf(i));
		}
	}

	/**
	 * Zeichnet die Minuten aus dem Array <code>minuten</code>.
	 * 
	 */
	private void zeichneMinuten() {
		for (int i = 0; i < minutenControls.length; i++) {
			final MinutenKontrolle minKontrolle = minutenControls[i];
			minKontrolle.setText(String.valueOf(i));
		}
	}

	/**
	 * Zeichnet die Sekunden aus dem Array <code>sekunden</code>.
	 * 
	 */
	private void zeichneSekunden() {
		for (int i = 0; i < sekundenControls.length; i++) {
			final SekundenKontrolle sekKontrolle = sekundenControls[i];
			sekKontrolle.setText(String.valueOf(i));
		}
	}

	@Override
	public void setFont(final Font font) {
		super.setFont(font);

		// Jahre
		for (final JahreKontrolle jahreKontrolle : jahreControls) {
			jahreKontrolle.setFont(font);
		}

		// Monate
		for (final MonateKontrolle monateKontrolle : monateControls) {
			monateKontrolle.setFont(font);
		}

		// Tage
		for (final TageKontrolle tageKontrolle : tageControls) {
			tageKontrolle.setFont(font);
		}

		// Stunden
		for (final StundenKontrolle element : stundenControls) {
			element.setFont(font);
		}

		// Minuten
		for (final MinutenKontrolle minutenKontrolle : minutenControls) {
			minutenKontrolle.setFont(font);
		}

		// Sekunden
		for (final SekundenKontrolle sekundenKontrolle : sekundenControls) {
			sekundenKontrolle.setFont(font);
		}

	}

	@Override
	public void focusGained(final FocusEvent event) {
		final int index = findeStunde(event.widget);
		stundenControls[index].setBackground(getSelectionBackgroundColor());
		stundenControls[index].setForeground(getSelectionForegroundColor());
	}

	@Override
	public void focusLost(final FocusEvent event) {
		final int index = findeStunde(event.widget);
		stundenControls[index].setBackground(getSelectionBackgroundColor());
		stundenControls[index].setForeground(getSelectionForegroundColor());
	}

	/**
	 * Setzt die Zeit im Kalender. Dabei wird <code>calendar</code> und die
	 * grafische Darstellung aktuallisiert.
	 * 
	 * @param jahre
	 *            Zu setzende Jahre
	 * @param monate
	 *            Zu setzende Monate
	 * @param tage
	 *            Zu setzende Tage
	 * @param stunde
	 *            Zu setzende Stunde
	 * @param minute
	 *            Zu setzende Minute
	 * @param sekunde
	 *            Zu setzemde Sekunde
	 */
	public void setzeZeit(final int jahre, final int monate, final int tage,
			final int stunde, final int minute, final int sekunde) {
		verwalteJahre(jahre);
		verwalteMonate(monate);
		verwalteTage(tage);
		verwalteStunden(stunde);
		verwalteMinuten(minute);
		verwalteSekunden(sekunde);

		waehleJahr(jahre);
		waehleMonat(monate);
		waehleTag(tage);
		waehleStunde(stunde);
		waehleMinute(minute);
		waehleSekunde(sekunde);

	}

	/**
	 * Setzt in den internen Variablen <code>altesSelektiertesjtunde</code> und
	 * <code>neueSelektiertesJahr</code> die entsprechenden Werte.
	 * 
	 * @param index
	 *            Index des neuen Jahres
	 */
	private void verwalteJahre(final int index) {
		if ((altesSelektiertesJahr == -1) && (neuesSelektiertesJahr == -1)) {
			neuesSelektiertesJahr = index;
		} else {
			altesSelektiertesJahr = neuesSelektiertesJahr;
			neuesSelektiertesJahr = index;
			jahrZuruecksetzen(altesSelektiertesJahr);
		}

		if (dataSource != null) {
			dataSource.setJahre(index);
		}
	}

	/**
	 * Setzt in den internen Variablen <code>alterSelektierterMonat</code> und
	 * <code>neuerSelektierterMonat</code> die entsprechenden Werte.
	 * 
	 * @param index
	 *            Index des neuen Monats
	 */
	private void verwalteMonate(final int index) {
		if ((alterSelektierterMonat == -1) && (neuerSelektierterMonat == -1)) {
			neuerSelektierterMonat = index;
		} else {
			alterSelektierterMonat = neuerSelektierterMonat;
			neuerSelektierterMonat = index;
			monatZuruecksetzen(alterSelektierterMonat);
		}

		if (dataSource != null) {
			dataSource.setMonate(index);
		}
	}

	/**
	 * Setzt in den internen Variablen <code>alterSelektierterTag</code> und
	 * <code>neuerSelektierterTag</code> die entsprechenden Werte.
	 * 
	 * @param index
	 *            Index des neuen Tags
	 */
	private void verwalteTage(final int index) {
		if ((alterSelektierterTag == -1) && (neuerSelektierterTag == -1)) {
			neuerSelektierterTag = index;
		} else {
			alterSelektierterTag = neuerSelektierterTag;
			neuerSelektierterTag = index;
			tagZuruecksetzen(alterSelektierterTag);
		}

		if (dataSource != null) {
			dataSource.setTage(index);
		}
	}

	/**
	 * Setzt in den internen Variablen <code>alteSelektierteStunde</code> und
	 * <code>neueSelektierteStunde</code> die entsprechenden Werte.
	 * 
	 * @param index
	 *            Index der neuen Stunde
	 */
	private void verwalteStunden(final int index) {
		if ((alteSelektierteStunde == -1) && (neueSelektierteStunde == -1)) {
			neueSelektierteStunde = index;
		} else {
			alteSelektierteStunde = neueSelektierteStunde;
			neueSelektierteStunde = index;
			stundeZuruecksetzen(alteSelektierteStunde);
		}

		if (dataSource != null) {
			dataSource.setStunden(index);
		}
	}

	/**
	 * Setzt in den internen Variablen <code>alteSelektierteMinute</code> und
	 * <code>neueSelektierteMinute</code> die entsprechenden Werte.
	 * 
	 * @param index
	 *            Index der neuen Minute
	 */
	private void verwalteMinuten(final int index) {
		if ((alteSelektierteMinute == -1) && (neueSelektierteMinute == -1)) {
			neueSelektierteMinute = index;
		} else {
			alteSelektierteMinute = neueSelektierteMinute;
			neueSelektierteMinute = index;
			minuteZuruecksetzen(alteSelektierteMinute);
		}

		if (dataSource != null) {
			dataSource.setMinuten(index);
		}
	}

	/**
	 * Setzt in den internen Variablen <code>alteSelektierteSekunde</code> und
	 * <code>neueSelektierteSekunde</code> die entsprechenden Werte.
	 * 
	 * @param index
	 *            Index der neuen Sekunde
	 */
	private void verwalteSekunden(final int index) {
		if ((alteSelektierteSekunde == -1) && (neueSelektierteSekunde == -1)) {
			neueSelektierteSekunde = index;
		} else {
			alteSelektierteSekunde = neueSelektierteSekunde;
			neueSelektierteSekunde = index;
			sekundeZuruecksetzen(alteSelektierteSekunde);
		}

		if (dataSource != null) {
			dataSource.setSekunden(index);
		}
	}

	/**
	 * Findet die Position eines Controls im <code>jahre</code> Array.
	 * 
	 * @param jahrKontrolle
	 *            Das Widget, dessen zugehriges Jahrelement gefunden werden
	 *            soll
	 * @return Index des gefundenen Elements
	 */
	private int findeJahr(final Widget jahrKontrolle) {
		for (int i = 0; i < jahreControls.length; i++) {
			if (jahreControls[i] == jahrKontrolle) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Findet die Position eines Controls im <code>monate</code> Array.
	 * 
	 * @param monatKontrolle
	 *            Das Widget, dessen zugehriges Monatelement gefunden werden
	 *            soll
	 * @return Index des gefundenen Elements
	 */
	private int findeMonat(final Widget monatKontrolle) {
		for (int i = 0; i < monateControls.length; i++) {
			if (monateControls[i] == monatKontrolle) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Findet die Position eines Controls im <code>tage</code> Array.
	 * 
	 * @param tagKontrolle
	 *            Das Widget, dessen zugehriges Tagelement gefunden werden soll
	 * @return Index des gefundenen Elements
	 */
	private int findeTag(final Widget tagKontrolle) {
		for (int i = 0; i < tageControls.length; i++) {
			if (tageControls[i] == tagKontrolle) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Findet die Position eines Controls im <code>stunden</code> Array.
	 * 
	 * @param stuKontrolle
	 *            Das Widget, dessen zugehriges Stundenelement gefunden werden
	 *            soll
	 * @return Index des gefundenen Elements
	 */
	private int findeStunde(final Widget stuKontrolle) {
		for (int i = 0; i < stundenControls.length; i++) {
			if (stundenControls[i] == stuKontrolle) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Findet die Position eines Controls im <code>minuten</code> Array.
	 * 
	 * @param minKontrolle
	 *            Das Widget, dessen zugehriges Minutenelement gefunden werden
	 *            soll
	 * @return Index des gefundenen Elements
	 */
	private int findeMinute(final Widget minKontrolle) {
		for (int i = 0; i < minutenControls.length; i++) {
			if (minutenControls[i] == minKontrolle) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Findet die Position eines Controls im <code>sekunden</code> Array.
	 * 
	 * @param sekKontrolle
	 *            Das Widget, dessen zugehriges Sekundenelement gefunden werden
	 *            soll
	 * @return Index des gefundenen Elements
	 */
	private int findeSekunde(final Widget sekKontrolle) {
		for (int i = 0; i < sekundenControls.length; i++) {
			if (sekundenControls[i] == sekKontrolle) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Whlt das Jahr aus und markiert es.
	 * 
	 * @param iJahr
	 *            Index des Jahres
	 */
	private void waehleJahr(final int iJahr) {
		final JahreKontrolle gewaehltesJahr = jahreControls[iJahr];

		gewaehltesJahr.setBackground(getSelectionBackgroundColor());
		gewaehltesJahr.setForeground(getSelectionForegroundColor());

	}

	/**
	 * Whlt den Monat aus und markiert ihn.
	 * 
	 * @param iMonat
	 *            Index des Monats
	 */
	private void waehleMonat(final int iMonat) {
		final MonateKontrolle gewaehlterMonat = monateControls[iMonat];

		gewaehlterMonat.setBackground(getSelectionBackgroundColor());
		gewaehlterMonat.setForeground(getSelectionForegroundColor());

	}

	/**
	 * Whlt den Tag aus und markiert ihn.
	 * 
	 * @param iTag
	 *            Index des Tags
	 */
	private void waehleTag(final int iTag) {
		final TageKontrolle gewaehlterTag = tageControls[iTag];

		gewaehlterTag.setBackground(getSelectionBackgroundColor());
		gewaehlterTag.setForeground(getSelectionForegroundColor());

	}

	/**
	 * Whlt die Stunde aus und markiert sie.
	 * 
	 * @param iStunde
	 *            Index der Stunde
	 */
	private void waehleStunde(final int iStunde) {
		final StundenKontrolle gewaehlteStunde = stundenControls[iStunde];

		gewaehlteStunde.setBackground(getSelectionBackgroundColor());
		gewaehlteStunde.setForeground(getSelectionForegroundColor());

	}

	/**
	 * Whlt die Minute aus und markiert sie.
	 * 
	 * @param iMinute
	 *            Index der Minute
	 */
	private void waehleMinute(final int iMinute) {
		final MinutenKontrolle gewaehlteMinute = minutenControls[iMinute];
		gewaehlteMinute.setBackground(getSelectionBackgroundColor());
		gewaehlteMinute.setForeground(getSelectionForegroundColor());

	}

	/**
	 * Whlt die Sekunde aus und markiert sie.
	 * 
	 * @param iSekunde
	 *            Index der Sekunde
	 */
	private void waehleSekunde(final int iSekunde) {
		final SekundenKontrolle gewaehlteSekunde = sekundenControls[iSekunde];
		gewaehlteSekunde.setBackground(getSelectionBackgroundColor());
		gewaehlteSekunde.setForeground(getSelectionForegroundColor());

	}

	private void waehleJahrLaufzeit(final int iJahr) {
		final JahreKontrolle gewaehltesJahr = jahreControls[iJahr];

		gewaehltesJahr.setBackground(getSelectionBackgroundColor());
		gewaehltesJahr.setForeground(getSelectionForegroundColor());

	}

	private void waehleMonatLaufzeit(final int iMonat) {
		final MonateKontrolle gewaehlterMonat = monateControls[iMonat];

		gewaehlterMonat.setBackground(getSelectionBackgroundColor());
		gewaehlterMonat.setForeground(getSelectionForegroundColor());

	}

	private void waehleTagLaufzeit(final int iTag) {
		final TageKontrolle gewaehlterTag = tageControls[iTag];

		gewaehlterTag.setBackground(getSelectionBackgroundColor());
		gewaehlterTag.setForeground(getSelectionForegroundColor());

	}

	private void waehleStundeLaufzeit(final int iStunde) {
		final StundenKontrolle gewaehlteStunde = stundenControls[iStunde];

		gewaehlteStunde.setBackground(getSelectionBackgroundColor());
		gewaehlteStunde.setForeground(getSelectionForegroundColor());

	}

	private void waehleMinuteLaufzeit(final int iMinute) {
		final MinutenKontrolle gewaehlteMinute = minutenControls[iMinute];
		gewaehlteMinute.setBackground(getSelectionBackgroundColor());
		gewaehlteMinute.setForeground(getSelectionForegroundColor());

	}

	private void waehleSekundeLaufzeit(final int iSekunde) {
		final SekundenKontrolle gewaehlteSekunde = sekundenControls[iSekunde];
		gewaehlteSekunde.setBackground(getSelectionBackgroundColor());
		gewaehlteSekunde.setForeground(getSelectionForegroundColor());

	}

	private Color getSelectionBackgroundColor() {
		return isFocusControl() ? aktiveAuswahlHintergrund
				: inaktiveAuswahlHintergrund;
	}

	private Color getSelectionForegroundColor() {
		return isFocusControl() ? aktiveAuswahlVordergrund
				: inaktiveAuswahlVordergrund;
	}

	@Override
	public boolean isFocusControl() {
		for (Control control = getDisplay().getFocusControl(); control != null; control = control
				.getParent()) {
			if (control == this) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Setzt ein demarkiertes Jahr in den Ursprungszustand zurck.
	 * 
	 * @param index
	 *            Index des demarkierten Jahres
	 */
	private void jahrZuruecksetzen(final int index) {
		final JahreKontrolle altesGewaehltesJahr = jahreControls[index];
		altesGewaehltesJahr.setBackground(textHintergrundFarbe);
		altesGewaehltesJahr.setForeground(textVordergrundFarbe);
	}

	/**
	 * Setzt einen demarkierten Monat in seinen Ursprungszustand zurck.
	 * 
	 * @param index
	 *            Index des demarkierten Monats
	 */
	private void monatZuruecksetzen(final int index) {
		final MonateKontrolle alterGewaehlterMonat = monateControls[index];
		alterGewaehlterMonat.setBackground(textHintergrundFarbe);
		alterGewaehlterMonat.setForeground(textVordergrundFarbe);
	}

	/**
	 * Setzt einen demarkierten Tag in seinen Ursprungszustand zurck.
	 * 
	 * @param index
	 *            Index des demarkierten Tags
	 */
	private void tagZuruecksetzen(final int index) {
		final TageKontrolle alterGewaehlterTag = tageControls[index];
		alterGewaehlterTag.setBackground(textHintergrundFarbe);
		alterGewaehlterTag.setForeground(textVordergrundFarbe);
	}

	/**
	 * Setzt eine demarkierte Stunde in ihren Ursprungszustand zurck.
	 * 
	 * @param index
	 *            Index der demarkierten Stunde
	 */
	private void stundeZuruecksetzen(final int index) {
		final StundenKontrolle alteGewaehlteStunde = stundenControls[index];
		alteGewaehlteStunde.setBackground(textHintergrundFarbe);
		alteGewaehlteStunde.setForeground(textVordergrundFarbe);
	}

	/**
	 * Setzt eine demarkierte Minute in ihren Ursprungszustand zurck.
	 * 
	 * @param index
	 *            Index der demarkierten Minute
	 */
	private void minuteZuruecksetzen(final int index) {
		final MinutenKontrolle alteGewaehlteMinute = minutenControls[index];
		alteGewaehlteMinute.setBackground(textHintergrundFarbe);
		alteGewaehlteMinute.setForeground(textVordergrundFarbe);
	}

	/**
	 * Setzt eine demarkierte Sekunde in ihren Ursprungszustand zurck.
	 * 
	 * @param index
	 *            Index der demarkierten Sekunde
	 */
	private void sekundeZuruecksetzen(final int index) {
		final SekundenKontrolle alteGewaehlteSekunde = sekundenControls[index];
		alteGewaehlteSekunde.setBackground(textHintergrundFarbe);
		alteGewaehlteSekunde.setForeground(textVordergrundFarbe);
	}

	/**
	 * Jahrekontrolle.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class JahreKontrolle extends Composite implements Listener {
		private final Composite fueller;

		private final Label label;

		/**
		 * Stundenkontrolle.
		 * 
		 * @param parent
		 *            Composite, in dem das Element gezeichnet werden soll
		 */
		public JahreKontrolle(final Composite parent) {
			super(parent, SWT.NO_FOCUS);

			GridLayout gridLayout = new GridLayout();
			gridLayout.marginWidth = 1;
			gridLayout.marginHeight = 1;
			setLayout(gridLayout);

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));

			gridLayout = new GridLayout();
			gridLayout.marginWidth = 2;
			gridLayout.marginHeight = 0;
			fueller.setLayout(gridLayout);

			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(final String text) {
			label.setText(text);
		}

		@Override
		public void setFont(final Font font) {
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		@Override
		public void setBackground(final Color color) {
			fueller.setBackground(color);
			label.setBackground(color);
		}

		@Override
		public void setForeground(final Color color) {
			label.setForeground(color);
		}

		public void setRahmenFarbe(final Color color) {
			super.setBackground(color);
		}

		@Override
		public void handleEvent(final Event event) {
			notifyListeners(event.type, event);
		}
	}

	/**
	 * Monatekontrolle.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class MonateKontrolle extends Composite implements Listener {

		private final Composite fueller;

		// private boolean debug = false;

		private final Label label;

		/**
		 * Stundenkontrolle.
		 * 
		 * @param parent
		 *            Composite, in dem das Element gezeichnet werden soll
		 */
		public MonateKontrolle(final Composite parent) {
			super(parent, SWT.NO_FOCUS);

			GridLayout gridLayout = new GridLayout();
			gridLayout.marginWidth = 1;
			gridLayout.marginHeight = 1;
			setLayout(gridLayout);

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));

			gridLayout = new GridLayout();
			gridLayout.marginWidth = 2;
			gridLayout.marginHeight = 0;
			fueller.setLayout(gridLayout);

			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(final String text) {
			label.setText(text);
		}

		@Override
		public void setFont(final Font font) {
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		@Override
		public void setBackground(final Color color) {
			fueller.setBackground(color);
			label.setBackground(color);
		}

		@Override
		public void setForeground(final Color color) {
			label.setForeground(color);
		}

		public void setRahmenFarbe(final Color color) {
			super.setBackground(color);
		}

		@Override
		public void handleEvent(final Event event) {
			notifyListeners(event.type, event);
		}
	}

	/**
	 * Tagekontrolle.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class TageKontrolle extends Composite implements Listener {
		private final Composite fueller;

		private final Label label;

		/**
		 * Stundenkontrolle.
		 * 
		 * @param parent
		 *            Composite, in dem das Element gezeichnet werden soll
		 */
		public TageKontrolle(final Composite parent) {
			super(parent, SWT.NO_FOCUS);

			GridLayout gridLayout = new GridLayout();
			gridLayout.marginWidth = 1;
			gridLayout.marginHeight = 1;
			setLayout(gridLayout);

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));

			gridLayout = new GridLayout();
			gridLayout.marginWidth = 2;
			gridLayout.marginHeight = 0;
			fueller.setLayout(gridLayout);

			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(final String text) {
			label.setText(text);
		}

		@Override
		public void setFont(final Font font) {
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		@Override
		public void setBackground(final Color color) {
			fueller.setBackground(color);
			label.setBackground(color);
		}

		@Override
		public void setForeground(final Color color) {
			label.setForeground(color);
		}

		public void setRahmenFarbe(final Color color) {
			super.setBackground(color);
		}

		@Override
		public void handleEvent(final Event event) {
			notifyListeners(event.type, event);
		}
	}

	/**
	 * Stundenkontrolle.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class StundenKontrolle extends Composite implements Listener {
		private final Composite fueller;

		private final Label label;

		/**
		 * Stundenkontrolle.
		 * 
		 * @param parent
		 *            Composite, in dem das Element gezeichnet werden soll
		 */
		public StundenKontrolle(final Composite parent) {
			super(parent, SWT.NO_FOCUS);

			GridLayout gridLayout = new GridLayout();
			gridLayout.marginWidth = 1;
			gridLayout.marginHeight = 1;
			setLayout(gridLayout);

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));

			gridLayout = new GridLayout();
			gridLayout.marginWidth = 2;
			gridLayout.marginHeight = 0;
			fueller.setLayout(gridLayout);

			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(final String text) {
			label.setText(text);
		}

		@Override
		public void setFont(final Font font) {
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		@Override
		public void setBackground(final Color color) {
			fueller.setBackground(color);
			label.setBackground(color);
		}

		@Override
		public void setForeground(final Color color) {
			label.setForeground(color);
		}

		public void setRahmenFarbe(final Color color) {
			super.setBackground(color);
		}

		@Override
		public void handleEvent(final Event event) {
			notifyListeners(event.type, event);
		}
	}

	/**
	 * Zustndig fr die Minuten.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class MinutenKontrolle extends Composite implements Listener {
		private final Composite fueller;

		private final Label label;

		/**
		 * Stundenkontrolle.
		 * 
		 * @param parent
		 *            Composite, in dem das Element gezeichnet werden soll
		 */
		public MinutenKontrolle(final Composite parent) {
			super(parent, SWT.NO_FOCUS);

			GridLayout gridLayout = new GridLayout();
			gridLayout.marginWidth = 1;
			gridLayout.marginHeight = 1;
			setLayout(gridLayout);

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));

			gridLayout = new GridLayout();
			gridLayout.marginWidth = 2;
			gridLayout.marginHeight = 0;
			fueller.setLayout(gridLayout);

			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(final String text) {
			label.setText(text);
		}

		@Override
		public void setFont(final Font font) {
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		@Override
		public void setBackground(final Color color) {
			fueller.setBackground(color);
			label.setBackground(color);
		}

		@Override
		public void setForeground(final Color color) {
			label.setForeground(color);
		}

		public void setRahmenFarbe(final Color color) {
			super.setBackground(color);
		}

		@Override
		public void handleEvent(final Event event) {
			notifyListeners(event.type, event);
		}
	}

	/**
	 * Zustndig fr die Sekunden.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class SekundenKontrolle extends Composite implements
			Listener {
		private final Composite fueller;

		private final Label label;

		/**
		 * Stundenkontrolle.
		 * 
		 * @param parent
		 *            Composite, in dem das Element gezeichnet werden soll
		 */
		public SekundenKontrolle(final Composite parent) {
			super(parent, SWT.NO_FOCUS);

			GridLayout gridLayout = new GridLayout();
			gridLayout.marginWidth = 1;
			gridLayout.marginHeight = 1;
			setLayout(gridLayout);

			fueller = new Composite(this, SWT.NO_FOCUS);
			fueller.setLayoutData(new GridData(GridData.FILL_BOTH));

			gridLayout = new GridLayout();
			gridLayout.marginWidth = 2;
			gridLayout.marginHeight = 0;
			fueller.setLayout(gridLayout);

			fueller.addListener(SWT.MouseDown, this);
			fueller.addListener(SWT.MouseUp, this);
			fueller.addListener(SWT.MouseDoubleClick, this);

			label = new ElementLabel(fueller, SWT.RIGHT | SWT.NO_FOCUS);
			label.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
					| GridData.HORIZONTAL_ALIGN_CENTER));
			label.addListener(SWT.MouseDown, this);
			label.addListener(SWT.MouseUp, this);
			label.addListener(SWT.MouseDoubleClick, this);

			setRahmenFarbe(parent.getBackground());
			setBackground(parent.getBackground());
			setFont(parent.getFont());
		}

		public void setText(final String text) {
			label.setText(text);
		}

		@Override
		public void setFont(final Font font) {
			super.setFont(font);
			fueller.setFont(font);
			label.setFont(font);
		}

		@Override
		public void setBackground(final Color color) {
			fueller.setBackground(color);
			label.setBackground(color);
		}

		@Override
		public void setForeground(final Color color) {
			label.setForeground(color);
		}

		public void setRahmenFarbe(final Color color) {
			super.setBackground(color);
		}

		@Override
		public void handleEvent(final Event event) {
			notifyListeners(event.type, event);
		}
	}

	/** interne Label-Reprsentation fr die Eingabefelder. */
	private static class ElementLabel extends Label {
		public ElementLabel(final Composite parent, final int style) {
			super(parent, style);
		}

		@Override
		public Point computeSize(final int wHint, final int hHint,
				final boolean changed) {

			int wUsed = wHint;
			if (wUsed == SWT.DEFAULT) {
				final GC gc = new GC(this);
				wUsed = gc.textExtent("22").x;
				gc.dispose();
			}

			return super.computeSize(wUsed, hHint, changed);
		}

		@Override
		protected void checkSubclass() {
			// FIXME Harter Hack, das Subclassing von SWT-Widgets wird hier
			// einfach bergangen
		}
	}
}
