/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.legende;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPart;

/**
 * Implementierungen dieser Schnittstelle werden benutzt um die LegendenSicht zu
 * fllen. Unterklassen von {@link IWorkbenchPart} werden ber die Methode
 * {@link IAdaptable#getAdapter(Class)} nach Instanzen dieser Schnittstelle
 * gefragt. Der Inhalt wird in der Legende angezeigt, so lange der zugehrige
 * {@link IWorkbenchPart} fr den Benutzer sichtbar ist.
 * 
 * @author Hofmann
 * @deprecated das Interface ist nur aus Kompatibilittsgrnden vorhanden. Zur
 *             Anzeige einer Legende fr einen WorkbenchPart muss dieser einen
 *             Adapter fr "ILegende.class" liefern, ansonsten wird die Aktion
 *             zum Ein/Ausblenden der Legende nicht aktiviert.
 * 
 *             Das Interface sollte durch das ersetzt werden oder die
 *             zustzlichen Funktionen:
 *             <ul>
 *             <li>{@link #createControl(Composite)} - entspricht
 *             {@link #erzeugeAbschnittsInhalt(Composite)}</li>
 *             <li>{@link #getControl()} - kann <code>null</code> liefern</li>
 *             <li>{@link #getDefaultCorner()} - kann <code>null</code> liefern</li>
 *             </ul>
 *             mssen implementiert werden.
 */
@Deprecated
public interface ILegendenAbschnitt extends ICustomLegende {

	/**
	 * Fllt die bergebene Flche mit dem Legendeninhalt dieses Abschnitts.
	 * Titel und Rahmen werden von der LegendenSicht dargestellt und sollten
	 * nicht Teil des Abschnittsinhalts sein.
	 * 
	 * @param flaeche
	 *            Zu fllende Flche dieses Legendenabschnitts
	 */
	void erzeugeAbschnittsInhalt(Composite flaeche);

	/**
	 * Eindeutige Bezeichnung dieses Legendenabschnitts.
	 * 
	 * @return eindeutiger Bezeichner
	 */
	String getId();
}
