/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;

import de.bsvrz.buv.rw.compatibility.internal.RwCompatActivator;
import de.kupzog.ktable.KTableModel;
import de.kupzog.ktable.KTableSortComparator;
import de.kupzog.ktable.KTableSortedModel;
import de.kupzog.ktable.renderers.DefaultCellRenderer;
import de.kupzog.ktable.renderers.FixedCellRenderer;

/**
 * Erweiterter {@link FixedCellRenderer} zur Darstellung von gefalteten Zellen.
 * 
 * @author HofmannJ
 * 
 */
public class FixedCellRendererFaltbar extends FixedCellRenderer {
	/**
	 * Pfeil nach rechts.
	 */
	public static final Image IMAGE_PFEIL_RECHTS = RwCompatActivator
			.getDefault().getImage("icons/table/rechts_10.png");

	/**
	 * Pfeil nach links.
	 */
	public static final Image IMAGE_PFEIL_LINKS = RwCompatActivator
			.getDefault().getImage("icons/table/links_10.png");

	/**
	 * Konstruktor entsprechend {@link FixedCellRenderer#FixedCellRenderer(int)}
	 * .
	 * 
	 * @param Argument
	 *            entsprechend {@link FixedCellRenderer#FixedCellRenderer(int)}.
	 */
	public FixedCellRendererFaltbar(final int style) {
		super(style);
	}

	@Override
	protected void drawCellContent(final GC gc, final Rectangle rect,
			final int col, final int row, final Object content,
			final KTableModel model, final Color bgColor, final Color fgColor) {
		drawCellContent(gc, rect, col, row, content, model, bgColor, fgColor,
				true);
	}

	/**
	 * Check for sort indicator and delegate content drawing to
	 * drawCellContent().
	 */
	protected void drawCellContent(final GC gc, final Rectangle rect,
			final int col, final int row, final Object content,
			final KTableModel model, final Color bgColor, final Color fgColor,
			final boolean fixed) {
		final int oldWidth = rect.width;
		Image indicator = null;
		Image pfeil = null;
		int x = 0, y = 0;
		int xPfeil = 0, yPfeil = 0;

		if (((m_Style & DefaultCellRenderer.INDICATION_SORT) != 0)
				&& (model instanceof KTableSortedModel)
				&& (((KTableSortedModel) model).getSortColumn() == col)
				&& (model.getFixedHeaderRowCount() > row) && (row == 0)) {

			final int sort = ((KTableSortedModel) model).getSortState();
			if (sort == KTableSortComparator.SORT_UP) {
				indicator = FixedCellRenderer.IMAGE_ARROWDOWN;
			} else if (sort == KTableSortComparator.SORT_DOWN) {
				indicator = FixedCellRenderer.IMAGE_ARROWUP;
			}

			if (indicator != null) {
				final int contentLength = rect.x + 11
						+ gc.stringExtent(content.toString()).x;
				x = (rect.x + rect.width) - 8;
				if (contentLength < x) {
					x = contentLength;
				} else {
					rect.width -= indicator.getBounds().width + 8;
				}
				y = (rect.y + (rect.height / 2))
						- (indicator.getBounds().height / 2);

				// do not draw if there is not enough space for the image:
				if ((rect.width + 4) < indicator.getBounds().width) {
					rect.width += indicator.getBounds().width + 8;
					indicator = null;
				}
			}
		}

		/*
		 * Pfeilbild setzen
		 */
		if (fixed) {
			final int fixedHeaderRowCount = model.getFixedHeaderRowCount();
			final int fixedHeaderColumnCount = model
					.getFixedHeaderColumnCount();

			if ((row < fixedHeaderRowCount) && (col >= fixedHeaderColumnCount)) {
				// System.out.println("COL: " + col + " ROW: " + row +
				// " FIXED: " + fixed);

				if (model instanceof IFaltStautsSpalten) {
					final IFaltStautsSpalten faltStautsSpalten = (IFaltStautsSpalten) model;
					final FaltStatusSpalten faltStatusSpalten = faltStautsSpalten
							.getFaltStatusSpalten();
					if (faltStatusSpalten.isZelleGefaltet(col, row)) {
						pfeil = FixedCellRendererFaltbar.IMAGE_PFEIL_LINKS;
					} else {
						pfeil = FixedCellRendererFaltbar.IMAGE_PFEIL_RECHTS;
					}
				}

				final int contentLength = rect.x + 11
						+ gc.stringExtent(content.toString()).x;
				xPfeil = (rect.x + rect.width) - 8;
				if (contentLength < xPfeil) {
					xPfeil = contentLength;
				} else {
					if (pfeil != null) {
						rect.width -= pfeil.getBounds().width + 8;
					}
				}

				if (pfeil != null) {
					yPfeil = (rect.y + (rect.height / 2))
							- (pfeil.getBounds().height / 2);
				}

				// do not draw if there is not enough space for the image:
				if (pfeil != null) {
					if ((rect.width + 4) < pfeil.getBounds().width) {
						rect.width += pfeil.getBounds().width + 8;
						pfeil = null;
					}
				}
			}

		}

		drawCellContent(gc, rect, content.toString(), null, fgColor, bgColor);

		// draw sort indicator:
		if (indicator != null) {
			gc.fillRectangle(
					rect.x + rect.width,
					rect.y,
					Math.min(indicator.getBounds().width + 8, oldWidth
							- rect.width - 1), rect.height);
			gc.drawImage(indicator, x, y);
		}

		/*
		 * Faltungspfeil
		 */
		if (pfeil != null) {
			gc.fillRectangle(
					rect.x + rect.width,
					rect.y,
					Math.min(pfeil.getBounds().width + 8, oldWidth - rect.width
							- 1), rect.height);
			gc.drawImage(pfeil, xPfeil, yPfeil);
		}
	}
}
