/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.mehrfachzwischenablage;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IMarkSelection;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Speichert die Daten der Mehrfachzwischenablage.
 * 
 * @author HofmannJ
 * 
 */
public final class MehrfachZwischenablageDaten {
	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Speichert die Eintrge der Mehrfachzwischenablage.
	 */
	private static List<ZwischenablageDatensatz> eintraege = new LinkedList<ZwischenablageDatensatz>();

	/**
	 * Liste der Zuhrer.
	 */
	private static List<IMehrfachZwischenablageDatenZuhoerer> zuhoererListe = new LinkedList<IMehrfachZwischenablageDatenZuhoerer>();

	/**
	 * Speichert die letzte Selektion in einem externen Viewer/Editor.
	 */
	private static List<ZwischenablageDatensatz> selektionsSpeicherExtern = new LinkedList<ZwischenablageDatensatz>();

	/**
	 * Speichert die aktuelle Selektion der Mehrfachzwischenablage.
	 */
	private static List<ZwischenablageDatensatz> selektionsSpeicherMehrfachZwischenablage = new LinkedList<ZwischenablageDatensatz>();

	/**
	 * Liste der Ausschneidezuhrer.
	 */
	private static List<IMehrfachZwischenablageAusschneiden> zuhoererAusschneiden = new LinkedList<IMehrfachZwischenablageAusschneiden>();

	private static class Innen {
		private static MehrfachZwischenablageDaten instanz = new MehrfachZwischenablageDaten();
	}

	/**
	 * Liefert die Singleton-Instanz dieser Klasse.
	 * 
	 * @return Singleton-Instanz dieser Klasse.
	 */
	public static MehrfachZwischenablageDaten getInstanz() {
		return Innen.instanz;
	}

	/**
	 * Fgt einen {@link Object} am Anfang der Liste ein.
	 * 
	 * @param datensatz
	 *            Einzufgendes {@link Object}.
	 */
	protected void hinzufuegenEintrag(final ZwischenablageDatensatz datensatz) {
		if (datensatz != null) {
			synchronized (MehrfachZwischenablageDaten.eintraege) {

				if (!pruefeObObjektInListe(datensatz,
						MehrfachZwischenablageDaten.eintraege)) {
					MehrfachZwischenablageDaten.eintraege.add(0, datensatz);
					benachrichtigeZuhoerer();
					MehrfachZwischenablageDaten.LOGGER
							.fine(getClass().getName()
									+ "::hinzufuegenEintrag \n\tEs wurde ein Eintrag hinzugefgt.\n");
				} else {
					MehrfachZwischenablageDaten.LOGGER
							.error(getClass().getName()
									+ "::hinzufuegenEintraege \n\t=>Das Objekt ist bereits in der Liste vorhanden und wird nicht erneut hinzugefgt.\t");
				}
			}

		} else {
			MehrfachZwischenablageDaten.LOGGER
					.error(getClass().getName()
							+ "::hinzufuegenEintrag \n\t=>Es kann kein NULL Objekt akzeptiert werden!\n");
		}

	}

	/**
	 * Fgt eine Liste vom Typ {@link Object} am Anfang der Liste ein.
	 * 
	 * @param datenSatz
	 *            Liste der einzufgenden {@link Object}e.
	 */
	protected void hinzufuegenEintraege(
			final List<ZwischenablageDatensatz> datenSatz) {
		int counter = 0;
		final Iterator<ZwischenablageDatensatz> it = datenSatz.iterator();
		final List<ZwischenablageDatensatz> tempObjekte = new LinkedList<ZwischenablageDatensatz>();
		while (it.hasNext()) {
			final ZwischenablageDatensatz object = it.next();
			if (object != null) {
				synchronized (MehrfachZwischenablageDaten.eintraege) {
					if (!pruefeObObjektInListe(object,
							MehrfachZwischenablageDaten.eintraege)) {
						tempObjekte.add(object);
						// _eintraege.add(object);
						counter++;
					} else {
						MehrfachZwischenablageDaten.LOGGER
								.error(getClass().getName()
										+ "::hinzufuegenEintraege \n\t=>Das Objekt ist bereits in der Liste vorhanden und wird nicht erneut hinzugefgt.\t");
					}
				}

			} else {
				MehrfachZwischenablageDaten.LOGGER
						.error(getClass().getName()
								+ "::hinzufuegenEintraege \n\t=>Es kann kein NULL Objekt akzeptiert werden!\n");
			}
		}

		synchronized (MehrfachZwischenablageDaten.eintraege) {
			MehrfachZwischenablageDaten.eintraege.addAll(0, tempObjekte);
		}

		benachrichtigeZuhoerer();

		MehrfachZwischenablageDaten.LOGGER.fine(getClass().getName()
				+ "::hinzufuegenEintrag \n\tEs wurden " + counter
				+ " Eintrge hinzugefgt.\n");

	}

	/**
	 * Prft, ob ein Objekt in einer Liste von Objeken schon vorhanden ist.
	 * 
	 * @param objekt
	 *            Das Objekt, auf dessen vorgkommen geprft werden soll.
	 * @param liste
	 *            Die zu prfende Liste.
	 * @return Liefert <code>true</code>, wenn das Objekt schon in der Liste
	 *         vorkommer, sonst <code>false</code>.
	 */
	private boolean pruefeObObjektInListe(final ZwischenablageDatensatz objekt,
			final List<ZwischenablageDatensatz> liste) {
		boolean returnWert = false;
		synchronized (liste) {
			final Iterator<ZwischenablageDatensatz> it = liste.iterator();
			while (it.hasNext()) {
				if (it.next().equals(objekt)) {
					returnWert = true;
					break;
				}
			}
		}
		return returnWert;
	}

	/**
	 * Entfernt ein {@link Object} aus der Liste.
	 * 
	 * @param object
	 *            Zu entfernendes {@link Object}.
	 */
	protected void entferneEintrag(final Object object) {
		synchronized (MehrfachZwischenablageDaten.eintraege) {
			MehrfachZwischenablageDaten.eintraege.remove(object);
			benachrichtigeZuhoerer();
		}

		MehrfachZwischenablageDaten.LOGGER.fine(getClass().getName()
				+ "::hinzufuegenEintrag \n\tEs wurde ein Eintrag entfernt.\n");

	}

	protected void entferneSelektierteEintraege() {
		synchronized (MehrfachZwischenablageDaten.selektionsSpeicherMehrfachZwischenablage) {
			final Iterator<ZwischenablageDatensatz> it = MehrfachZwischenablageDaten.selektionsSpeicherMehrfachZwischenablage
					.iterator();
			while (it.hasNext()) {
				final Object loeschObjekt = it.next();
				synchronized (MehrfachZwischenablageDaten.eintraege) {
					MehrfachZwischenablageDaten.eintraege.remove(loeschObjekt);
				}
			}
			benachrichtigeZuhoerer();
		}
	}

	/**
	 * Fgt einen {@link IMehrfachZwischenablageDatenZuhoerer} zur Liste der
	 * Zuhrer hinzu.
	 * 
	 * @param zuhoerer
	 *            Hinzuzufgender {@link IMehrfachZwischenablageDatenZuhoerer}.
	 */
	public void hinzufuegenZuhoerer(
			final IMehrfachZwischenablageDatenZuhoerer zuhoerer) {
		MehrfachZwischenablageDaten.LOGGER.fine(getClass().getName()
				+ " => Zuhrer registriert.");
		synchronized (MehrfachZwischenablageDaten.zuhoererListe) {
			MehrfachZwischenablageDaten.zuhoererListe.add(zuhoerer);
		}
	}

	/**
	 * Entfernt einen {@link IMehrfachZwischenablageDatenZuhoerer} von der List
	 * der Zuhrer.
	 * 
	 * @param zuhoerer
	 *            Zu entferndender {@link IMehrfachZwischenablageDatenZuhoerer}.
	 */
	public void entferneZuhoerer(
			final IMehrfachZwischenablageDatenZuhoerer zuhoerer) {
		MehrfachZwischenablageDaten.LOGGER.fine(getClass().getName()
				+ " => Zuhrer entfernt");
		synchronized (MehrfachZwischenablageDaten.zuhoererListe) {
			MehrfachZwischenablageDaten.zuhoererListe.remove(zuhoerer);
		}
	}

	/**
	 * Benachrichtigt alle Zuhrer darber, dass sich etwas a.d. Datenstruktur
	 * gendert hat.
	 * 
	 */
	private void benachrichtigeZuhoerer() {
		int i = -1;

		MehrfachZwischenablageDaten.LOGGER.fine(getClass().getName()
				+ "::benachrichtigeZuhoerer \n\t=> Anzahl Zuhrer: "
				+ MehrfachZwischenablageDaten.zuhoererListe.size() + "\n");

		synchronized (MehrfachZwischenablageDaten.zuhoererListe) {

			final Iterator<IMehrfachZwischenablageDatenZuhoerer> it = MehrfachZwischenablageDaten.zuhoererListe
					.iterator();

			while (it.hasNext()) {
				synchronized (MehrfachZwischenablageDaten.eintraege) {
					i++;
					MehrfachZwischenablageDaten.LOGGER
							.fine(getClass().getName()
									+ "::benachrichtigeZuhoerer \n\t => Benachrichtige Zuhrer #"
									+ i + "\n");

					final IMehrfachZwischenablageDatenZuhoerer zuhoerer = it
							.next();
					zuhoerer.eintraegeListeAktualisiert(MehrfachZwischenablageDaten.eintraege);
				}

			}
		}
	}

	/**
	 * Liefert die Liste aller Eintrge (als {@link Object}) i.d.
	 * Mehrfachzwischenablage.
	 * 
	 * @return Liste vom Typ {@link Object}.
	 */
	public List<ZwischenablageDatensatz> getListe() {
		return MehrfachZwischenablageDaten.eintraege;
	}

	/**
	 * Privater Konstruktor.
	 * 
	 */
	private MehrfachZwischenablageDaten() {
		MehrfachZwischenablageDaten.LOGGER
				.fine(getClass().getName()
						+ "::MehrfachZwischenablageDaten \n\t=> erzeuge SelectionListener\n");

		final ISelectionListener listener = new ISelectionListener() {
			@Override
			public void selectionChanged(final IWorkbenchPart part,
					final ISelection selection) {
				/*
				 * IStructuredSelection
				 */
				if (selection instanceof IStructuredSelection) {
					final IStructuredSelection sel = (IStructuredSelection) selection;
					// Object firstElement = sel.getFirstElement();
					// _selektionsSpeicherExtern = firstElement;

					MehrfachZwischenablageDaten.selektionsSpeicherExtern = new LinkedList<ZwischenablageDatensatz>();

					final Iterator<Object> it = sel.iterator();
					while (it.hasNext()) {
						final Object object = it.next();

						MehrfachZwischenablageDaten.LOGGER.fine("SELEKTION: "
								+ object.getClass().getName());
						MehrfachZwischenablageDaten.selektionsSpeicherExtern
								.add(new ZwischenablageDatensatz(object, part));
					}
				} else if (selection instanceof ITextSelection) {
					MehrfachZwischenablageDaten.selektionsSpeicherExtern = new LinkedList<ZwischenablageDatensatz>();
					final ITextSelection ts = (ITextSelection) selection;
					final String text = ts.getText();
					MehrfachZwischenablageDaten.selektionsSpeicherExtern
							.add(new ZwischenablageDatensatz(text, part));
				} else {
					final IMarkSelection ms = (IMarkSelection) selection;
					// try {
					String text;
					try {
						text = ms.getDocument().get(ms.getOffset(),
								ms.getLength());
						MehrfachZwischenablageDaten.selektionsSpeicherExtern
								.add(new ZwischenablageDatensatz(text, part));
					} catch (final BadLocationException e) {
						MehrfachZwischenablageDaten.LOGGER.warning(
								e.getLocalizedMessage(), e);
					}
				}

			}

		};

		PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getSelectionService().addSelectionListener(listener);

		MehrfachZwischenablageDaten.LOGGER
				.fine("MehrfachZwischenablageDaten => SelectionListener registriert.");

	}

	/**
	 * Fgt die Selektion aus dem Selektionsspeicher hinzu.
	 * 
	 */
	public void selektionHinzufuegen() {
		aktionLoeschenBereitsEingefuegterObjekte();

		// _eintraege.add(_selektionsSpeicherExtern);

		/*
		 * Entfernen von Elementen, die bereits i.d. Liste der Eintrge
		 * vorhanden sind.
		 */
		final List<ZwischenablageDatensatz> tempListe = new LinkedList<ZwischenablageDatensatz>();

		synchronized (MehrfachZwischenablageDaten.selektionsSpeicherExtern) {

			final Iterator<ZwischenablageDatensatz> it = MehrfachZwischenablageDaten.selektionsSpeicherExtern
					.iterator();

			while (it.hasNext()) {
				final ZwischenablageDatensatz object = it.next();
				if (pruefeObObjektInListe(object,
						MehrfachZwischenablageDaten.eintraege)) {
					System.err
							.println(getClass().getName()
									+ "::selektionHinzufuegen \n\t=> Das selektierte Objekt ist bereits in der Zwischenablage vorhanden und wird nicht erneut hinzugefgt.");
				} else {
					tempListe.add(object);
				}
			}
		}

		synchronized (MehrfachZwischenablageDaten.eintraege) {
			// _eintraege.addAll(0, _selektionsSpeicherExtern);
			MehrfachZwischenablageDaten.eintraege.addAll(0, tempListe);
		}

		benachrichtigeZuhoerer();
	}

	/**
	 * Fgt die Selektion aus dem Selektionsspeicher hinzu.
	 * 
	 */
	public void selektionHinzufuegenAusschneiden() {
		aktionLoeschenBereitsEingefuegterObjekte();

		/*
		 * Die Methode selektionHinzufuegenAusschneiden wird nur dann aktiv,
		 * wenn der aktive ViewPart bzw. EditPart das Interface
		 * IMehrfachZwischenablageAusschneiden implementiert.
		 */

		/*
		 * Entfernen von Elementen, die bereits i.d. Liste der Eintrge
		 * vorhanden sind.
		 */
		final List<ZwischenablageDatensatz> tempListe = new LinkedList<ZwischenablageDatensatz>();
		synchronized (MehrfachZwischenablageDaten.selektionsSpeicherExtern) {

			final Iterator<ZwischenablageDatensatz> it = MehrfachZwischenablageDaten.selektionsSpeicherExtern
					.iterator();

			while (it.hasNext()) {
				final ZwischenablageDatensatz datenSatz = it.next();
				if (pruefeObObjektInListe(datenSatz,
						MehrfachZwischenablageDaten.eintraege)) {
					System.err
							.println(getClass().getName()
									+ "::selektionHinzufuegen \n\t=> Das selektierte Objekt ist bereits in der Zwischenablage vorhanden und wird nicht erneut hinzugefgt.");
				} else {
					datenSatz.setAusgeschnitten(true);
					tempListe.add(datenSatz);
				}
			}
		}

		synchronized (MehrfachZwischenablageDaten.eintraege) {
			// _eintraege.addAll(0, _selektionsSpeicherExtern);
			MehrfachZwischenablageDaten.eintraege.addAll(0, tempListe);
		}

		benachrichtigeZuhoerer();
		// benachrichtigeZuhoererAusschneiden();
	}

	/**
	 * Lscht bereits eingefgte Elemente aus der Zwischenablage.
	 */
	private void aktionLoeschenBereitsEingefuegterObjekte() {
		final LinkedList<ZwischenablageDatensatz> loeschListe = new LinkedList<ZwischenablageDatensatz>();

		synchronized (MehrfachZwischenablageDaten.eintraege) {

			/*
			 * Elmente der Zwischenablage, die bereits eingefgt wurden, in die
			 * Lschliste aufnehmen.
			 */
			final Iterator<ZwischenablageDatensatz> itElementeZwischenablage = MehrfachZwischenablageDaten.eintraege
					.iterator();
			while (itElementeZwischenablage.hasNext()) {
				final ZwischenablageDatensatz datensatz = itElementeZwischenablage
						.next();
				if (datensatz.isBereitsEingefuegt()) {
					loeschListe.add(datensatz);
				}
			}

			/*
			 * Lschliste durchlaufen, gefundene Elemente aus Zwischenablage
			 * entfernen.
			 */
			if (!loeschListe.isEmpty()) {
				final Iterator<ZwischenablageDatensatz> it = loeschListe
						.iterator();
				while (it.hasNext()) {
					final ZwischenablageDatensatz datensatz = it.next();
					if (MehrfachZwischenablageDaten.eintraege
							.contains(datensatz)) {
						MehrfachZwischenablageDaten.eintraege.remove(datensatz);
					}
				}
			}
		}
	}

	/**
	 * Setzt das aktuell selektierte Objekt der
	 * {@link SichtMehrfachZwischenablage}.
	 * 
	 * @param objekt
	 *            Ein Objekt.
	 */
	protected void setSelektionMehrfachZwischenablageIntern(
			final List<ZwischenablageDatensatz> objekte) {
		synchronized (MehrfachZwischenablageDaten.selektionsSpeicherMehrfachZwischenablage) {
			MehrfachZwischenablageDaten.selektionsSpeicherMehrfachZwischenablage = objekte;
		}
	}

	/**
	 * Liefert das aktuell selektierte Objekt der
	 * {@link SichtMehrfachZwischenablage}.
	 * 
	 * @return Ein Objekt.
	 */
	protected List<ZwischenablageDatensatz> getSelektionMehrfachZwischenablageIntern() {
		final LinkedList<ZwischenablageDatensatz> re = new LinkedList<ZwischenablageDatensatz>();

		if (MehrfachZwischenablageDaten.selektionsSpeicherMehrfachZwischenablage
				.isEmpty()) {
			synchronized (MehrfachZwischenablageDaten.eintraege) {
				final Iterator<ZwischenablageDatensatz> it = MehrfachZwischenablageDaten.eintraege
						.iterator();
				while (it.hasNext()) {
					re.add(it.next());
				}
			}
		} else {
			synchronized (MehrfachZwischenablageDaten.selektionsSpeicherMehrfachZwischenablage) {
				final Iterator<ZwischenablageDatensatz> it = MehrfachZwischenablageDaten.selektionsSpeicherMehrfachZwischenablage
						.iterator();
				while (it.hasNext()) {
					re.add(it.next());
				}
			}
		}

		// return _selektionsSpeicherMehrfachZwischenablage;
		return re;
	}

	/**
	 * Fgt einen Zuhrer vom Typ {@link IMehrfachZwischenablageAusschneiden}
	 * zur Liste der Zuhrer hinzu.
	 * 
	 * @param ausschneiden
	 *            Hinzuzufgender {@link IMehrfachZwischenablageAusschneiden}
	 *            -Zuhrer.
	 */
	public void hinzufuegenZuhoererAusschneiden(
			final IMehrfachZwischenablageAusschneiden ausschneiden) {
		synchronized (MehrfachZwischenablageDaten.zuhoererAusschneiden) {
			MehrfachZwischenablageDaten.zuhoererAusschneiden.add(ausschneiden);
		}
	}

	/**
	 * Entfernt einen Zuhrer vom Typ
	 * {@link IMehrfachZwischenablageAusschneiden} von der Liste der Zuhrer.
	 * 
	 * @param ausschneiden
	 *            Zu entfernender {@link IMehrfachZwischenablageAusschneiden}
	 *            -Zuhrer.
	 */
	public void entferneZuhoererAusschneiden(
			final IMehrfachZwischenablageAusschneiden ausschneiden) {
		synchronized (MehrfachZwischenablageDaten.zuhoererAusschneiden) {
			MehrfachZwischenablageDaten.zuhoererAusschneiden
					.remove(ausschneiden);
		}
	}

	/**
	 * Benachrichtigt die AusschneideZuhrer ber die zu entfernenden Objekte.
	 * Nur Objekte der Datenstze, die auf isAusgeschnitten <code>true</code>
	 * liefern, werden ausgeschnitten.
	 */
	protected void benachrichtigeZuhoererAusschneiden(
			final ZwischenablageDatensatz zwischenablageDatensatz) {
		synchronized (MehrfachZwischenablageDaten.zuhoererAusschneiden) {
			final Iterator<IMehrfachZwischenablageAusschneiden> it = MehrfachZwischenablageDaten.zuhoererAusschneiden
					.iterator();
			while (it.hasNext()) {
				final IMehrfachZwischenablageAusschneiden ausschneiden = it
						.next();

				if (ausschneiden.getTeil().equals(
						zwischenablageDatensatz.getHerkunft())
						&& zwischenablageDatensatz.isAusgeschnitten()) {
					ausschneiden.entferneSelektion(zwischenablageDatensatz
							.getWert());
				}
			}
		}
	}

	/**
	 * Leert die Eintrge in der Mehrfachzwischenablage und informiert alle
	 * Zuhrer darber.
	 */
	protected void leereMehrfachZwischenAblage() {
		MehrfachZwischenablageDaten.eintraege.clear();
		benachrichtigeZuhoerer();
	}

}
