/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.berechtigung.IOberflaechenBerechtigungZuhoererErweitert;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenFunktion;
import de.bsvrz.buv.rw.basislib.datenidentifikation.DatenidentifikationGtm;
import de.bsvrz.buv.rw.basislib.datenidentifikation.GtmListeComposite;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.buv.rw.compatibility.internal.RwCompatActivator;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.DaVPreLader;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.IDaVPreLader;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse liefert eine Sicht (Instanz der Klasse {@link ViewPart}), mit
 * deren Hilfe sich Objekte zusammenstellen lassen.<br>
 * <br>
 * <br>
 * <b><u>Lebenszyklusabhniges Laden und Speichern aus dem Memento der
 * View:</u></b><br>
 * War whrend des Lebenszyklus dieser View eine Verbindung zum DaV aktiv, so
 * wir das Flag <code>_davConnectionInLifetime<code> auf <code>true</code>
 * gesetzt. Dies kann in <code>createPartControl</code> sowie in dem dort
 * erzeugten {@link IDaVVerbindungZuhoerer} ausgelst werden.<br>
 * Regeln:
 * <ul>
 * <li>Wenn beim Beenden
 * <code>_davConnectionInLifetime == true<code> galt, wird der Zustand der View gespeichert.
 * </li>
 * <li>Ist beim Starten der View
 * <code>_davConnectionInLifetime == true<code>, wird der Zustand der View wiederhergestellt.
 * </li>
 * <li>Ist beim Starten der View
 * 
 * <code>_davConnectionInLifetime == false<code>, kann der alte Zustand nicht wiederhergestellt werden. Meldet jedoch der Listener <code>_zuhoererDaVVerbindung</code>
 * dass eine Verbindung zum DaV aktiv ist, wird auch
 * <code>_davConnectionInLifetime<code> auf <code>true</code> gesetzt und der
 * Zustand der View wiederhergestellt.</li>
 * </ul>
 * <br>
 * Der Vorteil ist, dass der Zustand der Objektzusammenstellung auch dann
 * gesichert wird, wenn whrend des Betriebes die Verbindung zum DaV
 * unterbrochen wird. <br>
 * <br>
 * Erfllt die Anforderungen von TBuV-98
 * 
 * @author HofmannJ
 * 
 */
public class SichtObjektzusammenstellung extends ViewPart {
	/**
	 * Im Memento dieser View knnen maximal 9999 Attribute mit Wert abgelegt
	 * werden. Hierfr ist Eclipse 3.3 verantwortlich.<br>
	 * Das Memento selbst vertrgt 10001 Attribute, jedoch geht eins fr die
	 * Anzahl der SystemObjekte und eines fr die SecondaryID dieser View drauf. <br>
	 * Hinweismeldung erfolgt bei der bernahme aus dem
	 * Datenidentifikationsdialog, bei der Speicherung im Memento der View
	 * werden dem entsprechend nur die ersten 9999 bercksichtigt.
	 */
	private static final int MAX_SIZE_XML_MEMENTO = 9999;

	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * ID des {@link ViewPart}s.
	 */
	public static final String ID = "de.bsvrz.buv.rw.basislib.SichtObjektzusammenstellung";

	/**
	 * Secondary ID dieser View fr Multipleviews.
	 */
	private String secondaryID;

	private Composite oben;

	/**
	 * Gruppe zur Darstellung von Objekten.
	 */
	private Group gruppeObjekte;

	/**
	 * Mittlerer Bereich.
	 */
	private Composite compositeMitte;

	/**
	 * Gruppe zur Darstellung der Beschreibung.
	 */
	private Group gruppeBeschreibung;

	/**
	 * Textfeld zur Darstellung der Beschreibung.
	 */
	private Text textFeldBeschreibung;

	/**
	 * CheckBox zum Einschrnken des Objekttyps.
	 */
	private Button checkBoxObjekttypEinschraenken;

	/**
	 * Linker Bereich.
	 */
	private Composite compositeObjekteLinks;

	/**
	 * Rechter Bereich.
	 */
	private Composite compositeObjekteRechts;

	/**
	 * Schaltflche Hinzufgen zum Hinzufgen von Objekten.
	 */
	private Button schaltflaecheHinzufuegen;

	/**
	 * Schaltflche Lschen zum Entfernen von selektierten Objekten.
	 */
	private Button schaltflaecheLoeschen;

	/**
	 * Bereich unten Rechts.
	 */
	private Composite compositeObjekteRechtsUnten;

	/**
	 * Schaltflche Hoch, bewegt das selektierte Objekt um eine Position nach
	 * oben.
	 */
	private Button schaltflaecheHoch;

	/**
	 * Schaltflche Runter, bewegt das selektierte Objekt um eine Position nach
	 * unten.
	 */
	private Button schaltflaecheRunter;

	/**
	 * Bereich fr die Schaltflaechen.
	 */
	private Composite compositeKontrolle;

	/**
	 * Schaltflche OK.
	 */
	private Button schaltflaecheOK;

	/**
	 * Schaltflche Abbrechen.
	 */
	private Button schaltflaecheAbbrechen;

	/**
	 * Schaltflche Hilfe.
	 */
	private Button schaltflaecheHilfe;

	/**
	 * Liste der Oberflchen Elemente die dem Berechtigungskonzept unterliegen
	 * sollen.
	 */
	private final Map<String, Object> oberflaechenElemente = new HashMap<String, Object>();

	/**
	 * Der Listener fr das Berechtigungskonzept.
	 */
	private IOberflaechenBerechtigungZuhoererErweitert berechtigungsZuhoererErweitert;

	/**
	 * Instanz der Oberflchen Berechtigung.
	 */
	private final OberflaechenBerechtigung oberflaechenBerechtigung = OberflaechenBerechtigung
			.getInstanz();

	/**
	 * Verbindung zum DaV. Wird selbststndig besorgt.
	 */
	private ClientDavInterface connection;

	/**
	 * ber die Check-Box "Objekttypen einschrnken" wird vorgegeben, dass nur
	 * Datenidentifikationen in die Objektzusammenstellung bernommen werden
	 * knnen, die von einem Objekttyp sind, der bereits in der
	 * Objektzusammenstellung vorkommt. Wenn z.B. nur Objekte vom Typ MQ in der
	 * Objektzusammenstellung enthalten sind und die Check-Box aktiviert ist,
	 * dann knnen auch nur weitere Objekte vom Typ MQ hinzugefht werden.
	 * Dieser Umstand muss sich direkt fr die Datenidentifikations-Eingabe
	 * auswirken. D.h. wenn in diesem Fall "Hinzufgen" bettigt wird, muss die
	 * Datenidentifikations-Eingabe bereits auf Objekte vom Typ "MQ" filtern.
	 * Drag und Drop wird nur fr Objekte der entsprechenden Objekttypen
	 * zugelassen Die Check-Box ist optional. Fr die Zusammenstellung eines
	 * Streckenprofils ist die Check-Box nicht erforderlich. In diesen Fllen
	 * bestimmt die Anwendung bereits welche Objekttypen zulssig sind. Checkbox
	 * ja/nein
	 * 
	 * Gibt an, ob nach ObjektTyp eingeschrnkt werden soll oder nicht.
	 */
	private boolean einschraenkenObjektTyp;

	/**
	 * Liste der Systemobjekte.
	 */
	private List<SystemObject> listeDaten = new LinkedList<SystemObject>();

	/**
	 * {@link TableViewer} zur Darstellung der {@link SystemObject}s.
	 */
	private static TableViewer tabelleSystemObjekte;

	/**
	 * Zuhrer auf die DaVVerbindung.
	 */
	private static DavVerbindungsListener davListener;

	/**
	 * Liste der {@link IObjektzusammenstellungZuhoerer}, die beim Klicken auf
	 * OK darber informiert werden wollen, dass {@link SystemObject}s
	 * zusammengestellt wurden.
	 */
	private static List<IObjektzusammenstellungZuhoerer> zuhoererObjektzusammenstellung = new LinkedList<IObjektzusammenstellungZuhoerer>();

	/**
	 * DropAdapter fr den TableViewer.
	 */
	private ObjektzusammenstellungFallAdapter objektzusammenstellungFallAdapter;

	/**
	 * DragListener fr den TableViewer.
	 */
	private ZugZuhoererSystemObjekt zugZuhoererSystemObjekt;

	/**
	 * Memento fr die Wiederherstellung.
	 */
	private IMemento sichtMemento;

	/**
	 * Speichert die IDs der Systemobjekte fr das Memento.
	 */
	private final LinkedList<Long> systemObjekteMemento = new LinkedList<Long>();

	/**
	 * Prelader fr Informationen vom DaV.
	 */
	private final DaVPreLader preLader = DaVPreLader.getInstanz();

	/**
	 * Listener fr den Prelader. Ist u.a. fr das Ein- und Ausblenden des
	 * Hinzufgenknopfes zustndig.
	 */
	private IDaVPreLader preLaderListener;

	@Override
	public void createPartControl(final Composite parent) {
		connection = RahmenwerkService.getService().getRahmenWerk()
				.getDavVerbindung();

		if (SichtObjektzusammenstellung.davListener != null) {
			SichtObjektzusammenstellung.LOGGER.fine(getClass().getName()
					+ "::createPartControl \t\n=>Entferne alten Listener.\n");

			RahmenwerkService
					.getService()
					.getRahmenWerk()
					.removeDavVerbindungsListener(
							SichtObjektzusammenstellung.davListener);
			SichtObjektzusammenstellung.davListener = null;
		}

		SichtObjektzusammenstellung.davListener = new DavVerbindungsListener() {

			@Override
			public void verbindungHergestellt(final DavVerbindungsEvent event) {
				connection = event.getDavVerbindung();

				System.out
						.println(getClass().getName()
								+ "::createPartControl \n\t=> Es wurde eine Verbindung zum DaV aufgebaut. Der Dialog zur Datenidentifikation kann angezeigt werden.\n");

				if (schaltflaecheHinzufuegen != null) {
					if (preLader.istInitialesLadenAbgeschlossen()) {
						schaltflaecheHinzufuegen.setEnabled(true);
					}

				}
			}

			@Override
			public void verbindungGetrennt(final DavVerbindungsEvent event) {
				connection = null;
				SichtObjektzusammenstellung.LOGGER
						.info(getClass().getName()
								+ "::createPartControl \n\t=> Die Verbindung zum DaV wurde getrennt oder wurde noch nicht aufgebaut. Der Dialog zur Datenidentifikation kann NICHT werden.\n");

				if (schaltflaecheHinzufuegen != null) {
					final Runnable run = new Runnable() {
						@Override
						public void run() {
							if (!schaltflaecheHinzufuegen.isDisposed()) {
								schaltflaecheHinzufuegen.setEnabled(false);
							}
						}
					};

					final Display display = PlatformUI.getWorkbench()
							.getDisplay();
					if (!display.isDisposed()) {
						PlatformUI.getWorkbench().getDisplay().asyncExec(run);
					}
				}
			}

			@Override
			public boolean verbindungHalten(final DavVerbindungsEvent event) {
				return false;
			}

		};
		RahmenwerkService
				.getService()
				.getRahmenWerk()
				.addDavVerbindungsListener(
						SichtObjektzusammenstellung.davListener);

		preLaderListener = new IDaVPreLader() {

			@Override
			public void systemObjekteAktualisierung(
					final GtmDatenBasis datenBasis) {
				if (schaltflaecheHinzufuegen != null) {
					final Runnable run = new Runnable() {
						@Override
						public void run() {
							schaltflaecheHinzufuegen.setEnabled(true);
						}
					};

					final Display display = PlatformUI.getWorkbench()
							.getDisplay();
					if (!display.isDisposed()) {
						display.asyncExec(run);
					}

				}

			}

			@Override
			public void systemObjekteInitial(final GtmDatenBasis datenBasis) {
				if (schaltflaecheHinzufuegen != null) {
					final Runnable run = new Runnable() {
						@Override
						public void run() {
							schaltflaecheHinzufuegen.setEnabled(true);
						}
					};

					final Display display = PlatformUI.getWorkbench()
							.getDisplay();
					if (!display.isDisposed()) {
						display.asyncExec(run);
					}

				}

			}

		};
		preLader.hinzufuegenListenerPreLader(preLaderListener);

		oben = new Composite(parent, SWT.NONE);
		oben.setLayout(new GridLayout());
		erzeugeGruppeObjekte();
		erzeugeCompositeMitte();
		erzeugeGruppeBeschreibung();
		erzeugeCompositeKontrolle();

		// KmdParameter kp = _kommandoZeilenParameter
		// .getKommandoZeilenParameter("-berechtigungsKonzept");
		// String s = kp.getWert();
		// if (s.equals("true"))
		// {
		erzeugeOberflaechenBerechtigung();
		// }

		restoreState();
	}

	private void erzeugeOberflaechenBerechtigung() {

		berechtigungsZuhoererErweitert = new IOberflaechenBerechtigungZuhoererErweitert() {
			@Override
			public void freigabe(final OberflaechenBerechtigungEreignis e) {
				SichtObjektzusammenstellung.LOGGER
						.fine("Info Freigabe erhalten: " + e.hatFreigabe());

				final List<SystemObject> liste1 = e.getSystemObjekteFreigabe();
				final List<SystemObject> liste2 = e.getSystemObjekteSperrung();
				for (final SystemObject so : liste1) {
					SichtObjektzusammenstellung.LOGGER
							.fine("Objekte Freigabe: " + so);
				}
				for (final SystemObject so : liste2) {
					SichtObjektzusammenstellung.LOGGER
							.fine("Objekte Sperrung: " + so);
				}

				final List<OberflaechenFunktion> liste3 = e
						.getOberflaechenFunktionFreigabe();
				final List<OberflaechenFunktion> liste4 = e
						.getOberflaechenFunktionSperrung();
				for (final OberflaechenFunktion of : liste3) {
					SichtObjektzusammenstellung.LOGGER
							.fine("Funktion Freigabe: " + of.getFunktionsName());
					final Object object = oberflaechenElemente.get(of
							.getFunktionsName());

					if (object instanceof Button) {

						final Button button = (Button) object;

						final Display display = Display.getDefault();

						if (!display.isDisposed()) {
							display.asyncExec(new Runnable() {
								@Override
								public void run() {
									if (!button.isDisposed()) {
										button.setEnabled(true);
									}
								}
							});
						}

					}
				}
				for (final OberflaechenFunktion of : liste4) {
					SichtObjektzusammenstellung.LOGGER
							.fine("Funktion Sperrung: " + of.getFunktionsName());
				}

			}

			@Override
			public void sperrung(final OberflaechenBerechtigungEreignis e) {
				SichtObjektzusammenstellung.LOGGER
						.fine("Info Sperrung erhalten: " + e.hatFreigabe());

				final List<SystemObject> liste1 = e.getSystemObjekteFreigabe();
				final List<SystemObject> liste2 = e.getSystemObjekteSperrung();
				for (final SystemObject so : liste1) {
					SichtObjektzusammenstellung.LOGGER
							.fine("Objekte Freigabe: " + so);
				}
				for (final SystemObject so : liste2) {
					SichtObjektzusammenstellung.LOGGER
							.fine("Objekte Sperrung: " + so);
				}

				final List<OberflaechenFunktion> liste3 = e
						.getOberflaechenFunktionFreigabe();
				final List<OberflaechenFunktion> liste4 = e
						.getOberflaechenFunktionSperrung();
				for (final OberflaechenFunktion of : liste3) {
					SichtObjektzusammenstellung.LOGGER
							.fine("Funktion Freigabe: " + of.getFunktionsName());
				}
				for (final OberflaechenFunktion of : liste4) {
					SichtObjektzusammenstellung.LOGGER
							.fine("Funktion Sperrung: " + of.getFunktionsName());
					final Object object = oberflaechenElemente.get(of
							.getFunktionsName());

					if (object instanceof Button) {
						final Button button = (Button) object;
						final Display display = Display.getDefault();

						if (!display.isDisposed()) {
							display.asyncExec(new Runnable() {
								@Override
								public void run() {
									if (!button.isDisposed()) {
										button.setEnabled(false);
									}
								}
							});
						}

					}
				}
			}

			@Override
			public void oberflaechenBerechtigungBereit() {
				Display.getDefault().asyncExec(new Runnable() {
					@Override
					public void run() {
						final IStructuredSelection selection = (IStructuredSelection) SichtObjektzusammenstellung.tabelleSystemObjekte
								.getSelection();

						final Object firstElement = selection.getFirstElement();
						if (firstElement instanceof SystemObject) {
							final List<SystemObject> liste = new LinkedList<SystemObject>();
							final Iterator<SystemObject> it = selection
									.iterator();
							while (it.hasNext()) {
								final SystemObject so = it.next();
								liste.add(so);
							}
						}

					}

				});
			}
		};

		// XXX BenutzerRechte-Zuhoerer hinzufuegen ergnzen
		// de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation.getInstanz()
		// .hinzufuegenOberflaechenBenutzerZuhoerer(
		// _berechtigungsZuhoererErweitert);

		final OberflaechenFunktion funktion1 = new OberflaechenFunktion(
				"SichtObjektzusammenstellungOk",
				berechtigungsZuhoererErweitert, "SichtObjektzusammenstellung",
				"Ok");
		final OberflaechenFunktion funktion2 = new OberflaechenFunktion(
				"SichtObjektzusammenstellungAbbrechen",
				berechtigungsZuhoererErweitert, "SichtObjektzusammenstellung",
				"Abbrechen");
		final OberflaechenFunktion funktion3 = new OberflaechenFunktion(
				"SichtObjektzusammenstellungHilfe",
				berechtigungsZuhoererErweitert, "SichtObjektzusammenstellung",
				"Hilfe");
		final OberflaechenFunktion funktion4 = new OberflaechenFunktion(
				"SichtObjektzusammenstellungHinzufgen",
				berechtigungsZuhoererErweitert, "SichtObjektzusammenstellung",
				"Hinzufgen");
		final OberflaechenFunktion funktion5 = new OberflaechenFunktion(
				"SichtObjektzusammenstellungLschen",
				berechtigungsZuhoererErweitert, "SichtObjektzusammenstellung",
				"Lschen");

		oberflaechenElemente.put(funktion1.getFunktionsName(), schaltflaecheOK);
		oberflaechenElemente.put(funktion2.getFunktionsName(),
				schaltflaecheAbbrechen);
		oberflaechenElemente.put(funktion3.getFunktionsName(),
				schaltflaecheHilfe);
		oberflaechenElemente.put(funktion4.getFunktionsName(),
				schaltflaecheHinzufuegen);
		oberflaechenElemente.put(funktion5.getFunktionsName(),
				schaltflaecheLoeschen);

		oberflaechenBerechtigung.hinzufuegenOberflaechenFunktion(funktion1);
		oberflaechenBerechtigung.hinzufuegenOberflaechenFunktion(funktion2);
		oberflaechenBerechtigung.hinzufuegenOberflaechenFunktion(funktion3);
		// _oberflaechenBerechtigung.hinzufuegenOberflaechenFunktion(funktion4);
		// _oberflaechenBerechtigung.hinzufuegenOberflaechenFunktion(funktion5);
	}

	@Override
	public void setFocus() {
		this.setTitleToolTip("SecondaryID: " + getSecondaryId());
	}

	/**
	 * Setzt die angezeigte Liste der SystemObjekte.
	 * 
	 */
	public void setListeDaten(final List<SystemObject> systemObjekte) {
		this.listeDaten = systemObjekte;
		// _lvListe.setInput(_listeDaten);
		SichtObjektzusammenstellung.tabelleSystemObjekte.setInput(listeDaten);
	}

	/**
	 * Liefert die Liste der dargestellten Systemobjekte.
	 * 
	 * @return
	 */
	public List<SystemObject> getListeDaten() {
		return listeDaten;
	}

	/**
	 * Fgt ein Objekt zur Liste hinzu.
	 * 
	 */
	private void hinzufuegenObjekt() {
		if (connection != null) {

			LinkedList<SystemObjectType> objektTypen = null;
			if (einschraenkenObjektTyp) {
				final DataModel dataModel = connection.getDataModel();
				objektTypen = new LinkedList<SystemObjectType>();
				final String[] vorhandeneObjektTypen = bestimmeVorhandeneObjektTypen();
				for (final String pid : vorhandeneObjektTypen) {
					objektTypen.add(dataModel.getType(pid));
				}
			}

			LinkedList<SystemObject> systemObjekte = null;

			final DatenidentifikationGtm dialog = new DatenidentifikationGtm(
					PlatformUI.getWorkbench().getDisplay().getActiveShell(),
					null);
			if (objektTypen == null) {
				systemObjekte = dialog
						.oeffnen(GtmListeComposite.DARSTELLUNG_SYSTEMOBJEKTE_OBJEKTTYPEN_ATTRIBUTGRUPPEN_ASPEKTE);
			} else {
				System.out.println("Anzahl ObjektTypen: " + objektTypen.size());

				systemObjekte = dialog
						.oeffnen(
								objektTypen,
								GtmListeComposite.DARSTELLUNG_SYSTEMOBJEKTE_OBJEKTTYPEN_ATTRIBUTGRUPPEN_ASPEKTE);
			}

			if (systemObjekte != null) {
				listeDaten.addAll(systemObjekte);
				SichtObjektzusammenstellung.tabelleSystemObjekte
						.setInput(listeDaten);

				if (listeDaten.size() > SichtObjektzusammenstellung.MAX_SIZE_XML_MEMENTO) {
					final MessageBox mb = new MessageBox(PlatformUI
							.getWorkbench().getDisplay().getActiveShell(),
							SWT.OK | SWT.ICON_WARNING);
					String msg = "";
					msg = msg
							+ "Die Anzahl der Systemobjekte in dieser Sicht ist > 10000!\n";
					msg = msg
							+ "Zur Laufzeit dieser Applikation ist dies kein Problen,\n";
					msg = msg + "smtliche Daten bleiben verfgbar!\n";
					msg = msg
							+ "Bleibt diese Sicht jedoch beim Beenden von Eclipse geffnet,\n";
					msg = msg
							+ "so werden nur die ersten 10000 SystemObjekte zum Wiederaufbau\n";
					msg = msg
							+ "dieser Sicht gespeichert. Mehr kann durch die Eclipse RCP,\n";
					msg = msg
							+ "welche die Basis dieser Anwendung ist, nicht verarbeitet\n";
					msg = msg + "werden.";
					mb.setText("Anzahl Systemobjekte > 10000");
					mb.setMessage(msg);
					mb.open();
				}
			}

		} else {
			final MessageBox box = new MessageBox(new Shell(
					Display.getDefault()), SWT.OK | SWT.ICON_ERROR);
			box.setText("Keine Verbindung zum DaV");
			box.setMessage("Es wurde keine Verbindung zum DaV bergeben. \nDer Dialog zur Datenidentifikation wird nicht angezeigt.");
			box.open();
		}

	}

	/**
	 * Diese Methode entfernt die in der Tabelle selektierten Objekte.
	 */
	private void entferneSelektierteObjekte() {
		// IStructuredSelection selection = (IStructuredSelection) _lvListe
		// .getSelection();
		final IStructuredSelection selection = (IStructuredSelection) SichtObjektzusammenstellung.tabelleSystemObjekte
				.getSelection();
		final Object[] objects = selection.toArray();
		for (final Object object : objects) {
			if (object instanceof SystemObject) {
				final SystemObject systemObject = (SystemObject) object;
				listeDaten.remove(systemObject);
			}
		}
		// _lvListe.setInput(_listeDaten);
		SichtObjektzusammenstellung.tabelleSystemObjekte.setInput(listeDaten);

		/**
		 * Schaltflche Lschen deaktivieren.
		 */
		schaltflaecheLoeschen.setEnabled(false);
	}

	/**
	 * Bestimmt die in der Liste vorhandenen ObjektTypen und gibt deren Namen in
	 * einem Array von Strings zurck. Die wird bentigt wenn der der Haken fr
	 * die Einschrnkung gesetzt wird, dass nur Objekte vom gleichen Typ ber
	 * den Datenidentifiaktions- Dialog aufgenommen werden sollen.
	 */
	private String[] bestimmeVorhandeneObjektTypen() {
		// Rckgabe String
		String[] tempString = null;

		final TreeMap<String, String> tempMap = new TreeMap<String, String>();
		final Iterator<SystemObject> it = listeDaten.iterator();
		while (it.hasNext()) {
			final SystemObject tempSO = it.next();

			final String insertKey = tempSO.getType().getPid();
			final String insertValue = tempSO.getType().getName();
			if (!tempMap.containsKey(insertKey)) {
				tempMap.put(insertKey, insertValue);
			}
		}

		if (tempMap.size() > 0) {
			int i = 0;
			tempString = new String[tempMap.size()];
			final Iterator<Entry<String, String>> itTempMap = tempMap
					.entrySet().iterator();
			while (itTempMap.hasNext()) {
				tempString[i] = itTempMap.next().getKey();
				i++;
			}
		}

		return tempString;
	}

	/**
	 * Erzeugt die Gruppe gruppeObjekte.
	 * 
	 */
	private void erzeugeGruppeObjekte() {
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		gruppeObjekte = new Group(oben, SWT.NONE);
		gruppeObjekte.setLayoutData(gridData);
		erzeugeCompositeObjekteLinks();
		gruppeObjekte.setLayout(gridLayout);
		erzeugeCompositeObjekteRechts();
		gruppeObjekte.setText("Objekte");
	}

	/**
	 * Erzeugt den Bereich compositeMitte.
	 * 
	 */
	private void erzeugeCompositeMitte() {
		final RowLayout rowLayout = new RowLayout();
		rowLayout.fill = false;
		rowLayout.pack = true;
		rowLayout.justify = true;
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		compositeMitte = new Composite(oben, SWT.NONE);
		compositeMitte.setLayoutData(gridData1);
		compositeMitte.setLayout(rowLayout);
		checkBoxObjekttypEinschraenken = new Button(compositeMitte, SWT.CHECK);
		checkBoxObjekttypEinschraenken.setText("Objekttyp einschrnken");

		checkBoxObjekttypEinschraenken
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {
						final Object source = e.getSource();
						if (source instanceof Button) {
							final Button b = (Button) source;

							if (b.getSelection()) {
								einschraenkenObjektTyp = true;
							} else {
								einschraenkenObjektTyp = false;
							}
						}
					}
				});

		// Checkbox ObjekttypEinschraenken abhaken, wen eingeschrnkt werden
		// soll.
		if (einschraenkenObjektTyp) {
			checkBoxObjekttypEinschraenken.setSelection(true);
		} else {
			// Sonst nicht.
			checkBoxObjekttypEinschraenken.setSelection(false);
		}

	}

	/**
	 * Erzeugt die Gruppe gruppeBeschreibung.
	 * 
	 */
	private void erzeugeGruppeBeschreibung() {
		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		gruppeBeschreibung = new Group(oben, SWT.NONE);
		gruppeBeschreibung.setLayout(new FillLayout());
		gruppeBeschreibung.setLayoutData(gridData2);
		gruppeBeschreibung.setText("Beschreibung");
		textFeldBeschreibung = new Text(gruppeBeschreibung, SWT.MULTI
				| SWT.WRAP | SWT.V_SCROLL);
		textFeldBeschreibung.setEditable(false);
	}

	/**
	 * Erzeugt den Bereich compositeObjekteLinks.
	 * 
	 */
	private void erzeugeCompositeObjekteLinks() {
		final GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.FILL;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.grabExcessVerticalSpace = true;
		gridData4.verticalAlignment = GridData.FILL;
		compositeObjekteLinks = new Composite(gruppeObjekte, SWT.NONE);
		compositeObjekteLinks.setLayout(new FillLayout());
		compositeObjekteLinks.setLayoutData(gridData4);

		erzeugeTabelle();
	}

	/**
	 * Erzeugt die Tabelle zur Darstellung der Systemobjekte.
	 */
	private void erzeugeTabelle() {
		SichtObjektzusammenstellung.tabelleSystemObjekte = new TableViewer(
				compositeObjekteLinks, SWT.FULL_SELECTION | SWT.MULTI
						| SWT.BORDER);

		final TableColumn tc1 = new TableColumn(
				SichtObjektzusammenstellung.tabelleSystemObjekte.getTable(),
				SWT.CENTER);
		tc1.setText("PID:");
		tc1.setWidth(250);

		final TableColumn tc2 = new TableColumn(
				SichtObjektzusammenstellung.tabelleSystemObjekte.getTable(),
				SWT.LEFT);
		tc2.setText("SystemObjekt:");
		tc2.setWidth(350);

		SichtObjektzusammenstellung.tabelleSystemObjekte.getTable()
				.setHeaderVisible(true);
		SichtObjektzusammenstellung.tabelleSystemObjekte.getTable()
				.setLinesVisible(true);

		SichtObjektzusammenstellung.tabelleSystemObjekte
				.setContentProvider(new TabelleInhaltsVersorger());
		SichtObjektzusammenstellung.tabelleSystemObjekte
				.setLabelProvider(new TabelleDarstellungsVersorger());

		final int ops = DND.DROP_COPY | DND.DROP_MOVE;
		objektzusammenstellungFallAdapter = new ObjektzusammenstellungFallAdapter(
				SichtObjektzusammenstellung.tabelleSystemObjekte, listeDaten);
		final Transfer[] dropTransfers = new Transfer[] { TextTransfer
				.getInstance() };
		SichtObjektzusammenstellung.tabelleSystemObjekte.addDropSupport(ops,
				dropTransfers, objektzusammenstellungFallAdapter);

		zugZuhoererSystemObjekt = new ZugZuhoererSystemObjekt(
				SichtObjektzusammenstellung.tabelleSystemObjekte, listeDaten);
		final Transfer[] dragTransfers = new Transfer[] { TextTransfer
				.getInstance() };
		SichtObjektzusammenstellung.tabelleSystemObjekte.addDragSupport(ops,
				dragTransfers, zugZuhoererSystemObjekt);

		SichtObjektzusammenstellung.tabelleSystemObjekte
				.addSelectionChangedListener(new ISelectionChangedListener() {
					@Override
					public void selectionChanged(
							final SelectionChangedEvent event) {
						final IStructuredSelection selection = (IStructuredSelection) event
								.getSelection();

						/*
						 * Wenn mehr als 0 Elemente selektiert sind, wird der
						 * Lschen-Button aktiviert
						 */
						if (selection.size() > 0) {
							schaltflaecheLoeschen.setEnabled(true);
						}

						final Object firstElement = selection.getFirstElement();
						if (firstElement instanceof SystemObject) {
							final SystemObject systemObject = (SystemObject) firstElement;

							final List<SystemObject> liste = new LinkedList<SystemObject>();
							final Iterator<SystemObject> it = selection
									.iterator();
							while (it.hasNext()) {
								final SystemObject so = it.next();
								liste.add(so);
							}

							ObjektSpeicher.getInstanz().setSystemObjekte(liste);

							textFeldBeschreibung.setText(systemObject.getInfo()
									.getShortInfo());
						} else {
							textFeldBeschreibung.setText("");
						}
					}

				});

		// getSite().setSelectionProvider(_tabelleSystemObjekte);

	}

	/**
	 * Erzeugt den Bereich compositeObjekteRechts.
	 * 
	 */
	private void erzeugeCompositeObjekteRechts() {
		final RowLayout rowLayout1 = new RowLayout();
		rowLayout1.type = org.eclipse.swt.SWT.VERTICAL;
		rowLayout1.fill = true;
		rowLayout1.justify = false;
		final GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.BEGINNING;
		gridData3.grabExcessVerticalSpace = true;
		gridData3.verticalAlignment = GridData.FILL;
		compositeObjekteRechts = new Composite(gruppeObjekte, SWT.NONE);
		compositeObjekteRechts.setLayoutData(gridData3);
		compositeObjekteRechts.setLayout(rowLayout1);

		/*
		 * Hinzufgen
		 */
		schaltflaecheHinzufuegen = new Button(compositeObjekteRechts, SWT.NONE);
		schaltflaecheHinzufuegen.setText("Hinzufgen");
		schaltflaecheHinzufuegen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				hinzufuegenObjekt();
			}
		});

		if (!preLader.istInitialesLadenAbgeschlossen()) {
			schaltflaecheHinzufuegen.setEnabled(false);
		}

		/*
		 * Ist keine DaV-Verbindung verfgbar, so wird der Knopf zum hinzufgen
		 * von SystemObjekten deaktiviert.
		 */
		if (!RahmenwerkService.getService().getRahmenWerk().isOnline()) {
			schaltflaecheHinzufuegen.setEnabled(false);
		}

		/*
		 * Lschen
		 */
		schaltflaecheLoeschen = new Button(compositeObjekteRechts, SWT.NONE);
		schaltflaecheLoeschen.setText("Lschen");
		schaltflaecheLoeschen.setEnabled(false);
		schaltflaecheLoeschen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				entferneSelektierteObjekte();
			}
		});

		erzeugeCompositeObjekteRechtsUnten();
	}

	/**
	 * Erzeugt den Bereich compositeObjekteRechtsUnten.
	 * 
	 */
	private void erzeugeCompositeObjekteRechtsUnten() {
		final RowLayout rowLayout2 = new RowLayout();
		rowLayout2.fill = false;
		rowLayout2.justify = true;
		compositeObjekteRechtsUnten = new Composite(compositeObjekteRechts,
				SWT.NONE);
		compositeObjekteRechtsUnten.setLayout(rowLayout2);

		/*
		 * Hoch
		 */
		schaltflaecheHoch = new Button(compositeObjekteRechtsUnten, SWT.ARROW
				| SWT.UP);
		schaltflaecheHoch.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {

				final IStructuredSelection selection = (IStructuredSelection) SichtObjektzusammenstellung.tabelleSystemObjekte
						.getSelection();
				final Object firstElement = selection.getFirstElement();

				if (firstElement instanceof SystemObject) {
					final SystemObject systemObject = (SystemObject) firstElement;
					final int altePosition = listeDaten.indexOf(systemObject);
					if (altePosition > 0) {
						final int neuePosition = altePosition - 1;
						listeDaten.remove(systemObject);
						listeDaten.add(neuePosition, systemObject);
						// _lvListe.setInput(_listeDaten);
						SichtObjektzusammenstellung.tabelleSystemObjekte
								.setInput(listeDaten);
						// _lvListe.getList().setSelection(neuePosition);
					}
				}
			}

		});

		/*
		 * Runter
		 */
		schaltflaecheRunter = new Button(compositeObjekteRechtsUnten, SWT.ARROW
				| SWT.DOWN);
		schaltflaecheRunter.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {

				final IStructuredSelection selection = (IStructuredSelection) SichtObjektzusammenstellung.tabelleSystemObjekte
						.getSelection();
				final Object firstElement = selection.getFirstElement();

				if (firstElement instanceof SystemObject) {
					final SystemObject systemObject = (SystemObject) firstElement;
					final int altePosition = listeDaten.indexOf(systemObject);
					if (altePosition < (listeDaten.size() - 1)) {
						final int neuePosition = altePosition + 1;
						listeDaten.remove(systemObject);
						listeDaten.add(neuePosition, systemObject);
						SichtObjektzusammenstellung.tabelleSystemObjekte
								.setInput(listeDaten);
					}
				}
			}

		});

	}

	/**
	 * Erzeugt den Bereich compositeKontrolle.
	 * 
	 */
	private void erzeugeCompositeKontrolle() {
		final RowLayout rowLayout3 = new RowLayout();
		rowLayout3.fill = true;
		rowLayout3.type = org.eclipse.swt.SWT.HORIZONTAL;
		rowLayout3.wrap = true;
		rowLayout3.pack = false;
		rowLayout3.justify = true;
		final GridData gridData5 = new GridData();
		gridData5.horizontalAlignment = GridData.FILL;
		gridData5.grabExcessHorizontalSpace = true;
		gridData5.verticalAlignment = GridData.CENTER;
		compositeKontrolle = new Composite(oben, SWT.NONE);
		compositeKontrolle.setLayoutData(gridData5);
		compositeKontrolle.setLayout(rowLayout3);

		/*
		 * Button OK
		 */
		schaltflaecheOK = new Button(compositeKontrolle, SWT.NONE);
		schaltflaecheOK.setText("OK");

		schaltflaecheOK.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				benachrichtigeObjektzusammenstellungZuhoerer();
				versteckeSicht();
			}
		});

		/*
		 * Button Abbrechen
		 */
		schaltflaecheAbbrechen = new Button(compositeKontrolle, SWT.NONE);
		schaltflaecheAbbrechen.setText("Abbrechen");
		schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				versteckeSicht();
			}
		});

		/*
		 * Button Hilfe
		 */
		schaltflaecheHilfe = new Button(compositeKontrolle, SWT.NONE);
		schaltflaecheHilfe.setText("Hilfe");
		schaltflaecheHilfe.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDown(final MouseEvent e) {
				/*
				 * Nur bei mittlerer oder rechter Maustaste werden die Objekte
				 * erzeugt.
				 */
				if (e.button > 1) {

					final ArrayList<SystemObject> list = new ArrayList<SystemObject>();

					SystemObject o = connection.getDataModel().getObject(
							"mq.a10.0000");
					list.add(o);
					o = connection.getDataModel().getObject("mq.a10.0001");
					list.add(o);
					o = connection.getDataModel().getObject("mq.a10.0002");
					list.add(o);
					o = connection.getDataModel().getObject("mq.a10.0003");
					list.add(o);

					SichtObjektzusammenstellung.tabelleSystemObjekte
							.setInput(list);
				}
			}
		});

		schaltflaecheHilfe.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				/*
				 * Hilfeaufruf - nicht entfernene!
				 */
				final String hilfeKontextId = RwCompatActivator.PLUGIN_ID
						+ ".objektzusammenstellung";
				PlatformUI.getWorkbench().getHelpSystem()
						.displayHelp(hilfeKontextId);
			}

		});

	}

	/**
	 * Fgt einen {@link IObjektzusammenstellungZuhoerer} zur Liste der Zuhrer
	 * hinzu, wenn dieser noch nicht i.d. Liste vorhanden ist.
	 * 
	 * @param zuhoerer
	 *            Zuhrer, der zur Liste der Zuhrer hinzugefgt werden soll.
	 */
	public void hinzufuegenObjektzusammenstellungZuhoerer(
			final IObjektzusammenstellungZuhoerer zuhoerer) {
		if (!SichtObjektzusammenstellung.zuhoererObjektzusammenstellung
				.contains(zuhoerer)) {
			SichtObjektzusammenstellung.zuhoererObjektzusammenstellung
					.add(zuhoerer);
		} else {
			SichtObjektzusammenstellung.LOGGER
					.error(getClass().getName()
							+ "::hinzufuegenObjektzusammenstellungZuhoerer \n\t=> Der bergebene Zuhrer ist bereits in der Liste der Zuhrer vorhanden. Er wird nicht erneut hinzugefgt!");

		}
	}

	/**
	 * Entfernt einen {@link IObjektzusammenstellungZuhoerer} von der Liste der
	 * Zuhrer, der nicht mehr ber die aktuelle Objektzusammenstellung
	 * informiert werden mcht.
	 * 
	 * @param zuhoerer
	 *            Zu entfernender Zuhrer.
	 */
	public void entferneObjektzusammenstellungZuhoerer(
			final IObjektzusammenstellungZuhoerer zuhoerer) {
		if (SichtObjektzusammenstellung.zuhoererObjektzusammenstellung
				.contains(zuhoerer)) {
			SichtObjektzusammenstellung.zuhoererObjektzusammenstellung
					.remove(zuhoerer);
		} else {
			SichtObjektzusammenstellung.LOGGER
					.error(getClass().getName()
							+ "::hinzufuegenObjektzusammenstellungZuhoerer \n\t=>Der angegebenen Zuhrer existiert nicht in der Liste der Zuhrer und kann somit auch nicht entfernt werden. Abbruch.");

		}
	}

	/**
	 * Benachrichtigt alle Zuhrer ber die Fertigstellung einer neuen
	 * Objektzusammenstellung.
	 * 
	 */
	private void benachrichtigeObjektzusammenstellungZuhoerer() {
		if (SichtObjektzusammenstellung.zuhoererObjektzusammenstellung.size() > 0) {
			// SystemObjektliste kopieren
			final List<SystemObject> kopie = new LinkedList<SystemObject>();
			final Iterator<SystemObject> itKopieren = listeDaten.iterator();
			while (itKopieren.hasNext()) {
				kopie.add(itKopieren.next());
			}

			final ObjektzusammenstellungEreignis ereignis = new ObjektzusammenstellungEreignis(
					kopie);

			final LinkedList<IObjektzusammenstellungZuhoerer> kopieZuhoererObjektZusammenstellung = new LinkedList<IObjektzusammenstellungZuhoerer>();
			for (final IObjektzusammenstellungZuhoerer zuhoerer : SichtObjektzusammenstellung.zuhoererObjektzusammenstellung) {
				kopieZuhoererObjektZusammenstellung.add(zuhoerer);
			}

			final Iterator<IObjektzusammenstellungZuhoerer> itZu = kopieZuhoererObjektZusammenstellung
					.iterator();
			while (itZu.hasNext()) {
				final IObjektzusammenstellungZuhoerer zuhoerer = itZu.next();
				zuhoerer.systemObjekteAusgewaehlt(ereignis);
			}

		}
	}

	private void versteckeSicht() {
		getViewSite().getWorkbenchWindow().getActivePage().hideView(this);
	}

	/**
	 * {@link IStructuredContentProvider} fr die Tabelle zur Darstellung der
	 * zusammengestellten Objekte.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class TabelleInhaltsVersorger implements IStructuredContentProvider {
		@Override
		public Object[] getElements(final Object inputElement) {
			Object[] systemObjekte = null;

			if (inputElement instanceof List) {
				final List<?> tempList = (List<?>) inputElement;

				final List<SystemObject> tempListeSystemObjekte = new LinkedList<SystemObject>();

				final Iterator<?> it = tempList.iterator();
				while (it.hasNext()) {
					final Object tempObject = it.next();
					if (tempObject instanceof SystemObject) {
						final SystemObject tempSystemObject = (SystemObject) tempObject;
						tempListeSystemObjekte.add(tempSystemObject);
					}
				}

				systemObjekte = new Object[tempListeSystemObjekte.size()];

				int i = -1;
				final Iterator<SystemObject> itTempSo = tempListeSystemObjekte
						.iterator();
				while (itTempSo.hasNext()) {
					i++;
					systemObjekte[i] = itTempSo.next();
				}

			} else {
				SichtObjektzusammenstellung.LOGGER.fine(getClass().getName()
						+ "::getElements => KEINE Eingabe!");
				systemObjekte = new Object[0];
			}

			return systemObjekte;
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput,
				final Object newInput) {
			// Input wird hier nicht beachtet?
		}

	}

	/**
	 * {@link ITableLabelProvider} fr die Elemente der Tabelle zur
	 * Objektzusammenstellung.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class TabelleDarstellungsVersorger extends LabelProvider
			implements ITableLabelProvider {

		@Override
		public Image getColumnImage(final Object element, final int columnIndex) {
			return null;
		}

		@Override
		public String getColumnText(final Object object, final int columnIndex) {
			String returnString;
			if (object instanceof SystemObject) {
				final SystemObject systemObject = (SystemObject) object;

				// ID
				if (columnIndex == 0) {
					String tempName = "";
					if ((systemObject.getName() != null)
							&& (!systemObject.getName().isEmpty())) {
						tempName = tempName + systemObject.getName() + " ";
					}
					if ((systemObject.getPid() != null)
							&& (!systemObject.getPid().isEmpty())) {
						tempName = tempName + "(" + systemObject.getPid()
								+ ") ";
					}
					if (systemObject.getId() > 0) {
						tempName = tempName + "["
								+ String.valueOf(systemObject.getId()) + "]";
					}

					returnString = tempName;
					// returnString = systemObject.getPid();

				} else if (columnIndex == 1) {
					// Bezeichnung
					returnString = systemObject.getPidOrNameOrId();
				} else {
					// Sonst
					returnString = "Fr diese Spalte findet keine Versorgung statt.";
				}

			} else {
				returnString = "Nicht untersttztes Element vom Typ "
						+ object.getClass().getName() + ".";
			}

			return returnString;
		}
	}

	@Override
	public void dispose() {

		/*
		 * Zuhrer auf die DaV-Verbindung entfernen
		 */
		if (SichtObjektzusammenstellung.davListener != null) {
			RahmenwerkService
					.getService()
					.getRahmenWerk()
					.removeDavVerbindungsListener(
							SichtObjektzusammenstellung.davListener);
		}

		/*
		 * Zuhrer auf den Pre-Lader entfernen
		 */
		if (preLaderListener != null) {
			preLader.entfernenListenerPreLader(preLaderListener);
		}

		/*
		 * Zuhrer auf den Berechtigungs-Zuhoerer entfernen
		 */
		if (berechtigungsZuhoererErweitert != null) {
			// XXX Rechtezuhrer entfernen
			// DaVKommunikation.getInstanz().entferneOberflaechenBenutzerZuhoerer(
			// _berechtigungsZuhoererErweitert);
			SichtObjektzusammenstellung.LOGGER
					.warning("Implementierung fehlt noch");
		}

		ObjektSpeicher.getInstanz().getSystemObjekte().clear();

		super.dispose();
	}

	@Override
	public void init(final IViewSite site, final IMemento memento)
			throws PartInitException {
		this.sichtMemento = memento;
		init(site);
	}

	@Override
	public void saveState(final IMemento memento) {
		if (connection != null) {
			final TableItem[] items = SichtObjektzusammenstellung.tabelleSystemObjekte
					.getTable().getItems();
			int anzahlSystemObjekte = items.length;

			/*
			 * Sollen mehr SystemObjekte als durch XmlMemento mglich,
			 * gespeichert werden? Nein, wir beschrnken uns dann auf die ersten
			 * 10000.
			 */
			if (anzahlSystemObjekte > SichtObjektzusammenstellung.MAX_SIZE_XML_MEMENTO) {
				anzahlSystemObjekte = SichtObjektzusammenstellung.MAX_SIZE_XML_MEMENTO;
			}
			memento.putInteger("anzahlSystemObjekte", anzahlSystemObjekte);

			for (int i = 0; i < anzahlSystemObjekte; i++) {
				memento.putString(
						"id" + i,
						""
								+ ((SystemObject) SichtObjektzusammenstellung.tabelleSystemObjekte
										.getElementAt(i)).getId());
			}
		} else {
			memento.putInteger("anzahlSystemObjekte",
					systemObjekteMemento.size());
			int i = 0;
			final Iterator<Long> it = systemObjekteMemento.iterator();
			while (it.hasNext()) {
				final Long id = it.next();
				memento.putString("id" + i, "" + id);
				i++;
				if (i > SichtObjektzusammenstellung.MAX_SIZE_XML_MEMENTO) {
					break;
				}
			}
		}

		if (secondaryID != null) {
			memento.putString("secondaryID", secondaryID);
		}

	}

	/**
	 * Stellt aus einem Memento die View mit den aktuellen Daten wieder her. Ist
	 * keine Verbindung zum DaV vorhanden, so ist dies nicht mglich.
	 */
	private void restoreState() {
		if (sichtMemento != null) {
			final LinkedList<SystemObject> objs = new LinkedList<SystemObject>();

			DataModel dataModel = null;
			if (connection != null) {
				dataModel = connection.getDataModel();
			}

			final Integer anzahl = sichtMemento
					.getInteger("anzahlSystemObjekte");
			if ((anzahl != null) && (anzahl > 0)) {

				for (int i = 0; i < anzahl; i++) {
					final String string = sichtMemento.getString("id" + i);
					if (string != null) {
						try {
							final long id = Long.valueOf(string);

							if (dataModel != null) {
								final SystemObject object = dataModel
										.getObject(id);
								objs.add(object);
							} else {
								systemObjekteMemento.add(id);
							}
						} catch (final NumberFormatException nfe) {
							SichtObjektzusammenstellung.LOGGER.warning(nfe
									.getLocalizedMessage());
						}
					}
				}
			}

			if (objs.size() > 0) {
				setListeDaten(objs);
			}

			/*
			 * SecondaryID
			 */
			secondaryID = sichtMemento.getString("secondaryID");
		}
	}

	/**
	 * Liefert die secondary ID dieser View.
	 * 
	 * @return Secondary ID dieser View
	 */
	public String getSecondaryId() {
		return secondaryID;
	}

	/**
	 * Setzt die secondary ID dieser View.
	 * 
	 * @param secId
	 *            Secondary ID dieser View
	 */
	protected void setSecondaryId(final String secId) {
		this.secondaryID = secId;
	}

	public static TableViewer getTabelleSystemObjekte() {
		return SichtObjektzusammenstellung.tabelleSystemObjekte;
	}

} // @jve:decl-index=0:visual-constraint="10,10,610,284"
