/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.FilterAusdruck;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.TabellenFilter;
import de.bsvrz.buv.rw.compatibility.internal.RwCompatActivator;

public class DialogFilterVerwaltung extends Dialog {
	private Shell shell;
	private TableViewer tableViewer;

	/**
	 * Speichert den {@link XmlFilterVerwaltung}.
	 */
	private XmlFilterVerwaltung xmlFilterVerwaltung;

	/**
	 * Zeigt an, ob die XmlFilterVerwaltung bearbeitet wurde oder nicht.
	 */
	private boolean bearbeitet;

	/**
	 * Speichert die {@link TabellenDatenStruktur}, auf die sich die Filter der
	 * {@link XmlFilterVerwaltung} beziehen sollen.
	 */
	private TabellenDatenStruktur tds;

	public DialogFilterVerwaltung(final Shell parent) {
		super(parent, 0);

	}

	public XmlFilterVerwaltung open(
			final XmlFilterVerwaltung neueXmlFilterVerwaltung,
			final TabellenDatenStruktur newTds) {
		final Shell parent = getParent();

		if (neueXmlFilterVerwaltung != null) {
			this.xmlFilterVerwaltung = neueXmlFilterVerwaltung.clone();
		} else {
			this.xmlFilterVerwaltung = new XmlFilterVerwaltung();
		}

		this.tds = newTds;

		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL
				| SWT.RESIZE);
		shell.setText("Filter Verwaltung");
		shell.setImage(RwCompatActivator.getDefault().getImage(
				"icons/sonstiges/text_editor_16x16.png"));

		erzeugeDarstellung();

		if (this.xmlFilterVerwaltung != null) {
			tableViewer.setInput(this.xmlFilterVerwaltung);
		}

		shell.open();
		final Display display = parent.getDisplay();

		/*
		 * Positionieren des Dialogs bei der Position des Mauszeigers
		 */
		final Rectangle bounds = shell.getBounds();
		final Point cursorLocation = Display.getDefault().getCursorLocation();
		final Rectangle newBounds = new Rectangle(cursorLocation.x,
				cursorLocation.y, bounds.width, bounds.height);
		shell.setBounds(newBounds);

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		if (bearbeitet) {
			return xmlFilterVerwaltung;
		}
		return null;
	}

	/**
	 * This method initializes sShell.
	 */
	private void erzeugeDarstellung() {
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;

		erzeugeGruppeFilterUebersicht();
		shell.setLayout(gridLayout);
		erzeugeBereichKontrolle();
		shell.setSize(new Point(400, 250));
	}

	/**
	 * This method initializes _gruppeFilterUebersicht.
	 * 
	 */
	private void erzeugeGruppeFilterUebersicht() {
		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		final Group gruppeFilterUebersicht = new Group(shell, SWT.NONE);
		gruppeFilterUebersicht.setLayout(new GridLayout());
		gruppeFilterUebersicht.setLayoutData(gridData);
		gruppeFilterUebersicht.setText("Filter bersicht:");
		final Table tabelleFilter = new Table(gruppeFilterUebersicht,
				SWT.FULL_SELECTION | SWT.BORDER);
		tabelleFilter.setHeaderVisible(true);
		tabelleFilter.setLayoutData(gridData2);
		tabelleFilter.setLinesVisible(true);
		tableViewer = new TableViewer(tabelleFilter);

		/*
		 * Spalten erzeugen
		 */
		final TableColumn tcIndex = new TableColumn(tabelleFilter, SWT.CENTER);
		tcIndex.setText("Id:");
		tcIndex.setWidth(35);

		final TableColumn tcBeschreibung = new TableColumn(tabelleFilter,
				SWT.CENTER);
		tcBeschreibung.setText("Beschreibung:");
		tcBeschreibung.setWidth(200);

		/*
		 * ContentProvider
		 */
		final ContentProvider cp = new ContentProvider();
		tableViewer.setContentProvider(cp);

		/*
		 * LabelProvider
		 */
		final DialogFilterLabelProvider lp = new DialogFilterLabelProvider();
		tableViewer.setLabelProvider(lp);
	}

	/**
	 * Legt einen neuen Filter an und fgt ihn in die Darstellung ein.
	 */
	private void aktionAnlegen() {
		final DialogFilterEditor dialogFilterEditor = new DialogFilterEditor(
				shell);
		final TabellenFilter tabellenFilter = tds.new TabellenFilter(
				tds.getAnzahlAttribute(), tds);
		dialogFilterEditor.oeffne(tabellenFilter);

		/*
		 * Namen fr den Filter vergeben
		 */
		final InputDialog eingabeFilterName = new InputDialog(PlatformUI
				.getWorkbench().getDisplay().getActiveShell(),
				"Filternamen eingeben:",
				"Bitte geben Sie einen Namen fr den Filter ein!", "leer", null);

		if (eingabeFilterName.open() == Window.OK) {
			final String filterName = eingabeFilterName.getValue();
			if ((filterName != null) && (filterName.length() > 0)) {
				/*
				 * ElementFilter erzeugen
				 */
				final ElementFilter filter = new ElementFilter(
						xmlFilterVerwaltung
								.bestimmeNaechsteFreieIdElementFilter(),
						filterName);

				/*
				 * Regulre Ausdrcke erzeugen und zum ElementFilte hinzufgen
				 */
				final Iterator<FilterAusdruck> it = tabellenFilter
						.getFilterAusdruecke().iterator();
				while (it.hasNext()) {
					final FilterAusdruck filterAusdruck = it.next();
					final int spalte = filterAusdruck.getSpalte();
					final String bezeichnung = filterAusdruck.getBezeichnung();
					final String regExString = filterAusdruck.getFilter();

					final ElementRegEx elementRegEx = new ElementRegEx(spalte,
							bezeichnung, regExString);
					filter.addRegEx(elementRegEx);
				}

				/*
				 * Filter zur Filterverwaltung hinzufgen
				 */
				xmlFilterVerwaltung.addElementFilter(filter);

				/*
				 * Darstellung updaten
				 */
				tableViewer.setInput(xmlFilterVerwaltung);
			}
		}
	}

	private void aktionDuplizieren() {
		final ISelection selection = tableViewer.getSelection();
		if (selection instanceof IStructuredSelection) {
			final IStructuredSelection iss = (IStructuredSelection) selection;
			final Object firstElement = iss.getFirstElement();

			if (firstElement != null) {
				if (firstElement instanceof ElementFilter) {
					final ElementFilter elementFilter = (ElementFilter) firstElement;

					final ElementFilter clone = elementFilter.clone();

					clone.setBezeichnung("Kopie von " + clone.getBezeichnung());
					clone.setId(xmlFilterVerwaltung
							.bestimmeNaechsteFreieIdElementFilter());

					xmlFilterVerwaltung.addElementFilter(clone);

					tableViewer.setInput(xmlFilterVerwaltung);

				}
			}
		}
	}

	private void aktionBearbeiten() {
		final ISelection selection = tableViewer.getSelection();
		if (selection instanceof IStructuredSelection) {
			final IStructuredSelection iss = (IStructuredSelection) selection;
			final Object firstElement = iss.getFirstElement();
			if ((firstElement != null)
					&& (firstElement instanceof ElementFilter)) {
				final ElementFilter elementFilterBearbeiten = (ElementFilter) firstElement;

				final DialogFilterEditor dialogFilterEditor = new DialogFilterEditor(
						shell);
				final TabellenFilter tabellenFilter = tds.new TabellenFilter(
						tds.getAnzahlAttribute(), tds);

				final Iterator<ElementRegEx> it = elementFilterBearbeiten
						.getRegex().iterator();
				while (it.hasNext()) {
					final ElementRegEx elementRegEx = it.next();
					final int id = elementRegEx.getId();
					final String regEx = elementRegEx.getRegEx();

					tabellenFilter.setFilterAusdruck(id,
							tds.new FilterAusdruck(id, regEx, tds));
				}

				/*
				 * Dialog zum Filterbearbeiten ffnen
				 */

				final TabellenFilter tabellenFilterBearbeitet = dialogFilterEditor
						.oeffne(tabellenFilter);
				if (tabellenFilterBearbeitet != null) {
					/*
					 * Regulre Ausdrcke erzeugen und zum ElementFilte
					 * hinzufgen
					 */
					final Iterator<FilterAusdruck> itRegEx = tabellenFilterBearbeitet
							.getFilterAusdruecke().iterator();
					while (itRegEx.hasNext()) {
						final FilterAusdruck filterAusdruck = itRegEx.next();
						final int spalte = filterAusdruck.getSpalte();
						final String bezeichnung = filterAusdruck
								.getBezeichnung();
						final String regExString = filterAusdruck.getFilter();

						final ElementRegEx elementRegEx = new ElementRegEx(
								spalte, bezeichnung, regExString);

						elementFilterBearbeiten.addRegEx(elementRegEx);
					}

					/*
					 * Filter zur Filterverwaltung hinzufgen
					 */
					xmlFilterVerwaltung
							.addElementFilter(elementFilterBearbeiten);

					/*
					 * Darstellung updaten
					 */
					tableViewer.setInput(xmlFilterVerwaltung);
				}

			}
		}
	}

	private void aktionLoeschen() {
		final ISelection selection = tableViewer.getSelection();
		if (selection instanceof IStructuredSelection) {
			final IStructuredSelection iss = (IStructuredSelection) selection;
			final Object firstElement = iss.getFirstElement();
			if ((firstElement != null)
					&& (firstElement instanceof ElementFilter)) {
				final ElementFilter elementFilterLoeschen = (ElementFilter) firstElement;
				xmlFilterVerwaltung.removeElementFilter(elementFilterLoeschen);
				tableViewer.setInput(xmlFilterVerwaltung);
			}
		}
	}

	private void aktionUmbenennen() {
		final ISelection selection = tableViewer.getSelection();
		if (selection instanceof IStructuredSelection) {
			final IStructuredSelection iss = (IStructuredSelection) selection;
			final Object firstElement = iss.getFirstElement();
			if ((firstElement != null)
					&& (firstElement instanceof ElementFilter)) {
				final ElementFilter elementUmbenennen = (ElementFilter) firstElement;

				/*
				 * Namen fr den Filter vergeben
				 */
				final InputDialog eingabeFilterName = new InputDialog(
						PlatformUI.getWorkbench().getDisplay().getActiveShell(),
						"Neuen Filternamen eingeben:",
						"Bitte geben Sie einen neuen Namen fr den Filter ein!",
						elementUmbenennen.getBezeichnung(), null);

				if (eingabeFilterName.open() == Window.OK) {
					final String value = eingabeFilterName.getValue();
					if (!elementUmbenennen.getBezeichnung().equals(value)) {
						elementUmbenennen.setBezeichnung(value);
						xmlFilterVerwaltung.addElementFilter(elementUmbenennen);
						tableViewer.setInput(xmlFilterVerwaltung);
					}
				}
			}
		}
	}

	private void aktionUebernehmen() {
		bearbeitet = true;
		aktionSchliessen();
	}

	/**
	 * Schliet den Dialog.
	 */
	private void aktionSchliessen() {
		shell.dispose();
	}

	/**
	 * This method initializes _bereichKontrolle.
	 * 
	 */
	private void erzeugeBereichKontrolle() {
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.BEGINNING;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.verticalAlignment = GridData.FILL;
		final RowLayout rowLayout = new RowLayout();
		rowLayout.type = org.eclipse.swt.SWT.VERTICAL;
		rowLayout.pack = false;
		final Composite bereichKontrolle = new Composite(shell, SWT.NONE);
		bereichKontrolle.setLayout(rowLayout);
		bereichKontrolle.setLayoutData(gridData1);

		final Button schaltflaecheUebernehmen = new Button(bereichKontrolle,
				SWT.NONE);
		schaltflaecheUebernehmen.setText("bernehmen");
		schaltflaecheUebernehmen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionUebernehmen();
			}
		});

		final Button schaltflaecheAnlegen = new Button(bereichKontrolle,
				SWT.NONE);
		schaltflaecheAnlegen.setText("Anlegen");
		schaltflaecheAnlegen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionAnlegen();
			}
		});

		final Button schaltflaecheDuplizieren = new Button(bereichKontrolle,
				SWT.NONE);
		schaltflaecheDuplizieren.setText("Duplizieren");
		schaltflaecheDuplizieren.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionDuplizieren();
			}
		});

		final Button schaltflaecheBearbeiten = new Button(bereichKontrolle,
				SWT.NONE);
		schaltflaecheBearbeiten.setText("Bearbeiten");
		schaltflaecheBearbeiten.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionBearbeiten();
			}
		});

		final Button schaltflaecheLoeschen = new Button(bereichKontrolle,
				SWT.NONE);
		schaltflaecheLoeschen.setText("Lschen");
		schaltflaecheLoeschen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionLoeschen();
			}
		});

		final Button schaltflaecheUmbenennen = new Button(bereichKontrolle,
				SWT.NONE);
		schaltflaecheUmbenennen.setText("Umbenennen");
		schaltflaecheUmbenennen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionUmbenennen();
			}
		});

		final Button schaltflaecheSchliessen = new Button(bereichKontrolle,
				SWT.NONE);
		schaltflaecheSchliessen.setText("Schlieen");
		schaltflaecheSchliessen.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionSchliessen();
			}

		});
	}

	private static class ContentProvider implements IStructuredContentProvider {

		@Override
		public Object[] getElements(final Object inputElement) {
			Object[] re = null;

			if (inputElement instanceof XmlFilterVerwaltung) {
				final XmlFilterVerwaltung filterVerwaltung = (XmlFilterVerwaltung) inputElement;
				final LinkedList<ElementFilter> listeElementFilter = filterVerwaltung
						.getElementFilter();
				re = new Object[listeElementFilter.size()];

				int i = 0;
				final Iterator<ElementFilter> it = listeElementFilter
						.iterator();
				while (it.hasNext()) {
					final ElementFilter elementFilter = it.next();
					re[i] = elementFilter;
					i++;
				}

			}

			return re;
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput,
				final Object newInput) {
			// Input wird global betrachtet?
		}

	}

	private static class DialogFilterLabelProvider extends LabelProvider
			implements ITableLabelProvider {

		@Override
		public Image getColumnImage(final Object element, final int columnIndex) {
			return null;
		}

		@Override
		public String getColumnText(final Object element, final int columnIndex) {
			String re = "";
			if (element instanceof ElementFilter) {
				final ElementFilter elementFilter = (ElementFilter) element;
				if (columnIndex == 0) {
					re = "" + elementFilter.getId();
				} else if (columnIndex == 1) {
					re = elementFilter.getBezeichnung();
				}
			}
			return re;
		}

		@Override
		public boolean isLabelProperty(final Object element,
				final String property) {
			return false;
		}
	}
}
