/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.util.LinkedList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.compatibility.internal.RwCompatActivator;

public class DialogSpaltenSortierungsReihenfolge extends Dialog {

	private Shell shell;
	private Composite compositeOben;
	private Composite compositeUnten;
	private Group gruppeSpalten;
	private Group gruppeSortierung;
	private Composite compositeKontrolle;
	private Button schaltflaecheOk;
	private Button schaltflaecheAbbrechen;
	private Button schaltflaecheHilfe;
	private Table tabelleSpalten;
	private Table tabelleSortierung;
	private TableViewer tableViewerSpalten;
	private TableViewer tableViewerSortierung;

	/**
	 * Flag zeigt an, ob die Spaltenreihenfolge gesetzt wurde.
	 */
	private boolean spaltenreihenfolgeGesetzt;

	/**
	 * Struktur der {@link TabellenDatenStruktur}.
	 */
	private TabellenDatenStruktur tabellenDatenStruktur;

	/**
	 * Reihenfolge der Spalten.
	 */
	private final LinkedList<SpaltenElement> spaltenSortierung = new LinkedList<SpaltenElement>();
	private Button schaltflaecheLoeschen;

	public DialogSpaltenSortierungsReihenfolge(final Shell parent) {
		super(parent, 0);
	}

	/**
	 * This method initializes compositeOben.
	 *
	 */
	private void createCompositeOben() {
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		compositeOben = new Composite(shell, SWT.NONE);
		createGruppeSpalten();
		compositeOben.setLayout(gridLayout);
		createGruppeSortierung();
		compositeOben.setLayoutData(gridData);
	}

	public boolean open(final TabellenDatenStruktur tds) {
		final Shell parent = getParent();
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);

		/*
		 * TDS setzen
		 */
		tabellenDatenStruktur = tds;

		erzeugeDialog();

		/*
		 * bersicht aller Spalten
		 */
		tableViewerSpalten
				.setInput(tabellenDatenStruktur.getAlleSpaltenTitel());

		sortierungAusTabelleDatenStrukturLaden();
		aktuelleSortierungInTabelleDarstellen();

		shell.setSize(500, 300);
		shell.open();
		final Display display = parent.getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		return spaltenreihenfolgeGesetzt;
	}

	protected void sortierungAusTabelleDatenStrukturLaden() {
		spaltenSortierung.clear();

		/*
		 * bersicht der aktuellen Sortierung
		 */
		final Sortierung[] sortierung = tabellenDatenStruktur.getSortierung();
		for (final Sortierung wert : sortierung) {
			final SpaltenElement spaltenTitel = tabellenDatenStruktur
					.getSpaltenTitel(wert.getSpalte());
			spaltenSortierung.add(spaltenTitel);
		}
	}

	/**
	 * Aktualisiert die Darstellung der aktuellen Sortierung.
	 */
	protected void aktuelleSortierungInTabelleDarstellen() {

		final SpaltenElement[] ar = SpaltenElement
				.spaltenElementListe2Array(spaltenSortierung);

		tableViewerSortierung.setInput(ar);
	}

	/**
	 * This method initializes compositeUnten.
	 *
	 */
	private void createCompositeUnten() {
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		compositeUnten = new Composite(shell, SWT.NONE);
		compositeUnten.setLayout(new GridLayout());
		createCompositeKontrolle();
		compositeUnten.setLayoutData(gridData1);
	}

	/**
	 * Liefert die Bearbeitete Sortierung von {@link SpaltenElement}en als
	 * Liste.
	 *
	 * @return Sortierung von {@link SpaltenElement}en als Liste
	 */
	protected LinkedList<SpaltenElement> getSortierung() {
		return spaltenSortierung;
	}

	protected TabellenDatenStruktur getTabellenDatenStruktur() {
		return tabellenDatenStruktur;
	}

	/**
	 * This method initializes gruppeSpalten.
	 *
	 */
	private void createGruppeSpalten() {
		final GridData gridData5 = new GridData();
		gridData5.horizontalAlignment = GridData.FILL;
		gridData5.grabExcessHorizontalSpace = true;
		gridData5.grabExcessVerticalSpace = true;
		gridData5.verticalAlignment = GridData.FILL;
		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		gruppeSpalten = new Group(compositeOben, SWT.NONE);
		gruppeSpalten.setLayout(new GridLayout());
		gruppeSpalten.setLayoutData(gridData2);
		gruppeSpalten.setText("Verfgbare Spalten:");
		tabelleSpalten = new Table(gruppeSpalten, SWT.NONE);
		tabelleSpalten.setHeaderVisible(true);
		tabelleSpalten.setLayoutData(gridData5);
		tabelleSpalten.setLinesVisible(true);
		tableViewerSpalten = new TableViewer(tabelleSpalten);

		final TableColumn tc = new TableColumn(tabelleSpalten, SWT.CENTER);
		tc.setText("Titel der Spalte:");
		tc.setWidth(200);

		final UebersichtContentProvider cp = new UebersichtContentProvider();
		tableViewerSpalten.setContentProvider(cp);
		final UebersichtLabelProvider lp = new UebersichtLabelProvider();
		tableViewerSpalten.setLabelProvider(lp);

		final int ops = DND.DROP_MOVE;
		final Transfer[] dragTransfers = new Transfer[] { TextTransfer
				.getInstance() };
		final DragSourceAdapterTabellenSpalten dragSourceAdapter = new DragSourceAdapterTabellenSpalten(
				tableViewerSpalten);

		tableViewerSpalten
				.addDragSupport(ops, dragTransfers, dragSourceAdapter);
	}

	/**
	 * This method initializes gruppeSortierung.
	 *
	 */
	private void createGruppeSortierung() {
		final GridData gridData6 = new GridData();
		gridData6.horizontalAlignment = GridData.FILL;
		gridData6.grabExcessHorizontalSpace = true;
		gridData6.grabExcessVerticalSpace = true;
		gridData6.verticalAlignment = GridData.FILL;
		final GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.grabExcessVerticalSpace = true;
		gridData3.verticalAlignment = GridData.FILL;
		gruppeSortierung = new Group(compositeOben, SWT.NONE);
		gruppeSortierung.setLayout(new GridLayout());
		gruppeSortierung.setLayoutData(gridData3);
		gruppeSortierung.setText("Aktuelle Sortierung:");
		tabelleSortierung = new Table(gruppeSortierung, SWT.FULL_SELECTION);
		tabelleSortierung.setHeaderVisible(true);
		tabelleSortierung.setLayoutData(gridData6);
		tabelleSortierung.setLinesVisible(true);
		tableViewerSortierung = new TableViewer(tabelleSortierung);

		getKontextMenu(tableViewerSortierung.getControl());

		final TableColumn tcSpaltenTitel = new TableColumn(tabelleSortierung,
				SWT.CENTER);
		tcSpaltenTitel.setText("Spaltentitel:");
		tcSpaltenTitel.setWidth(100);

		final TableColumn tcSortierung = new TableColumn(tabelleSortierung,
				SWT.CENTER);
		tcSortierung.setText("Sortierung");
		tcSortierung.setWidth(100);

		final UebersichtContentProvider cp = new UebersichtContentProvider();
		tableViewerSortierung.setContentProvider(cp);
		final UebersichtLabelProvider lp = new UebersichtLabelProvider();
		tableViewerSortierung.setLabelProvider(lp);

		final int ops = DND.DROP_MOVE;
		final ViewerDropAdapterSpaltenReihenfolge dropAdapter = new ViewerDropAdapterSpaltenReihenfolge(
				tableViewerSortierung, this);
		final Transfer[] dragTransfers = new Transfer[] { TextTransfer
				.getInstance() };
		tableViewerSortierung.addDropSupport(ops, dragTransfers, dropAdapter);

	}

	private void aktionSchliessen() {
		shell.setVisible(false);
		shell.dispose();
	}

	private void aktionOk() {
		// Rckgabewert des Dialogs erzeugen
		tabellenDatenStruktur.setSortierung(spaltenSortierung);
		spaltenreihenfolgeGesetzt = true;

		aktionSchliessen();
	}

	private void aktionLoescheAktuelleSortierung() {
		spaltenSortierung.clear();
		aktuelleSortierungInTabelleDarstellen();
	}

	private void aktionHilfe() {
		final String helpContextId = RwCompatActivator.PLUGIN_ID
				+ ".sortierDialog";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	/**
	 * This method initializes compositeKontrolle.
	 *
	 */
	private void createCompositeKontrolle() {
		final RowLayout rowLayout = new RowLayout();
		rowLayout.justify = false;
		rowLayout.pack = false;
		final GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.END;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.verticalAlignment = GridData.CENTER;
		compositeKontrolle = new Composite(compositeUnten, SWT.NONE);
		compositeKontrolle.setLayoutData(gridData4);
		compositeKontrolle.setLayout(rowLayout);
		schaltflaecheOk = new Button(compositeKontrolle, SWT.NONE);
		schaltflaecheOk.setText("OK");
		schaltflaecheOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionOk();
			}
		});

		schaltflaecheAbbrechen = new Button(compositeKontrolle, SWT.NONE);
		schaltflaecheAbbrechen.setText("Abbrechen");
		schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionSchliessen();
			}
		});

		schaltflaecheLoeschen = new Button(compositeKontrolle, SWT.NONE);
		schaltflaecheLoeschen.setText("Sortierung Loeschen");
		schaltflaecheLoeschen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionLoescheAktuelleSortierung();
			}
		});

		schaltflaecheHilfe = new Button(compositeKontrolle, SWT.NONE);
		schaltflaecheHilfe.setText("Hilfe");
		schaltflaecheHilfe.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionHilfe();
			}
		});

	}

	/**
	 * Erzeugt die Darstellung des Dialogs.
	 */
	private void erzeugeDialog() {
		shell.setText("Spaltensortierung");
		createCompositeOben();
		createCompositeUnten();

		shell.setLayout(new GridLayout());
		shell.setSize(new Point(400, 250));
	}

	public MenuManager getKontextMenu(final Control control) {
		final MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(final IMenuManager manager) {
				fuelleKontextMenu(manager);
			}
		});

		final Menu menu = menuMgr.createContextMenu(control);
		control.setMenu(menu);

		return menuMgr;
	}

	public void fuelleKontextMenu(final IMenuManager manager) {
		final Action aufsteigend = new Action() {
			@Override
			public void run() {
				setSortierungAufAktuellerSelektion(true);
				aktuelleSortierungInTabelleDarstellen();
			}
		};
		aufsteigend.setText("Aufsteigend");

		final Action absteigend = new Action() {
			@Override
			public void run() {
				setSortierungAufAktuellerSelektion(false);
				aktuelleSortierungInTabelleDarstellen();
			}
		};
		absteigend.setText("Absteigend");

		manager.add(aufsteigend);
		manager.add(absteigend);

		final ISelection selection = tableViewerSortierung.getSelection();
		if (selection instanceof IStructuredSelection) {
			final IStructuredSelection iss = (IStructuredSelection) selection;
			final Object firstElement = iss.getFirstElement();
			if (firstElement instanceof SpaltenElement) {
				final SpaltenElement spaltenElement = (SpaltenElement) firstElement;
				if (spaltenElement.istAufsteigend()) {
					aufsteigend.setChecked(true);
				} else {
					absteigend.setChecked(true);
				}
			}
		}

	}

	private void setSortierungAufAktuellerSelektion(final boolean aufsteigend) {
		final ISelection selection = tableViewerSortierung.getSelection();
		if (selection instanceof IStructuredSelection) {
			final IStructuredSelection iss = (IStructuredSelection) selection;
			final Object firstElement = iss.getFirstElement();
			if (firstElement instanceof SpaltenElement) {
				final SpaltenElement spaltenElement = (SpaltenElement) firstElement;
				spaltenElement.setAufsteigend(aufsteigend);
			}
		}
	}

	/**
	 * ContentProvider fr die bersicht der vorhandenen {@link SpaltenElement}
	 * e.
	 *
	 * @author HofmannJ
	 *
	 */
	private static class UebersichtContentProvider implements
			IStructuredContentProvider {

		@Override
		public Object[] getElements(final Object inputElement) {
			Object[] re = null;

			if (inputElement instanceof SpaltenElement[]) {
				final SpaltenElement[] spaltenElemente = (SpaltenElement[]) inputElement;
				re = spaltenElemente;
			}

			return re;
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput,
				final Object newInput) {
			// Input wird nicht von hier bernommen?
		}

	}

	/**
	 * LabelProvider fr die bersicht der vorhandenen {@link SpaltenElement}e.
	 *
	 * @author HofmannJ
	 *
	 */
	private static class UebersichtLabelProvider extends LabelProvider
			implements ITableLabelProvider {

		@Override
		public String getColumnText(final Object element, final int columnIndex) {
			String re = null;
			if (element instanceof SpaltenElement) {
				if (columnIndex == 0) {
					final SpaltenElement se = (SpaltenElement) element;
					re = se.toString();
				} else if (columnIndex == 1) {
					final SpaltenElement se = (SpaltenElement) element;
					if (se.istAufsteigend()) {
						re = "aufsteigend";
					} else {
						re = "absteigend";
					}
				}
			}
			return re;
		}

		@Override
		public boolean isLabelProperty(final Object element,
				final String property) {
			return false;
		}

		@Override
		public Image getColumnImage(final Object element, final int columnIndex) {
			return null;
		}
	}

}
