/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.util;

import de.bsvrz.buv.rw.basislib.internal.RahmenwerkService;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationTaskException;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.management.UserAdministration;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Utility-Klasse mit allgemeinen Funktionen im Zusammenhang mit dem Rahmenwerk.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RahmenwerkUtil.java 48448 2014-01-28 12:16:28Z peuker $
 */
public final class RahmenwerkUtil {

	private static final Debug LOGGER = Debug.getLogger();

	private RahmenwerkUtil() {
	}

	/**
	 * liefert das Systemobject, das den Nutzer mit dem bergebenen Name in der
	 * Datenverteilerkonfiguration reprsentiert. Wenn keine
	 * Datenverteilerverbindung besteht oder ein entsprechender Nutzer nicht
	 * existiert, wird <code>null</code> geliefert.
	 * 
	 * @param name
	 *            der Name des gesuchten Nutzers
	 * @return das SystemObject oder <code>null</code>
	 */
	public static SystemObject getBenutzer(final String name) {

		SystemObject benutzer = null;
		if (name != null) {
			final RahmenwerkService service = RahmenwerkService.getService();
			if (service != null) {
				final ClientDavInterface davVerbindung = service
						.getRahmenWerk().getDavVerbindung();

				if (davVerbindung != null) {
					for (final SystemObject element : davVerbindung
							.getDataModel().getType("typ.benutzer")
							.getElements()) {
						if (name.equals(element.getName())) {
							benutzer = element;
							break;
						}
					}
				}
			}
		}

		return benutzer;
	}

	/**
	 * ermittelt ob der angemeldete Benutzer Administratorrechte hat.
	 * 
	 * @return den Status
	 */
	public static boolean isUserAdmin() {

		final SystemObject user = RahmenwerkService.getService()
				.getRahmenWerk().getBenutzer();
		boolean result = false;

		if ((user != null)
				&& RahmenwerkService.getService().getRahmenWerk().isOnline()) {
			final ClientDavInterface connection = RahmenwerkService
					.getService().getRahmenWerk().getDavVerbindung();
			final DataModel model = connection.getDataModel();
			final UserAdministration userAdministration = model
					.getUserAdministration();

			final String benutzerName = user.getName();
			final String passwort = RahmenwerkService.getService()
					.getRahmenWerk().getPasswort();

			try {
				result = userAdministration.isUserAdmin(benutzerName, passwort,
						benutzerName);
			} catch (final ConfigurationTaskException e) {
				RahmenwerkUtil.LOGGER
						.warning(
								"Nutzer konnte nicht auf Adminrechte berprft werden!",
								e);
			}
		}

		return result;
	}
}
