package de.bsvrz.buv.rw.rw.berechtigungen;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.berechtigung.FunktionMitBerechtigung;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.BerechtigungsKey;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.BerechtigungsKlasse;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.Funktion;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.PluginMitBerechtigungen;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.WildcardBerechtigungsKlasse;
import de.bsvrz.buv.rw.rw.berechtigungen.editor.WildcardFunktion;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Reprstention des Berechtigungs-Parameter-Datensatzes aus dem
 * Datenverteiler-System.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungsParameterDatensatz.java 46953 2013-11-05 10:05:57Z peuker $
 */
public class BerechtigungsParameterDatensatz {

	/**
	 * ein Eintrag aus dem Parameterdatensatz fr die Ausgabe der Informationen
	 * zum Datenverteiler.
	 */
	private static class BerechtigungsEintrag {

		private final String klasse;
		private final Set<String> eintragsFunktionen = new HashSet<String>();
		private final Boolean status;

		public BerechtigungsEintrag(final String klasse, final String funktion,
				final Boolean status) {
			this.klasse = klasse;
			this.eintragsFunktionen.add(funktion);
			this.status = status;
		}

		public void addFunktion(final String id) {
			eintragsFunktionen.add(id);
		}

		@Override
		public String toString() {
			return klasse + ": " + status + " --> " + eintragsFunktionen.size();
		}
	}

	private final WildcardBerechtigungsKlasse wildcardBerechtigungsKlasse = new WildcardBerechtigungsKlasse();
	private final WildcardFunktion wildcardFunktion = new WildcardFunktion();

	private final SortedMap<String, BerechtigungsKlasse> berechtigungsKlassen = new TreeMap<String, BerechtigungsKlasse>();
	private final Map<String, Funktion> funktionen = new HashMap<String, Funktion>();

	private final Map<String, PluginMitBerechtigungen> plugins = new HashMap<String, PluginMitBerechtigungen>();
	private final PluginMitBerechtigungen nichtZugeordneteFunktionen = new PluginMitBerechtigungen();

	private final Map<BerechtigungsKey, Boolean> matrix = new HashMap<BerechtigungsKey, Boolean>();

	/**
	 * Konstruktor, erzeugt Reprsentation des Datensatzes fr die bergebene
	 * Berechtigungsverwaltung mit den angegebenen Daten.
	 * 
	 * @param verwaltung
	 *            die Berechtigungsverwaltung
	 * @param datenSatz
	 *            die Daten
	 */
	public BerechtigungsParameterDatensatz(final BerechtigungenImpl verwaltung,
			final Data datenSatz) {

		/* initialisiere Berechtigungsklassen aus der Berechtigungsverwaltung. */
		for (final SystemObject klasse : verwaltung.getBerechtigungsKlassen()) {
			berechtigungsKlassen.put(klasse.getPid(), new BerechtigungsKlasse(
					klasse));
		}

		/* initialisiere Funktionen uas der Berechtigungsverwaltung. */
		for (final FunktionMitBerechtigung funktion : verwaltung
				.getFunktionen()) {
			funktionen.put(funktion.getId(), new Funktion(funktion));
		}

		if (datenSatz != null) {
			final Array array = datenSatz.getArray("OberflchenBerechtigung");
			for (int idx = 0; idx < array.getLength(); idx++) {

				final boolean freigabe = "Freigabe".equals(array.getItem(idx)
						.getUnscaledValue("Berechtigung").getState().getName());

				final SortedSet<BerechtigungsKlasse> itemKlassen = new TreeSet<BerechtigungsKlasse>();
				final Array klassen = array.getItem(idx).getArray(
						"Berechtigungsklasse");
				for (int itemIdx = 0; itemIdx < klassen.getLength(); itemIdx++) {
					final String id = klassen.getTextValue(itemIdx).getText();
					BerechtigungsKlasse newKlasse;
					if ("*".equals(id)) {
						newKlasse = wildcardBerechtigungsKlasse;
					} else {
						newKlasse = berechtigungsKlassen.get(id);
					}
					if (newKlasse == null) {
						newKlasse = new BerechtigungsKlasse(id);
						berechtigungsKlassen.put(id, newKlasse);
					}
					itemKlassen.add(newKlasse);
				}

				final Array funktionsArray = array.getItem(idx).getArray(
						"Funktion");
				for (int funcIdx = 0; funcIdx < funktionsArray.getLength(); funcIdx++) {
					final String id = funktionsArray.getTextValue(funcIdx)
							.getText();
					Funktion funktion = funktionen.get(id);
					if ("*".equals(id)) {
						funktion = wildcardFunktion;
					} else {
						funktion = funktionen.get(id);
					}
					if (funktion == null) {
						funktion = new Funktion(id);
						funktionen.put(funktion.getId(), funktion);
					}

					for (final BerechtigungsKlasse klasse : itemKlassen) {
						final BerechtigungsKey key = new BerechtigungsKey(
								klasse, funktion);
						final Boolean status = matrix.get(key);
						if (!Boolean.FALSE.equals(status)) {
							matrix.put(key, freigabe);
						}
					}
				}
			}
		}

		/*
		 * ermitteln der zugehrigen Plugins und Kategorien fr die
		 * Prsentation.
		 */
		for (final Funktion funktion : funktionen.values()) {
			PluginMitBerechtigungen plugin = nichtZugeordneteFunktionen;
			final FunktionMitBerechtigung funktionMitBerechtigung = verwaltung
					.getFunktion(funktion.getId());
			if (funktionMitBerechtigung != null) {
				final String pluginId = funktionMitBerechtigung.getPluginId();

				if (pluginId != null) {
					plugin = plugins.get(pluginId);
					if (plugin == null) {
						plugin = new PluginMitBerechtigungen(pluginId);
						plugins.put(plugin.getId(), plugin);
					}
				}
			}

			plugin.addFunktion(funktion);
		}

	}

	/**
	 * liefert die Stellvertreterklasse fr die Freigabe/Sperrung einer Funktion
	 * fr alle Berechtigungsklassen.
	 * 
	 * @return die Klasse
	 */
	public WildcardBerechtigungsKlasse getWildcardBerechtigungsKlasse() {
		return wildcardBerechtigungsKlasse;
	}

	/**
	 * liefert die Stellvertreterfunktion fr die Freigabe/Sperrung aller
	 * Funktion fr eine Berechtigungsklasse.
	 * 
	 * @return die Funktion
	 */
	public WildcardFunktion getWildcardFunktion() {
		return wildcardFunktion;
	}

	/**
	 * liefert die Liste aller enthaltenen Berechtigungsklassen.
	 * 
	 * @return die Liste der Klassen
	 */
	public Collection<BerechtigungsKlasse> getBerechtigungsKlassen() {
		return berechtigungsKlassen.values();
	}

	/**
	 * liefert das Stellvertreter-Plugin fr alle nicht zuordenbaren Funktionen.
	 * 
	 * @return das Plugin
	 */
	public PluginMitBerechtigungen getNichtZugeordneteFunktionen() {
		return nichtZugeordneteFunktionen;
	}

	/**
	 * liefert die Liste aller Plugins mit Funktionen.
	 * 
	 * @return die Liste der Funktionen
	 */
	public Collection<PluginMitBerechtigungen> getPlugins() {
		return plugins.values();
	}

	/**
	 * liefert den Freigabezustand fr die bergebene Kombination aus
	 * Berechtigungsklasse und Funktion. Wenn die Matrix keine konkrete
	 * Definition fr den Schlssel enthlt, wird der Wert <code>null</code>
	 * geliefert.
	 * 
	 * @param key
	 *            der Schlssel
	 * @return der Frigabewert oder <code>null</code>
	 */
	public Boolean getFreigabeStatus(final BerechtigungsKey key) {
		return matrix.get(key);
	}

	/**
	 * liefert den Freigabezustand fr die bergebene Kombination aus
	 * Berechtigungsklasse und Funktion.
	 * 
	 * Wird <code>null</code> bergeben, wird der entsprechende Eintrag
	 * gelscht.
	 * 
	 * @param key
	 *            der Schlssel
	 * @param status
	 *            der Freigabewert oder <code>null</code>, um einen Eintrag aus
	 *            der Matrix zu entfernen
	 */
	public void setFreigabeStatus(final BerechtigungsKey key,
			final Boolean status) {
		if (status == null) {
			matrix.remove(key);
		} else {
			matrix.put(key, status);
		}

	}

	/**
	 * ermittelt ber die Vererbungsregeln, ob fr den bergebenen Schlssel
	 * eine Freigabe besteht.
	 * 
	 * @param key
	 *            der Schlssel
	 * @return der Freigabezustand
	 */
	public boolean isFreigegeben(final BerechtigungsKey key) {
		BerechtigungsKey testKey = key;
		Boolean result = matrix.get(testKey);
		if (result == null) {
			testKey = new BerechtigungsKey(key.getKlasse(), wildcardFunktion);
			result = matrix.get(testKey);
		}

		if (result == null) {
			testKey = new BerechtigungsKey(wildcardBerechtigungsKlasse,
					key.getFunktion());
			result = matrix.get(testKey);
		}

		if (result == null) {
			testKey = new BerechtigungsKey(wildcardBerechtigungsKlasse,
					wildcardFunktion);
			result = matrix.get(testKey);
		}

		if (result == null) {
			result = false;
		}

		return result;
	}

	/**
	 * erzeugt einen Datensatz aus der aktuellen Matrix, der an den
	 * Datenverteiler versendet werden kann.
	 * 
	 * @return den Datensatz oder <code>null</code>, wenn keine Onlineverbindung
	 *         besteht
	 */
	public ResultData erzeugeDatenSatz() {
		final Rahmenwerk rahmenwerk = BerechtigungenImpl.getInstance()
				.getRahmenwerk();
		if (!rahmenwerk.isOnline()) {
			return null;
		}

		final List<BerechtigungsEintrag> eintrage = new ArrayList<BerechtigungsEintrag>();

		/* globalen Eintrag ermitteln. */
		Boolean status = matrix.get(new BerechtigungsKey(
				wildcardBerechtigungsKlasse, getWildcardFunktion()));
		if (status != null) {
			eintrage.add(new BerechtigungsEintrag("*", "*", status));
		}

		for (final Funktion funktion : funktionen.values()) {
			final BerechtigungsKey key = new BerechtigungsKey(
					wildcardBerechtigungsKlasse, funktion);

			status = matrix.get(key);
			if (status != null) {
				eintrage.add(new BerechtigungsEintrag("*", funktion.getId(),
						status));
			}
		}

		for (final BerechtigungsKlasse klasse : berechtigungsKlassen.values()) {
			BerechtigungsEintrag frei = null;
			BerechtigungsEintrag gesperrt = null;

			status = matrix.get(new BerechtigungsKey(klasse, wildcardFunktion));
			if (status != null) {
				if (status) {
					frei = new BerechtigungsEintrag(klasse.getId(), "*", true);
				} else {
					gesperrt = new BerechtigungsEintrag(klasse.getId(), "*",
							false);
				}
			}

			for (final Funktion funktion : funktionen.values()) {
				final BerechtigungsKey key = new BerechtigungsKey(klasse,
						funktion);

				status = matrix.get(key);
				if (status != null) {
					if (status) {
						if (frei == null) {
							frei = new BerechtigungsEintrag(klasse.getId(),
									funktion.getId(), true);
						} else {
							frei.addFunktion(funktion.getId());
						}
					} else {
						if (gesperrt == null) {
							gesperrt = new BerechtigungsEintrag(klasse.getId(),
									funktion.getId(), false);
						} else {
							gesperrt.addFunktion(funktion.getId());
						}
					}
				}
			}

			if (frei != null) {
				eintrage.add(frei);
			}

			if (gesperrt != null) {
				eintrage.add(gesperrt);
			}
		}

		final ClientDavInterface davVerbindung = rahmenwerk.getDavVerbindung();
		final AttributeGroup attributeGroup = davVerbindung.getDataModel()
				.getAttributeGroup("atg.oberflchenBerechtigung");
		final Aspect aspect = davVerbindung.getDataModel().getAspect(
				"asp.parameterVorgabe");

		final Data daten = davVerbindung.createData(attributeGroup);
		final Array array = daten.getArray("OberflchenBerechtigung");
		array.setLength(eintrage.size());

		int idx = 0;
		for (final BerechtigungsEintrag eintrag : eintrage) {
			final Array klassenArray = array.getItem(idx).getArray(
					"Berechtigungsklasse");
			klassenArray.setLength(1);
			klassenArray.getTextValue(0).setText(eintrag.klasse);

			final Array funktionenArray = array.getItem(idx).getArray(
					"Funktion");
			funktionenArray.setLength(eintrag.eintragsFunktionen.size());

			int funcIdx = 0;
			for (final String funcId : eintrag.eintragsFunktionen) {
				funktionenArray.getTextValue(funcIdx++).setText(funcId);
			}

			array.getItem(idx).getUnscaledValue("Berechtigung")
					.setText(eintrag.status ? "Freigabe" : "Sperrung");

			array.getItem(idx).getArray("ObjektMenge").setLength(0);
			array.getItem(idx).getUnscaledValue("AusnahmeDefinition").set(0);
			array.getItem(idx).getUnscaledValue("Ebene").set(0);

			idx++;
		}

		return new ResultData(rahmenwerk.getOberflaechenObject(),
				new DataDescription(attributeGroup, aspect), 0, daten);
	}

	/**
	 * entfernt die angegebene Funktion aus der Matrix.
	 * 
	 * @param funktion
	 *            die Funktion
	 */
	public void entferneFunktion(final Funktion funktion) {
		if (funktion.isAvailable()) {
			throw new IllegalStateException(
					"Eine verfgbare Funktion kann nicht aus der Matrix entfernt werden!");
		}
		synchronized (matrix) {
			final Collection<BerechtigungsKey> removeKeys = new ArrayList<BerechtigungsKey>();
			for (final BerechtigungsKey key : matrix.keySet()) {
				if (key.getFunktion().equals(funktion)) {
					removeKeys.add(key);
				}
			}
			for (final BerechtigungsKey key : removeKeys) {
				matrix.remove(key);
			}
		}

		synchronized (funktionen) {
			funktionen.remove(funktion.getId());
		}

		for (final PluginMitBerechtigungen plugin : plugins.values()) {
			plugin.removeFunktion(funktion.getId());
		}

		nichtZugeordneteFunktionen.removeFunktion(funktion.getId());
	}

	/**
	 * entfernt die angegebene Berechtigungsklasse aus der Matrix.
	 * 
	 * @param klasse
	 *            die Berechtigungsklasse
	 */
	public void entferneKlasse(final BerechtigungsKlasse klasse) {
		if (klasse.isAvailable()) {
			throw new IllegalStateException(
					"Eine verfgbare Berechtigungsklasse kann nicht aus der Matrix entfernt werden!");
		}
		synchronized (matrix) {
			final Collection<BerechtigungsKey> removeKeys = new ArrayList<BerechtigungsKey>();
			for (final BerechtigungsKey key : matrix.keySet()) {
				if (key.getKlasse().equals(klasse)) {
					removeKeys.add(key);
				}
			}
			for (final BerechtigungsKey key : removeKeys) {
				matrix.remove(key);
			}
		}

		synchronized (berechtigungsKlassen) {
			berechtigungsKlassen.remove(klasse.getId());
		}
	}
}
