package de.bsvrz.buv.rw.rw.berechtigungen;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.activities.IActivityManager;
import org.eclipse.ui.activities.IWorkbenchActivitySupport;

import de.bsvrz.buv.rw.basislib.berechtigung.BerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.Berechtigungen;
import de.bsvrz.buv.rw.basislib.berechtigung.IBerechtigungListener;
import de.bsvrz.buv.rw.basislib.util.RahmenwerkUtil;

/**
 * Aktivierung der UI-Komponenten in Abhngigkeit von den Berechtigungen
 * (Berechtigungseditor).
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: WorkbenchActivation.java 48341 2014-01-24 14:17:33Z peuker $
 */
public class WorkbenchActivation {

	/** Listener der Berechtigungsnderungen fr die Aktivierung verarbeitet. */
	private final class BerechtigungsListener implements IBerechtigungListener {
		@Override
		public void sperrung(final BerechtigungEreignis e) {
			aktiviereEditor();
		}

		@Override
		public void freigabe(final BerechtigungEreignis e) {
			sperrung(e);
		}
	}

	private IWorkbench workbench;
	private Berechtigungen berechtigungen;
	private final IBerechtigungListener berechtigungsListener = new BerechtigungsListener();

	/**
	 * bindet die bergebene Workbenchkomponente an den Service.
	 * 
	 * @param newWorkbench
	 *            die Workbench
	 */
	protected void bindWorkbench(final IWorkbench newWorkbench) {
		workbench = newWorkbench;
	}

	/**
	 * trennt die bergebene Workbenchkomponente vom Service.
	 * 
	 * @param oldWorkbench
	 *            die Workbench
	 */
	protected void unbindWorkbench(final IWorkbench oldWorkbench) {
		workbench = null;
	}

	/**
	 * bindet die bergebene Berechtigungsverwaltung an den Service.
	 * 
	 * @param newBerechtigungen
	 *            die Berechtigungsverwaltung
	 */
	protected void bindBerechtigungen(final Berechtigungen newBerechtigungen) {
		berechtigungen = newBerechtigungen;
	}

	/**
	 * trennt die bergebene Berechtigungsverwaltung vom Service.
	 * 
	 * @param oldBerechtigungen
	 *            die Berechtigungsverwaltung
	 */
	protected void unbindBerechtigungen(final Berechtigungen oldBerechtigungen) {
		berechtigungen.removeListener(berechtigungsListener);
		berechtigungen = null;
	}

	/** aktiviert den Service und den Editor entsprechend den Berechtigungen. */
	protected void activate() {
		berechtigungen.addListenerForFunktion(berechtigungsListener,
				BerechtigungenBerechtigungsFunktion.BERECHTIGUNG_AENDERN
						.getFunktionMitBerechtigung());

		aktiviereEditor();
	}

	private void aktiviereEditor() {
		final IWorkbenchActivitySupport activitySupport = workbench
				.getActivitySupport();
		final IActivityManager activityManager = activitySupport
				.getActivityManager();

		final String id = "de.bsvrz.buv.rw.rw.berechtigungen.activity.editor";
		if (activityManager.getActivity(id).isDefined()) {
			final Set<String> enabled = new HashSet<String>(
					activityManager.getEnabledActivityIds());
			if (BerechtigungenBerechtigungsFunktion.BERECHTIGUNG_AENDERN
					.isFreigegeben() || RahmenwerkUtil.isUserAdmin()) {
				enabled.add(id);
			} else {
				enabled.remove(id);
			}
			activitySupport.setEnabledActivityIds(enabled);
		}
	}
}
