package de.bsvrz.buv.rw.rw.dav;

/**
 * Speichert den Benutzer, der sich im Offlinemodus angemeldet hat.
 * 
 * @author HofmannJ
 * 
 */
public final class BenutzerOffline {
	private static BenutzerOffline instance;

	/**
	 * Benutzername.
	 */
	private String name;

	/**
	 * Passwort.
	 */
	private String passwort;

	/**
	 * Adresse des jeweiligen DaVs.
	 */
	private String ip;

	/**
	 * Port.
	 */
	private int tcpPort;

	private BenutzerOffline() {
	}

	public static synchronized BenutzerOffline getInstance() {
		if (BenutzerOffline.instance == null) {
			BenutzerOffline.instance = new BenutzerOffline();
		}

		return BenutzerOffline.instance;
	}

	/**
	 * Setzt den offline Benutzer.
	 * 
	 * @param newName
	 *            Name des Benutzers
	 * @param newPasswort
	 *            Passwort des Benutzers
	 * @param newIp
	 *            IP-Adresse des DaV
	 * @param port
	 *            Port-Nummer des DaV
	 */
	public void setOfflineBenutzer(final String newName,
			final String newPasswort, final String newIp, final int port) {
		this.name = newName;
		this.passwort = newPasswort;
		this.ip = newIp;
		this.tcpPort = port;
	}

	/**
	 * Liefert den Namen.
	 * 
	 * @return Name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Liefert das Passwort.
	 * 
	 * @return Passwort.
	 */
	public String getPasswort() {
		return passwort;
	}

	/**
	 * Liefert die IP bzw. die Domain.
	 * 
	 * @return IP bzw. die Domain.
	 */
	public String getIp() {
		return ip;
	}

	/**
	 * Liefert den Port.
	 * 
	 * @return port.
	 */
	public int getTcpPort() {
		return tcpPort;
	}

	/**
	 * Setzt die Daten zurck, nur von {@link BenutzerAngemeldet} aus
	 * zugnglich.
	 */
	protected void zuruecksetzen() {
		this.name = null;
		this.passwort = null;
		this.ip = null;
		this.tcpPort = -1;
	}
}
