/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.einstellungen;

import java.io.IOException;

import de.bsvrz.buv.rw.basislib.einstellungen.Einstellung;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsFactory;

/**
 * Implementierung einer {@link Einstellung}.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: EinstellungImpl.java 45420 2013-08-09 12:12:12Z peuker $
 */
class EinstellungImpl implements Einstellung {

	private final String type;
	private final String text;
	private Object object;

	EinstellungImpl(final String type, final String text) {
		this.type = type;
		this.text = text;
		if (String.class.getName().equals(type)) {
			this.object = text;
		}
	}

	@Override
	public String getText() {
		return text;
	}

	@Override
	public Object getObject() throws IOException {
		if (object == null) {
			final EinstellungsFactory factory = EinstellungenImpl.getInstance()
					.getFactory(type);
			if (factory == null) {
				throw new IOException("Keine Factory fr den Typ " + type
						+ " registriert!");
			}
			object = factory.deserialisiere(text);
		}
		return object;
	}
}
