/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.einstellungen;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsAdresse;
import de.bsvrz.buv.rw.basislib.einstellungen.SpeicherKey;

/**
 * XML-Handler zum Einlesen der Daten eines Einstellungsspeichers.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: EinstellungenXmlHandler.java 46953 2013-11-05 10:05:57Z peuker $
 */
class EinstellungenXmlHandler extends DefaultHandler {

	private String currentId;
	private final StringBuilder currentData = new StringBuilder();
	private final StringBuilder teilData = new StringBuilder();
	private final EinstellungenSpeichermanager speichermanager;
	private final SpeicherKey speicherKey;

	public EinstellungenXmlHandler(
			final EinstellungenSpeichermanager speichermanager,
			final SpeicherKey speicherKey) {
		this.speichermanager = speichermanager;
		this.speicherKey = speicherKey;
	}

	@Override
	public void startElement(final String uri, final String localName,
			final String qName, final Attributes attributes)
			throws SAXException {

		if ("parameter".equals(qName)) {
			currentData.setLength(0);
			currentId = attributes.getValue("id");
		} else if ("teilParameter".equals(qName)) {
			teilData.setLength(0);
		}
	}

	@Override
	public void endElement(final String uri, final String localName,
			final String qName) throws SAXException {

		if ("parameter".equals(qName)) {
			saveData();
			currentId = null;
			currentData.setLength(0);
		} else if ("teilParameter".equals(qName)) {
			currentData
					.append(DavConnector.entpackeString(teilData.toString()));
			teilData.setLength(0);
		}
	}

	private void saveData() {
		if (currentData.length() > 0) {
			final EinstellungsAdresse adresse = new EinstellungsAdresse(null,
					currentId, speicherKey.getOwnerType(),
					speicherKey.getPid(), speicherKey.getLocation());
			speichermanager.setEinstellung(adresse,
					DavConnector.entpackeString(currentData.toString()));
		}
	}

	@Override
	public void characters(final char[] ch, final int start, final int length)
			throws SAXException {
		final String strValue = new String(ch, start, length);
		currentData.append(strValue);
	}
}
