/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */
package de.bsvrz.buv.rw.rw.internal;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Tray;
import org.eclipse.swt.widgets.TrayItem;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.dav.daf.main.ClientDavInterface;

/**
 * Visualisierung des Zustandes der Datenverteilerverbindung des Rahmenwerks im
 * System-Tray des Betriebssystems.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RahmenwerkTray.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class RahmenwerkTray {

	private final TrayItem trayItem;
	private final Image grayImage;
	private final Image greenImage;

	/**
	 * Konstruktor, erzeugt ein entsprechendes Anzeigeelement im bergebenen
	 * Systemtray.
	 * 
	 * @param parent
	 *            der Systemtray
	 */
	public RahmenwerkTray(final Tray parent) {

		grayImage = createTrayImage("icons/tray/traygray.png");
		greenImage = createTrayImage("icons/tray/traygreen.png");

		trayItem = new TrayItem(parent, SWT.NONE);
		updateTrayDisplay();

		final DavVerbindungsListener listener = new DavVerbindungsListener() {
			@Override
			public void verbindungHergestellt(final DavVerbindungsEvent event) {
				updateStatus();
			}

			@Override
			public void verbindungGetrennt(final DavVerbindungsEvent event) {
				updateStatus();
			}

			@Override
			public boolean verbindungHalten(final DavVerbindungsEvent event) {
				return false;
			}
		};
		RahmenwerkImpl.getInstanz().addDavVerbindungsListener(listener);

	}

	private Image createTrayImage(final String imagePath) {
		final ImageDescriptor descriptor = AbstractUIPlugin
				.imageDescriptorFromPlugin(RahmenwerkActivator.PLUGIN_ID,
						imagePath);
		if (descriptor == null) {
			return null;
		}
		return descriptor.createImage();
	}

	private void updateStatus() {
		if (trayItem != null) {
			trayItem.getDisplay().asyncExec(new Runnable() {
				@Override
				public void run() {
					updateTrayDisplay();
				}
			});
		}
	}

	/** gibt Image-ressourcen der Klasse frei. */
	public void dispose() {
		if (grayImage != null) {
			grayImage.dispose();
		}
		if (greenImage != null) {
			greenImage.dispose();
		}
		trayItem.dispose();
	}

	private void updateTrayDisplay() {
		if ((trayItem != null) && (!trayItem.isDisposed())) {
			final Rahmenwerk rahmenwerk = RahmenwerkImpl.getInstanz();
			ClientDavInterface dav = null;
			if (rahmenwerk.isOnline()) {
				dav = rahmenwerk.getDavVerbindung();
			}
			Image image;
			final String tooltip;
			if (dav == null) {
				tooltip = "Keine Datenverteilerverbindung";
				image = grayImage;

			} else {
				tooltip = "Verbindung aufgebaut\n"
						+ dav.getLocalUser()
						+ "\non "
						+ dav.getClientDavParameters()
								.getDavCommunicationAddress()
						+ ":"
						+ dav.getClientDavParameters()
								.getDavCommunicationSubAddress();
				image = greenImage;
			}
			trayItem.setToolTipText(tooltip);
			trayItem.setImage(image);
		}
	}
}
