/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.menu;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungLocation;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungOwnerType;
import de.bsvrz.buv.rw.basislib.einstellungen.SpeicherKey;
import de.bsvrz.buv.rw.basislib.util.RahmenwerkUtil;
import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Diese Klasse liefert einen Dialog, mit dessen Hilfe sich Speicherort und das
 * Subjekt, fr welches gespeichert werden soll, bestimmen lassen.<br>
 * <br>
 * Mgliche Speicherorte sind:
 * <ul>
 * <li>Benutzereinstellungen Netzweit</li>
 * <li>Benutzereinstellungen Lokal</li>
 * <li>Allgemeine Einstellungen Netzweit</li>
 * <li>Allgemeine Einstellungen Lokal</li>
 * </ul>
 * <br>
 * Mgliche Subjekte sind:
 * <ul>
 * <li>Benutzer angmeldet</li>
 * <li>Benutzer auswhlbar</li>
 * <li>Allgemein</li>
 * </ul>
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: LadeSpeicherDialogMenu.java 50269 2014-04-29 13:12:35Z peuker $
 */
public class LadeSpeicherDialogMenu extends TitleAreaDialog {

	/** interner Listener zur Aktualisierung der Darstellung des Dialogs. */
	private class SpeicherInfoUpdater extends SelectionAdapter implements
			ISelectionChangedListener {
		@Override
		public void widgetSelected(final SelectionEvent e) {
			updateSpeicherInfo();
		}

		@Override
		public void selectionChanged(final SelectionChangedEvent event) {
			updateSpeicherInfo();
		}
	}

	private final boolean online;

	private SystemObject benutzer;

	private final int style;

	/**
	 * Konstruktor, erzeugt einen Dialog mit der bergebenen bergeordneten
	 * Shell und dem angegebenen Stil.
	 * 
	 * Mgliche Werte fr den Stil sind:
	 * <ul>
	 * <li>{@value SWT#SAVE} zum Sichern einer Mendefinition und</li>
	 * <li>{@value SWT#OPEN} zum Laden einer Mendefinition</li>
	 * </ul>
	 * 
	 * @param parentShell
	 *            die bergeordnete Shell
	 * @param style
	 *            der Stil des Dialogs
	 */
	public LadeSpeicherDialogMenu(final Shell parentShell, final int style) {
		super(parentShell);
		this.style = style;

		final Rahmenwerk rahmenwerk = RahmenwerkImpl.getInstanz();
		online = rahmenwerk.isOnline();
		if (online) {
			benutzer = rahmenwerk.getBenutzer();
			speicherKey = new SpeicherKey(EinstellungOwnerType.BENUTZER,
					EinstellungLocation.NETZWERKWEIT, benutzer.getPid());
		} else {
			benutzer = null;
			speicherKey = new SpeicherKey(EinstellungOwnerType.SYSTEM,
					EinstellungLocation.LOKAL, null);
		}

	}

	private void validateSpeicherKey() {
		boolean valid = false;
		if (userAdmin || (speicherKey.getOwnerType() == EinstellungOwnerType.BENUTZER)) {
			valid = true;
		} else if (speicherKey.getLocation() == EinstellungLocation.NETZWERKWEIT) {
			valid = (style & SWT.OPEN) != 0;
		}

		getButton(IDialogConstants.OK_ID).setEnabled(valid);
		if (valid) {
			setErrorMessage(null);
		} else {
			setErrorMessage("Sie sind nicht berechtigt, Einstellungen an dieser Position zu speichern");
		}
	}

	private SpeicherKey speicherKey;

	private Group locationGroup;

	private Group ownerGroup;

	private ComboViewer benutzerAuswahl;

	private boolean userAdmin;

	@Override
	protected void cancelPressed() {
		speicherKey = null;
		super.cancelPressed();
	}

	@Override
	protected Control createDialogArea(final Composite parent) {

		userAdmin = RahmenwerkUtil.isUserAdmin();

		if ((style & SWT.SAVE) != 0) {
			getShell().setText("Sichern");
			setTitle("Menstruktur sichern");
			setMessage("Whlen Sie das entsprechende Ziel aus!");
		} else {
			getShell().setText("Laden");
			setTitle("Menstruktur laden");
			setMessage("Whlen Sie die entsprechende Quelle aus!");
		}

		setTitleImage(RahmenwerkActivator.getDefault().getImage(
				"icons/wizban/bitctrl_wizard.png"));

		final Composite panel = new Composite(parent, SWT.NONE);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(panel);
		panel.setLayout(new GridLayout(2, false));

		erzeugeGruppeAuswahlOrt(panel);
		erzeugeGruppeAuswahlSubjekt(panel);

		return panel;
	}

	@Override
	protected Control createButtonBar(final Composite parent) {
		final Control result = super.createButtonBar(parent);
		validateSpeicherKey();
		return result;
	}

	/**
	 * Erzeugt die Gruppe zur Auswahl von Speicherorten.
	 * 
	 */
	private void erzeugeGruppeAuswahlOrt(final Composite parent) {
		final GridData gridDataAuswahl = new GridData();
		gridDataAuswahl.horizontalAlignment = GridData.FILL;
		gridDataAuswahl.grabExcessHorizontalSpace = true;
		gridDataAuswahl.grabExcessVerticalSpace = true;
		gridDataAuswahl.verticalAlignment = GridData.FILL;

		locationGroup = new Group(parent, SWT.NONE);
		locationGroup.setText("Auswahl Speicherort:");
		locationGroup.setLayout(new GridLayout());
		locationGroup.setLayoutData(GridDataFactory.fillDefaults()
				.grab(true, true).create());

		/* Allgemeine Einstellungen. */
		Button button = new Button(locationGroup, SWT.RADIO);
		button.setText("Netzweit");
		button.setEnabled(online);
		button.setSelection(speicherKey.getLocation() == EinstellungLocation.NETZWERKWEIT);
		button.setData(EinstellungLocation.NETZWERKWEIT);
		button.addSelectionListener(new SpeicherInfoUpdater());

		/* Lokale Einstellungen */
		button = new Button(locationGroup, SWT.RADIO);
		button.setText("Lokal");
		button.setSelection(speicherKey.getLocation() == EinstellungLocation.LOKAL);
		button.setData(EinstellungLocation.LOKAL);
		button.addSelectionListener(new SpeicherInfoUpdater());
	}

	/**
	 * Erzeugt die Gruppe zur Auswahl von Subjekten.
	 * 
	 * @param panel
	 */
	private void erzeugeGruppeAuswahlSubjekt(final Composite panel) {
		final GridData gridDataAuswahlSubjekt = new GridData();
		gridDataAuswahlSubjekt.horizontalAlignment = GridData.FILL;
		gridDataAuswahlSubjekt.grabExcessHorizontalSpace = true;
		gridDataAuswahlSubjekt.grabExcessVerticalSpace = true;
		gridDataAuswahlSubjekt.verticalAlignment = GridData.FILL;

		ownerGroup = new Group(panel, SWT.NONE);
		ownerGroup.setLayout(new GridLayout());
		ownerGroup.setText("Auswahl Subjekt:");
		ownerGroup.setLayoutData(gridDataAuswahlSubjekt);

		/* Allgemeine Einstellungen */
		Button button = new Button(ownerGroup, SWT.RADIO);
		button.setText("Allgemein");
		button.setData(EinstellungOwnerType.SYSTEM);
		button.setSelection(speicherKey.getOwnerType() == EinstellungOwnerType.SYSTEM);
		button.addSelectionListener(new SpeicherInfoUpdater());

		/* Benutzereinstellungen */
		button = new Button(ownerGroup, SWT.RADIO);
		button.setText("Benutzer");
		button.setEnabled(online);
		button.setData(EinstellungOwnerType.BENUTZER);
		button.setSelection(speicherKey.getOwnerType() == EinstellungOwnerType.BENUTZER);
		button.addSelectionListener(new SpeicherInfoUpdater());

		final Label label = new Label(panel, SWT.NONE);
		label.setText("Benutzerauswahl:");
		label.setEnabled(online);

		benutzerAuswahl = new ComboViewer(panel);
		benutzerAuswahl.setContentProvider(new ArrayContentProvider());
		if (benutzer != null) {
			final Collection<SystemObject> benutzerListe = new ArrayList<SystemObject>();
			if (userAdmin) {
				benutzerListe.addAll(RahmenwerkImpl.getInstanz()
						.getDavVerbindung().getDataModel()
						.getType("typ.benutzer").getElements());
			} else {
				benutzerListe.add(benutzer);
			}
			benutzerAuswahl.setInput(benutzerListe.toArray());
			benutzerAuswahl.setSelection(new StructuredSelection(benutzer));
		} else {
			benutzerAuswahl.getControl().setEnabled(false);
		}
		benutzerAuswahl.addSelectionChangedListener(new SpeicherInfoUpdater());
	}

	/**
	 * Zur Bestimmung, ob der Dialog zum Laden oder Speichern verwendet wird.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum EnumDialogSpeichernLaden {
		/**
		 * Dialog zum Speichern.
		 */
		speichern,

		/**
		 * Dialog zum Laden.
		 */
		laden
	}

	/**
	 * liefert den im Dialog ausgewhlten Speicherkey, ber den Benutzer,
	 * Location und gegebenenfalls die PID des Benutzers bereitgestellt werden.
	 * 
	 * @return den Key
	 */
	public SpeicherKey getSpeicherKey() {
		return speicherKey;
	}

	private void updateSpeicherInfo() {

		EinstellungLocation location = EinstellungLocation.LOKAL;
		for (final Control control : locationGroup.getChildren()) {
			final Object data = control.getData();
			if ((data instanceof EinstellungLocation)
					&& (control instanceof Button)) {
				if (((Button) control).getSelection()) {
					location = (EinstellungLocation) data;
				}
			}
		}

		EinstellungOwnerType ownerType = EinstellungOwnerType.SYSTEM;
		for (final Control control : ownerGroup.getChildren()) {
			final Object data = control.getData();
			if ((data instanceof EinstellungOwnerType)
					&& (control instanceof Button)) {
				if (((Button) control).getSelection()) {
					ownerType = (EinstellungOwnerType) data;
				}
			}
		}

		String benutzerPid = null;
		if (ownerType == EinstellungOwnerType.BENUTZER) {
			final Object element = ((IStructuredSelection) benutzerAuswahl
					.getSelection()).getFirstElement();
			if (element instanceof SystemObject) {
				benutzerPid = ((SystemObject) element).getPid();
			}
		}

		this.speicherKey = new SpeicherKey(ownerType, location, benutzerPid);
		validateSpeicherKey();
	}
}
