/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.menu;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IContributionManager;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.services.IServiceLocator;

import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;

/**
 * Diese Klasse generiert Men und Toolbars des Rahmenwerks.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: MenuGenerator.java 46953 2013-11-05 10:05:57Z peuker $
 */
public class MenuGenerator {

	/**
	 * interne Klasse zur Untermen-Darstellung in der Symbolleiste des
	 * Rahmenwerks.
	 */
	private static class ToolBarSubMenue extends ContributionItem {

		private final ToolbarMenueAction action;

		public ToolBarSubMenue(final ToolbarMenueAction action) {
			super(action.getId());
			this.action = action;
		}

		@Override
		public void fill(final ToolBar parent, final int index) {
			final ToolItem toolItem = new ToolItem(parent, SWT.FLAT
					| SWT.DROP_DOWN);
			toolItem.setText(action.getText());
			toolItem.setWidth(0);
			toolItem.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(final SelectionEvent e) {
					final Event event = new Event();
					event.widget = e.widget;
					action.runWithEvent(event);
				}
			});
		}

		@Override
		public void fill(final Composite parent) {
			final String text = action.getText();
			final Button button = new Button(parent, SWT.PUSH);
			button.setText(text);
			button.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(final SelectionEvent e) {
					final Event event = new Event();
					event.widget = e.widget;
					action.runWithEvent(event);
				}
			});
		}
	}

	/**
	 * interne Aktion zum ffnen eines Untermens in der Symbolleiste des
	 * Rahmenwerks.
	 */
	private class ToolbarMenueAction extends Action {

		public ToolbarMenueAction(final IServiceLocator serviceLocator,
				final SubMenue element) {
			super(element.getVisibleName(), IAction.AS_DROP_DOWN_MENU);

			setMenuCreator(new IMenuCreator() {
				@Override
				public Menu getMenu(final Menu parent) {
					return null;
				}

				@Override
				public Menu getMenu(final Control parent) {
					final MenuManager manager = new MenuManager();
					for (final MenueElement child : element.getElemente()) {
						generiereMenuRekursiv(serviceLocator, manager, child);
					}

					final Menu menu = new Menu(parent);
					for (final IContributionItem item : manager.getItems()) {
						item.fill(menu, -1);
					}
					return menu;
				}

				@Override
				public void dispose() {
					// keine Ressourcen freizugeben
				}
			});
		}

		@Override
		public void runWithEvent(final Event event) {
			final Widget item = event.widget;
			final IMenuCreator mc = getMenuCreator();
			if (mc != null) {
				final Composite parentControl = ((Control) item).getParent();
				final Rectangle bounds = ((Control) item).getBounds();
				final Menu m = mc.getMenu(parentControl);
				if (m != null) {
					final Point point = parentControl.toDisplay(new Point(
							bounds.x, bounds.y));

					if (parentControl instanceof ToolBar) {
						m.setLocation(point.x, point.y + bounds.height);
					} else {
						m.setLocation(point.x, point.y);
					}
					m.setVisible(true);
				}
			}
		}
	}

	/** Menmanager, in dem das Men aufgebaut wird. */
	private final IContributionManager contributionManager;

	/** die Definition der Menleiste. */
	private final AbstractMenueLeiste menueLeiste;

	/**
	 * Konstruktor.
	 * 
	 * erzeugt einen {@link MenuGenerator} fr die bergebene Menleiste.
	 * 
	 * @param contributionManager
	 *            Manager, der die Anzeige des Mens oder der Toolbar realisiert
	 * @param menueLeiste
	 *            die Definition der darzustellenden Menleiste
	 */
	public MenuGenerator(final IContributionManager contributionManager,
			final AbstractMenueLeiste menueLeiste) {
		this.contributionManager = contributionManager;
		this.menueLeiste = menueLeiste;
	}

	/**
	 * erzeugt das Men fr den bergebenen ServiceLocator (in der Regel das
	 * Workbecnhfenster).
	 * 
	 * @param serviceLocator
	 *            der ServiceLocator
	 */
	public final void generiereMenu(final IServiceLocator serviceLocator) {

		final RwToolbarElementManagerImpl elementManager = RahmenwerkImpl
				.getInstanz().getRwToolBarManager();

		contributionManager.removeAll();
		for (final MenueElement child : menueLeiste.getElemente()) {
			if (contributionManager instanceof IMenuManager) {
				generiereMenuRekursiv(serviceLocator, contributionManager,
						child);
			} else if (contributionManager instanceof IToolBarManager) {
				generiereToolbarRekursiv(serviceLocator, contributionManager,
						child);
				elementManager.getSymbolLeisteManager().update(true);
			} else if (contributionManager instanceof IStatusLineManager) {
				generiereStatusLeisteRekursiv(serviceLocator,
						contributionManager, child);
			}
		}
		contributionManager.update(true);
	}

	private void generiereMenuRekursiv(final IServiceLocator serviceLocator,
			final IContributionManager parentManager, final MenueElement element) {
		// Item?
		if (element instanceof MenueAction) {
			final String tempId = ((MenueAction) element).getActionId();
			final RwToolbarElementManagerImpl elementManager = RahmenwerkImpl
					.getInstanz().getRwToolBarManager();

			// DescriptorAction
			final RwToolbarElement descriptorAction = elementManager
					.getToolBarElement(tempId);
			if (descriptorAction != null) {
				try {
					parentManager.add(descriptorAction.createContributionItem(
							serviceLocator, (MenueAction) element));
				} catch (final CoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

		} else if (element instanceof MenueSeparator) {
			final Separator separator = new Separator();
			parentManager.add(separator);
		} else if (element instanceof SubMenue) {
			final String tempSubmenuName = ((SubMenue) element).getName();
			final MenuManager tempSubmenu = new MenuManager(tempSubmenuName);
			parentManager.add(tempSubmenu);

			for (final MenueElement child : ((SubMenue) element).getElemente()) {
				generiereMenuRekursiv(serviceLocator, tempSubmenu, child);
			}
		}
	}

	private void generiereToolbarRekursiv(final IServiceLocator serviceLocator,
			final IContributionManager parentManager, final MenueElement element) {
		// Item?
		if (element instanceof MenueAction) {
			final String tempId = ((MenueAction) element).getActionId();
			final RwToolbarElementManagerImpl elementManager = RahmenwerkImpl
					.getInstanz().getRwToolBarManager();

			final RwToolbarElement descriptorAction = elementManager
					.getToolBarElement(tempId);
			if (descriptorAction != null) {
				try {
					final IContributionItem item = descriptorAction
							.createContributionItem(serviceLocator,
									(MenueAction) element);
					parentManager.add(item);
				} catch (final CoreException e) {
					e.printStackTrace();
				}
			}

		} else if (element instanceof MenueSeparator) {
			final Separator separator = new Separator();
			parentManager.add(separator);
		} else if (element instanceof SubMenue) {
			// final ActionContributionItem item = new ActionContributionItem(
			// new ToolbarMenueAction(serviceLocator, (SubMenue) element));
			// parentManager.add(item);
			final ToolBarSubMenue item = new ToolBarSubMenue(
					new ToolbarMenueAction(serviceLocator, (SubMenue) element));
			parentManager.add(item);
		}
	}

	private void generiereStatusLeisteRekursiv(
			final IServiceLocator serviceLocator,
			final IContributionManager parentManager, final MenueElement element) {
		// Item?

		if (element instanceof MenueAction) {
			final String tempId = ((MenueAction) element).getActionId();
			final RwToolbarElementManagerImpl elementManager = RahmenwerkImpl
					.getInstanz().getRwToolBarManager();
			final RwToolbarElement descriptorAction = elementManager
					.getToolBarElement(tempId);
			if (descriptorAction != null) {
				try {
					parentManager.add(descriptorAction.createContributionItem(
							serviceLocator, (MenueAction) element));
				} catch (final CoreException e) {
					e.printStackTrace();
				}
			}
		} else if (element instanceof MenueSeparator) {
			final Separator separator = new Separator();
			parentManager.add(separator);
		} else if (element instanceof SubMenue) {

			// final StatusLineContributionItem item = new
			// StatusLineContributionItem(
			// ((SubMenue) element).getName());
			// parentManager.add(item);

			// final ActionContributionItem item = new ActionContributionItem(
			// new ToolbarMenueAction(serviceLocator, (SubMenue) element));
			// parentManager.add(item);

			final ToolBarSubMenue item = new ToolBarSubMenue(
					new ToolbarMenueAction(serviceLocator, (SubMenue) element));
			parentManager.add(item);
		}
	}
}
