/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Reprsentation eines Untermens.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: SubMenue.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class SubMenue extends Menue {

	private Menue menue;
	private String name;

	/**
	 * Konstruktor, erzeugt ein Untermen mit dem angegebenen Name.
	 * 
	 * @param name
	 *            der Name
	 */
	public SubMenue(final String name) {
		this.name = name;
	}

	/**
	 * setzt den Name des Mens.
	 * 
	 * @param name
	 *            der Name
	 */
	public void setName(final String name) {
		this.name = name;
		setDirty();
	}

	/**
	 * liefert den Name des Mens.
	 * 
	 * @return den Name
	 */
	public String getName() {
		return name;
	}

	@Override
	public Menue getMenue() {
		return menue;
	}

	@Override
	public void setMenue(final Menue menue) {
		this.menue = menue;
	}

	@Override
	public String toString() {
		return "SUBMEN: " + name;
	}

	@Override
	protected void addXmlNode(final Document document, final Element root) {
		final Element element = document
				.createElement(MenuXmlHandler.TAG_SUBMENU);
		element.setAttribute(MenuXmlHandler.ATT_NAME, getName());
		root.appendChild(element);

		for (final MenueElement childElement : getElemente()) {
			((AbstractMenueElement) childElement).addXmlNode(document, element);
		}
	}

	@Override
	public String getVisibleName() {
		final String result = getName();
		if ("<leer>".equals(result)) {
			return "";
		}
		return result;
	}

	@Override
	public void setVisibleName(final String visibleName) {
		setName(visibleName);
	}

	@Override
	protected Menue createMenuCopy() {
		return new SubMenue(name);
	}
}
