/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.menu.editoren;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;

import javax.annotation.PostConstruct;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IWorkbenchCommandConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.menus.CommandContributionItem;
import org.eclipse.ui.menus.CommandContributionItemParameter;
import org.eclipse.ui.part.EditorPart;
import org.xml.sax.SAXException;

import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsAdresse;
import de.bsvrz.buv.rw.basislib.einstellungen.SpeicherKey;
import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.internal.InternalServices;
import de.bsvrz.buv.rw.rw.menu.AbstractMenueLeiste;
import de.bsvrz.buv.rw.rw.menu.LadeSpeicherDialogMenu;
import de.bsvrz.buv.rw.rw.menu.MenueAction;
import de.bsvrz.buv.rw.rw.menu.MenueLeiste;
import de.bsvrz.buv.rw.rw.menu.MenueSeparator;
import de.bsvrz.buv.rw.rw.menu.NamedMenueElement;
import de.bsvrz.buv.rw.rw.menu.StatusLeiste;
import de.bsvrz.buv.rw.rw.menu.SubMenue;
import de.bsvrz.buv.rw.rw.menu.SymbolLeiste;
import de.bsvrz.buv.rw.rw.menu.editoren.actions.ContextActionSeparatorEinfuegen;
import de.bsvrz.buv.rw.rw.menu.editoren.actions.ContextActionSubMenueEinfuegen;

/**
 * Implementiert den Men-Editor innerhalb eines TreeViewers. Dies beinhaltet
 * die grafische Darstellung innerhalb eines Views sowie die Bereitstellung
 * eines Kontextmens mit den entsprechenden Editierfunktionen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: AbstractMenueEditor.java 46953 2013-11-05 10:05:57Z peuker $
 */
public abstract class AbstractMenueEditor extends EditorPart implements
		PropertyChangeListener {

	/** Aktion zum Umbenennen eines Elements in der Menustruktur-Definition. */
	private class MenueElementUmbenennenAction extends Action {

		private NamedMenueElement menueElement;

		public MenueElementUmbenennenAction(final IStructuredSelection selection) {
			setText("Element umbenennen");
			setToolTipText("Benennt das aktuell selektierte Element um.");

			if (selection.size() == 1) {
				final Object element = selection.getFirstElement();
				if ((element instanceof NamedMenueElement)
						&& (((NamedMenueElement) element).getMenue() != null)) {
					menueElement = (NamedMenueElement) element;
				}
			}
			setEnabled(menueElement != null);
		}

		@Override
		public void run() {

			final String oldName = menueElement.getVisibleName();
			if (oldName != null) {
				final InputDialog eingabeDialog = new InputDialog(
						AbstractMenueEditor.this.getSite().getShell(),
						"Element umbenennen:",
						"Bitte geben Sie einen neuen Namen fr das Element\""
								+ oldName + "\" ein.", oldName,
						new IInputValidator() {
							@Override
							public String isValid(final String newText) {
								if (newText.trim().isEmpty()) {
									return "Der Name darf nicht leer sein!";
								}
								return null;
							}
						});

				if (eingabeDialog.open() == Window.OK) {
					menueElement.setVisibleName(eingabeDialog.getValue());
				}
			}
		}
	}

	/**
	 * Aktion zum Zurcksetzen des Namens eines Elements in der
	 * Menustruktur-Definition auf seinen Stndardwert.
	 */
	private static class MenueActionResetNameAction extends Action {

		private MenueAction menueElement;

		public MenueActionResetNameAction(final IStructuredSelection selection) {
			setText("Name zurcksetzen");
			setToolTipText("Setzt den vom Nutzer berschriebenen Name zurck.");

			if (selection.size() == 1) {
				final Object element = selection.getFirstElement();
				if (element instanceof MenueAction) {
					if (((MenueAction) element).getUserLabel() != null) {
						menueElement = (MenueAction) element;
					}
				}
			}
			setEnabled(menueElement != null);
		}

		@Override
		public void run() {
			menueElement.setUserLabel(null);
		}
	}

	/** Aktion zum Aktivieren der Menstruktur aus dem Editor. */
	private final class MenueAktivierenAction extends Action {

		public MenueAktivierenAction() {
			setText("Aktiviere aktuelle Menstruktur");
			setToolTipText("Aktiviert die aktuell dargestellte Menstruktur.");
			setImageDescriptor(AbstractMenueEditor.this
					.getImageDescriptor("icons/etool16/refresh.gif"));
		}

		@Override
		public void run() {
			aktiviereMenu();
		}
	}

	/** TreeViewer, in dem die Darstellung der Menstruktur erfolgen soll. */
	private TreeViewer treeViewer;

	private MenueDndAdapter dndAdapter;

	private InfoBereichComposite infoBereich;

	@Override
	public void createPartControl(final Composite parent) {

		final Composite panel = new Composite(parent, SWT.NONE);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(panel);
		panel.setLayout(new GridLayout());

		treeViewer = new TreeViewer(panel, SWT.BORDER | SWT.H_SCROLL
				| SWT.V_SCROLL);
		treeViewer.setContentProvider(new MenueEditorContentProvider());
		treeViewer.setLabelProvider(new MenueEditorLabelProvider());
		treeViewer.setInput(getEditorInput());
		treeViewer.expandAll();
		GridDataFactory.fillDefaults().grab(true, true)
				.applyTo(treeViewer.getControl());

		final int ops = DND.DROP_MOVE;
		final Transfer[] dropTransfers = new Transfer[] { LocalSelectionTransfer
				.getTransfer() };
		dndAdapter = new MenueDndAdapter(treeViewer);
		treeViewer.addDropSupport(ops, dropTransfers, dndAdapter);

		final Transfer[] dragTransfers = new Transfer[] { LocalSelectionTransfer
				.getTransfer() };
		treeViewer.addDragSupport(ops, dragTransfers,
				new LocalViewerSelectionDragAdapter(treeViewer));

		treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				infoBereich.updateFromSelection((IStructuredSelection) event
						.getSelection());
			}
		});

		infoBereich = new InfoBereichComposite(panel, SWT.NONE);
		GridDataFactory.fillDefaults().grab(true, false).hint(SWT.DEFAULT, 150)
				.applyTo(infoBereich);

		infoBereich.updateFromSelection((IStructuredSelection) treeViewer
				.getSelection());

		getSite().setSelectionProvider(treeViewer);

		// TODO Hilfe ergnzen
		// PlatformUI.getWorkbench().getHelpSystem()
		// .setHelp(panel, "de.bsvrz.buv.rw.basislib.menuleisteneditor");

		registerKontextMenu();
		registerActionBars();
	}

	/**
	 * Verankert das Kontextmen.
	 * 
	 */
	private void registerKontextMenu() {
		final MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(final IMenuManager manager) {
				fuelleKontextMenu(manager);
			}
		});

		final Menu menu = menuMgr.createContextMenu(treeViewer.getControl());
		treeViewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, treeViewer);
	}

	/**
	 * Fllt das Kontextmen.
	 * 
	 * @param manager
	 */
	@PostConstruct
	private void fuelleKontextMenu(@Optional final IMenuManager manager) {

		manager.add(new Action("Laden") {
			@Override
			public void run() {
				ladeMenuleiste();
			}
		});

		CommandContributionItemParameter parameter = new CommandContributionItemParameter(
				getEditorSite().getWorkbenchWindow(), null,
				IWorkbenchCommandConstants.FILE_SAVE,
				CommandContributionItem.STYLE_PUSH);
		manager.add(new CommandContributionItem(parameter));

		manager.add(new Separator());

		parameter = new CommandContributionItemParameter(getEditorSite()
				.getWorkbenchWindow(), null,
				IWorkbenchCommandConstants.FILE_EXPORT,
				CommandContributionItem.STYLE_PUSH);
		manager.add(new CommandContributionItem(parameter));

		parameter = new CommandContributionItemParameter(getEditorSite()
				.getWorkbenchWindow(), null,
				IWorkbenchCommandConstants.FILE_IMPORT,
				CommandContributionItem.STYLE_PUSH);
		manager.add(new CommandContributionItem(parameter));

		manager.add(new Separator());
		manager.add(new MenueAktivierenAction());

		manager.add(new Separator());

		parameter = new CommandContributionItemParameter(getEditorSite()
				.getWorkbenchWindow(), null,
				IWorkbenchCommandConstants.EDIT_CUT,
				CommandContributionItem.STYLE_PUSH);
		manager.add(new CommandContributionItem(parameter));

		parameter = new CommandContributionItemParameter(getEditorSite()
				.getWorkbenchWindow(), null,
				IWorkbenchCommandConstants.EDIT_COPY,
				CommandContributionItem.STYLE_PUSH);
		manager.add(new CommandContributionItem(parameter));

		parameter = new CommandContributionItemParameter(getEditorSite()
				.getWorkbenchWindow(), null,
				IWorkbenchCommandConstants.EDIT_PASTE,
				CommandContributionItem.STYLE_PUSH);
		manager.add(new CommandContributionItem(parameter));

		manager.add(new ContextActionSubMenueEinfuegen(treeViewer));

		manager.add(new MenueElementUmbenennenAction(
				(IStructuredSelection) treeViewer.getSelection()));
		manager.add(new MenueActionResetNameAction(
				(IStructuredSelection) treeViewer.getSelection()));

		manager.add(new ContextActionSeparatorEinfuegen(treeViewer));
		parameter = new CommandContributionItemParameter(getEditorSite()
				.getWorkbenchWindow(), null,
				IWorkbenchCommandConstants.EDIT_DELETE,
				CommandContributionItem.STYLE_PUSH);
		manager.add(new CommandContributionItem(parameter));
	}

	/**
	 * liefert den Name des Mens zur Anzeige im Reiter des Editors.
	 * 
	 * @return den Name
	 */
	protected abstract String getMenueName();

	/**
	 * Liefert einen {@link ImageDescriptor} aus der ImageRegistry. Ist dort der
	 * entsprechende {@link ImageDescriptor} noch nicht vorhanden wird aus dem
	 * bergebenen Pfad versucht, diesen aufzubauene.
	 * 
	 * @param imagePath
	 *            Pfad zum Image.
	 * @return {@link ImageDescriptor} oder <code>null</code>, wenn der Pfad
	 *         nicht existiert und die ImageRegistry den {@link ImageDescriptor}
	 *         nicht erzeugen konnte.
	 */
	private ImageDescriptor getImageDescriptor(final String imagePath) {
		return RahmenwerkActivator.getDefault().getImageDescriptor(imagePath);
	}

	/**
	 * liefert die ID mit der die Einstellungen fr die entsprechende
	 * Menstruktur im Einstellungsspeicher des Rahmenwerks hinterlegt sind.
	 * 
	 * @return die ID
	 */
	public abstract String getEinstellungsId();

	/**
	 * ldt die Struktur einer Menleiste aus den Einstellungen und setzt den
	 * Editorinput neu.
	 */
	private void ladeMenuleiste() {
		final LadeSpeicherDialogMenu dialog = new LadeSpeicherDialogMenu(
				getSite().getShell(), SWT.OPEN);
		if (dialog.open() == Window.OK) {
			final SpeicherKey speicherKey = dialog.getSpeicherKey();
			menueLeisteLaden(speicherKey);
		}
	}

	/**
	 * ldt die durch die bergebene Speicheroption definierte Menstruktur aus
	 * dem entsprechenden Einstellungsspeicher des Rahmenwerks.
	 * 
	 * @param speicherKey
	 *            die Speicherinformationen
	 */
	public void menueLeisteLaden(final SpeicherKey speicherKey) {
		final EinstellungsAdresse adresse = new EinstellungsAdresse(null,
				getEinstellungsId(), speicherKey.getOwnerType(),
				speicherKey.getPid(), speicherKey.getLocation());

		String parameter = null;
		try {
			parameter = (String) InternalServices.getInstance()
					.getEinstellungen().getValue(adresse);
		} catch (final IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if ((parameter != null) && (parameter.length() > 0)) {
			try {
				final MenueEditorInput input = createInput(parameter);
				setInput(input);
			} catch (final SAXException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (final IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (final ParserConfigurationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Override
	public MenueEditorInput getEditorInput() {
		return (MenueEditorInput) super.getEditorInput();
	}

	@Override
	public void setInput(final IEditorInput input) {

		final MenueEditorInput oldInput = getEditorInput();
		if (oldInput != null) {
			oldInput.getMenueLeiste().removePropertyChangeListener(this);
		}

		if (input instanceof MenueEditorInput) {
			super.setInput(input);
			if (treeViewer != null) {
				treeViewer.setInput(input);
				treeViewer.expandAll();
			}
			((MenueEditorInput) input).getMenueLeiste()
					.addPropertyChangeListener(this);
		}
	}

	/**
	 * erzeugt den Meneditor-Input fr die jeweilige Instanz des Editors.
	 * 
	 * @param parameter
	 *            die Daten des mens als XML-Struktur.
	 * @return die Instanz des Editor-Inputs
	 * @throws SAXException
	 *             Fehler beim Parsen der bergebenen Datenstruktur
	 * @throws IOException
	 *             allgemeiner Lesefehler
	 * @throws ParserConfigurationException
	 *             fehlerhafte Parserkonfiguration
	 */
	public abstract MenueEditorInput createInput(String parameter)
			throws SAXException, IOException, ParserConfigurationException;

	/** aktiviert die aktuell im Meneditor befindliche Menstruktur. */
	protected abstract void aktiviereMenu();

	@Override
	public void setFocus() {
		if ((treeViewer != null) && (!treeViewer.getControl().isDisposed())) {
			treeViewer.getControl().setFocus();
		}
	}

	/**
	 * Initiiert das Fllen der von der View bereitgestellten ActionBars.
	 * 
	 */
	private void registerActionBars() {
		final IToolBarManager toolBarManager = getEditorSite().getActionBars()
				.getToolBarManager();

		if (toolBarManager.find(IWorkbenchCommandConstants.FILE_SAVE) == null) {
			final CommandContributionItemParameter parameter = new CommandContributionItemParameter(
					getEditorSite().getWorkbenchWindow(), null,
					IWorkbenchCommandConstants.FILE_SAVE,
					CommandContributionItem.STYLE_PUSH);
			toolBarManager.add(new CommandContributionItem(parameter));
		}
		if (toolBarManager.find(IWorkbenchCommandConstants.EDIT_CUT) == null) {
			final CommandContributionItemParameter parameter = new CommandContributionItemParameter(
					getEditorSite().getWorkbenchWindow(), null,
					IWorkbenchCommandConstants.EDIT_CUT,
					CommandContributionItem.STYLE_PUSH);
			toolBarManager.add(new CommandContributionItem(parameter));
		}
		if (toolBarManager.find(IWorkbenchCommandConstants.EDIT_COPY) == null) {
			final CommandContributionItemParameter parameter = new CommandContributionItemParameter(
					getEditorSite().getWorkbenchWindow(), null,
					IWorkbenchCommandConstants.EDIT_COPY,
					CommandContributionItem.STYLE_PUSH);
			toolBarManager.add(new CommandContributionItem(parameter));
		}
		if (toolBarManager.find(IWorkbenchCommandConstants.EDIT_PASTE) == null) {
			final CommandContributionItemParameter parameter = new CommandContributionItemParameter(
					getEditorSite().getWorkbenchWindow(), null,
					IWorkbenchCommandConstants.EDIT_PASTE,
					CommandContributionItem.STYLE_PUSH);
			toolBarManager.add(new CommandContributionItem(parameter));
		}
		if (toolBarManager.find(IWorkbenchCommandConstants.EDIT_DELETE) == null) {
			final CommandContributionItemParameter parameter = new CommandContributionItemParameter(
					getEditorSite().getWorkbenchWindow(), null,
					IWorkbenchCommandConstants.EDIT_DELETE,
					CommandContributionItem.STYLE_PUSH);
			toolBarManager.add(new CommandContributionItem(parameter));
		}
		if (toolBarManager.find(IWorkbenchCommandConstants.FILE_EXPORT) == null) {
			final CommandContributionItemParameter parameter = new CommandContributionItemParameter(
					getEditorSite().getWorkbenchWindow(), null,
					IWorkbenchCommandConstants.FILE_EXPORT,
					CommandContributionItem.STYLE_PUSH);
			toolBarManager.add(new CommandContributionItem(parameter));
		}
		if (toolBarManager.find(IWorkbenchCommandConstants.FILE_IMPORT) == null) {
			final CommandContributionItemParameter parameter = new CommandContributionItemParameter(
					getEditorSite().getWorkbenchWindow(), null,
					IWorkbenchCommandConstants.FILE_IMPORT,
					CommandContributionItem.STYLE_PUSH);
			toolBarManager.add(new CommandContributionItem(parameter));
		}

		toolBarManager.update(true);

	}

	/**
	 * Contentprovider fr den Meneditor TreeViewer. Stellt die Inhalte fr den
	 * verwendeten <code>TreeViewer</code> bereit.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class MenueEditorContentProvider implements
			ITreeContentProvider {

		/**
		 * Liefert die Kindelemente des angegebenen Elternelements.
		 * 
		 * @param parentElement
		 *            Elternelement, zu dem die Kindelemente bestimmt werden
		 *            sollen
		 * @return Kindelemente des Elternelements
		 */
		@Override
		public Object[] getChildren(final Object parentElement) {
			final Collection<Object> result = new ArrayList<Object>();

			if (parentElement instanceof MenueEditorInput) {
				result.add(((MenueEditorInput) parentElement).getMenueLeiste());
			} else if (parentElement instanceof AbstractMenueLeiste) {
				result.addAll(((AbstractMenueLeiste) parentElement)
						.getElemente());
			} else if (parentElement instanceof SubMenue) {
				result.addAll(((SubMenue) parentElement).getElemente());
			}
			return result.toArray();
		}

		/**
		 * Bestimmt das Elternelement eines Elements.
		 * 
		 * @param element
		 *            Element, zu dem das Elternelement bestimmt werden soll
		 * @return Elternelement des Elements oder <code>null</code>, falls kein
		 *         Elternelement bestimmt werden kann
		 */
		@Override
		public Object getParent(final Object element) {
			// XXX if (element instanceof Element) {
			// final Element tempElement = (Element) element;
			// return tempElement.getParentElement();
			// }
			return null;

		}

		/**
		 * Prft, ob das bergebene Element ber Kindelemente verfgt.
		 * 
		 * @param element
		 *            Element, zu dem die Prfung durchefhrt werden soll
		 * @return <code>true</code> falls das Element ber Kindelemente verfgt
		 *         sonst <code>false</code>
		 */
		@Override
		public boolean hasChildren(final Object element) {
			return getChildren(element).length > 0;
		}

		/**
		 * Liefert die Elemente eines Elements.
		 * 
		 * @param inputElement
		 *            Element, zu dem die Elemente bestimmt werden sollen
		 * @return Elemente, die zu einem Element bestimmt wurden
		 */
		@Override
		public Object[] getElements(final Object inputElement) {
			return getChildren(inputElement);
		}

		@Override
		public void dispose() {
			// Keine Ressourcen freizugeben
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput,
				final Object newInput) {
			// der Input wird nicht verwendet
		}

	}

	/**
	 * LabelProvider fr den <code>TreeViewer</code> der Baumdarstellung Stellt
	 * die Bezeichnungen sowie die Symbole der Elemente des verwendeten
	 * <code>TreeViewers</code> bereit.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class MenueEditorLabelProvider extends LabelProvider
			implements IColorProvider {

		/**
		 * Liefert den Text zu einem Element.
		 * 
		 * @param element
		 *            Element, zu dem der Text bestimmt werden soll
		 * @return Text des Elements
		 */
		@Override
		public String getText(final Object element) {

			String result = null;

			if (element instanceof StatusLeiste) {
				result = "Statusleiste";
			} else if (element instanceof MenueLeiste) {
				result = "Menleiste";
			} else if (element instanceof SymbolLeiste) {
				result = "Symbolleiste";
			} else if (element instanceof SubMenue) {
				result = ((SubMenue) element).getName();
			} else if (element instanceof MenueAction) {
				result = ((MenueAction) element).getDescription();
			} else if (element instanceof MenueSeparator) {
				result = "----------------------------------";
			}

			if (result == null) {
				return "unbekanntes Element";
			}

			return result;
		}

		/**
		 * Bestimmt das Symbol eines Elements.
		 * 
		 * @param element
		 *            Element, zu dem das Symbol bestimmt werden soll
		 * @return Symbol des Elements als Image
		 */
		@Override
		public Image getImage(final Object element) {
			String iconPfad = null;

			if (element instanceof MenueLeiste) {
				iconPfad = "icons/obj16/menu.gif";
			} else if (element instanceof MenueLeiste) {
				iconPfad = "icons/obj16/menubar.gif";
			} else if (element instanceof SubMenue) {
				iconPfad = "icons/obj16/menusubmenu.png";
			} else if (element instanceof MenueAction) {
				iconPfad = "icons/obj16/menuitem.gif";
			} else if (element instanceof MenueSeparator) {
				iconPfad = "icons/obj16/menuseparator.gif";
			} else {
				iconPfad = "icons/obj16/menu.gif";
			}

			return RahmenwerkActivator.getDefault().getImage(iconPfad);
		}

		@Override
		public Color getForeground(final Object element) {
			if (element instanceof MenueAction) {
				if (!((MenueAction) element).isAvailable()) {
					return Display.getDefault().getSystemColor(SWT.COLOR_RED);
				}
			}
			return null;
		}

		@Override
		public Color getBackground(final Object element) {
			if (element instanceof MenueAction) {
				if (!((MenueAction) element).isAvailable()) {
					return Display.getDefault().getSystemColor(SWT.COLOR_GRAY);
				}
			}
			return null;
		}
	}

	@Override
	public void doSave(final IProgressMonitor monitor) {

		final LadeSpeicherDialogMenu dialog = new LadeSpeicherDialogMenu(
				getSite().getShell(), SWT.SAVE);

		if (dialog.open() == Window.OK) {

			final SpeicherKey key = dialog.getSpeicherKey();

			try {
				final EinstellungsAdresse adresse = new EinstellungsAdresse(
						null, getEinstellungsId(), key.getOwnerType(),
						key.getPid(), key.getLocation());

				final AbstractMenueLeiste menueLeiste = getEditorInput()
						.getMenueLeiste();
				if (menueLeiste.isEmpty()) {
					InternalServices.getInstance().getEinstellungen()
							.setValue(adresse, null);
				} else {
					final String xmlStruktur = menueLeiste.getXmlFormat();
					InternalServices.getInstance().getEinstellungen()
							.setValue(adresse, xmlStruktur);
				}
				menueLeiste.clean();
			} catch (final IOException e) {
				e.printStackTrace();
			} catch (final ParserConfigurationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (final TransformerException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Override
	public void doSaveAs() {
		doSave(new NullProgressMonitor());
	}

	@Override
	public void init(final IEditorSite site, final IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
		setPartName(input.getName());
	}

	@Override
	public boolean isDirty() {
		return getEditorInput().getMenueLeiste().isDirty();
	}

	@Override
	public boolean isSaveAsAllowed() {
		return true;
	}

	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(AbstractMenueLeiste.PROP_DIRTY)) {
			firePropertyChange(IEditorPart.PROP_DIRTY);
			if ((Boolean) evt.getNewValue()) {
				treeViewer.refresh();
			}
		}
	}

}
