/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu.editoren;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import de.bsvrz.buv.rw.rw.menu.PluginCollection;
import de.bsvrz.buv.rw.rw.menu.RwPluginElemente;

/**
 * Contentprovider fr die Darstellung der fr den Men-Editor verfgbaren
 * Men-Elemente.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: CommandSelektorContentProvider.java 45420 2013-08-09 12:12:12Z peuker $
 */
class CommandSelektorContentProvider implements ITreeContentProvider {

	@Override
	public Object[] getChildren(final Object parentElement) {
		final Collection<Object> result = new ArrayList<Object>();

		if (parentElement instanceof PluginCollection) {
			/* DescriptorWurzel. */
			result.addAll(((PluginCollection) parentElement).getPlugins());
		} else if (parentElement instanceof RwPluginElemente) {
			/* DescriptorPlugin. */
			result.addAll(((RwPluginElemente) parentElement).getElemente());
		}

		return result.toArray();
	}

	@Override
	public Object getParent(final Object element) {
		return null;
	}

	@Override
	public boolean hasChildren(final Object element) {
		/*
		 * DescriptorWurzel
		 */
		if (element instanceof PluginCollection) {
			final PluginCollection descriptorWurzel = (PluginCollection) element;
			return (descriptorWurzel.getPlugins().size() > 0);
		} else if (element instanceof RwPluginElemente) {
			/*
			 * DescriptorPlugin
			 */
			final RwPluginElemente descriptorPlugin = (RwPluginElemente) element;
			return (!descriptorPlugin.getElemente().isEmpty());
		} else {
			return false;
		}
	}

	@Override
	public Object[] getElements(final Object inputElement) {
		return getChildren(inputElement);
	}

	@Override
	public void dispose() {
		// keine Ressourcen freizugeben
	}

	@Override
	public void inputChanged(final Viewer viewer, final Object oldInput,
			final Object newInput) {
		// Input wird nicht von hier ausgewertet?
	}
}
