/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu.editoren;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.rw.rw.menu.MenueAction;
import de.bsvrz.buv.rw.rw.menu.MenueAction.Parameter;

/**
 * Composite zur Darstellung zustzlicher Informationen im Men-Editor.
 *
 * Insbesondere knnen hier die Parameter fr einzelne Men-Elemente bearbeitet
 * werden.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: InfoBereichComposite.java 50140 2014-04-11 11:48:29Z peuker $
 */
class InfoBereichComposite extends Composite {

	/** Support zum Editieren der Parameterwerte von Men-Elementen. */
	private class ValueEditingSupport extends EditingSupport {

		private final TableViewer viewer;

		public ValueEditingSupport(final TableViewer viewer) {
			super(viewer);
			this.viewer = viewer;
		}

		@Override
		protected CellEditor getCellEditor(final Object element) {
			return new TextCellEditor(viewer.getTable());
		}

		@Override
		protected boolean canEdit(final Object element) {
			return true;
		}

		@Override
		protected Object getValue(final Object element) {
			return ((MenueAction.Parameter) element).getValue();
		}

		@Override
		protected void setValue(final Object element, final Object value) {
			final MenueAction.Parameter parameter = (Parameter) element;
			parameter.setValue((String) value);
			if (menueAction != null) {
				menueAction.setParameter(parameter.getParameterId(),
						parameter.getName(), parameter.getValue());
			}
			viewer.update(element, null);
		}
	}

	private final TableViewer parameterViewer;
	private MenueAction menueAction;
	private Text commandIdLabel;

	public InfoBereichComposite(final Composite parent, final int style) {

		super(parent, style);
		setLayout(new GridLayout());

		Label label = new Label(this, SWT.NONE);
		label.setText("Command-ID: ");

		commandIdLabel = new Text(this, SWT.BORDER | SWT.SHADOW_OUT);
		commandIdLabel.setText("");
		commandIdLabel.setEditable(false);
		GridDataFactory.fillDefaults().grab(true, false)
		.applyTo(commandIdLabel);

		label = new Label(this, SWT.NONE);
		label.setText("Parameter: ");

		final Composite tableComposite = new Composite(this, SWT.NONE);
		final TableColumnLayout tabLayout = new TableColumnLayout();
		tableComposite.setLayout(tabLayout);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(tableComposite);

		parameterViewer = new TableViewer(tableComposite, SWT.BORDER
				| SWT.FULL_SELECTION);
		parameterViewer.setContentProvider(new ArrayContentProvider());
		parameterViewer.getTable().setHeaderVisible(true);
		parameterViewer.getTable().setLinesVisible(true);

		TableViewerColumn col = new TableViewerColumn(parameterViewer, SWT.NONE);
		col.getColumn().setText("Name");
		col.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(final Object element) {
				return ((MenueAction.Parameter) element).getName();
			}
		});
		tabLayout.setColumnData(col.getColumn(), new ColumnWeightData(1));

		col = new TableViewerColumn(parameterViewer, SWT.NONE);
		col.getColumn().setText("Wert");
		col.setLabelProvider(new ColumnLabelProvider() {

			@Override
			public String getText(final Object element) {
				final String str = ((MenueAction.Parameter) element).getValue();
				if (str == null) {
					return "";
				}
				return str;
			}
		});
		tabLayout.setColumnData(col.getColumn(), new ColumnWeightData(1));
		col.setEditingSupport(new ValueEditingSupport(parameterViewer));
	}

	public void updateFromSelection(final IStructuredSelection selection) {
		final Object element = selection.getFirstElement();
		menueAction = null;
		if ((selection.size() != 1) || (!(element instanceof MenueAction))) {
			commandIdLabel.setText("<keine Auswahl>");
			parameterViewer.setInput(new Object[0]);
			setEnabled(false);
			parameterViewer.getControl().setEnabled(false);
			return;
		}

		setEnabled(true);
		parameterViewer.getControl().setEnabled(true);
		menueAction = (MenueAction) element;
		commandIdLabel.setText(menueAction.getActionId());
		parameterViewer.setInput(menueAction.getParameters().toArray(
				new MenueAction.Parameter[menueAction.getParameters().size()]));
	}
}
