/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu.editoren;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;
import de.bsvrz.buv.rw.rw.menu.MenuGenerator;
import de.bsvrz.buv.rw.rw.menu.RwToolbarElementManagerImpl;
import de.bsvrz.buv.rw.rw.menu.SymbolLeiste;

/**
 * Editor zur Bearbeitung der Hauptsymbolleiste des Rahmenwerks.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: ToolbarEditor.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class ToolbarEditor extends AbstractMenueEditor {
	/** ID dieses Parts. */
	public static final String ID = "de.bsvrz.buv.rw.rw.editor.toolbareditor";

	/** Parameter fr die Struktur der Symbolleiste. */
	public static final String PARAM_STRUCT_SYMBOLLEISTE = "xmlStrukturSymbolleiste";

	@Override
	public MenueEditorInput createInput(final String parameter)
			throws SAXException, IOException, ParserConfigurationException {
		return new MenueEditorInput(new SymbolLeiste(parameter));
	}

	@Override
	public String getEinstellungsId() {
		return ToolbarEditor.PARAM_STRUCT_SYMBOLLEISTE;
	}

	@Override
	protected String getMenueName() {
		return "Symbolleiste";
	}

	@Override
	protected void aktiviereMenu() {
		final RwToolbarElementManagerImpl elementManager = RahmenwerkImpl
				.getInstanz().getRwToolBarManager();

		final MenuGenerator mg = new MenuGenerator(
				elementManager.getSymbolLeisteManagerToolbar(),
				getEditorInput().getMenueLeiste());
		mg.generiereMenu(getSite().getWorkbenchWindow());
		elementManager.getSymbolLeisteManager().update(true);
	}
}
