/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu.editoren.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;

import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.menu.Menue;
import de.bsvrz.buv.rw.rw.menu.MenueElement;
import de.bsvrz.buv.rw.rw.menu.MenueSeparator;

/**
 * Kontextmen-Aktion des Meneditors zum Einfgen eines Separators.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: ContextActionSeparatorEinfuegen.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class ContextActionSeparatorEinfuegen extends Action {

	private final StructuredViewer viewer;
	private MenueElement zielElement;
	private Menue menue;

	/**
	 * Konstruktor, erzeugt die Kontextmen-Aktion fr den bergebenen Viewer.
	 * 
	 * @param viewer
	 *            der Viewer
	 */
	public ContextActionSeparatorEinfuegen(final TreeViewer viewer) {

		this.viewer = viewer;

		setText("Separator einfgen");
		setImageDescriptor(RahmenwerkActivator.getDefault().getImageDescriptor(
				"icons/etool16/menuseparator.gif"));

		final IStructuredSelection selection = (IStructuredSelection) viewer
				.getSelection();
		if (selection.size() == 1) {
			final Object selected = selection.getFirstElement();
			if (selected instanceof MenueElement) {
				menue = ((MenueElement) selected).getMenue();
				if (menue != null) {
					zielElement = (MenueElement) selected;
				}
			}
		}

	}

	@Override
	public void run() {
		if (isEnabled()) {
			final MenueSeparator separator = new MenueSeparator();
			menue.addElementAfter(separator, zielElement);
			viewer.refresh(menue);
		}
	}

	@Override
	public boolean isEnabled() {
		final boolean result = (menue != null) && (zielElement != null);
		return result && super.isEnabled();
	}
}
