/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu.editoren.handler;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;

import de.bsvrz.buv.rw.rw.menu.Menue;
import de.bsvrz.buv.rw.rw.menu.MenueElement;

/**
 * Handler fr das allgemeine Command "Ausschneiden" ("org.eclipse.ui.edit.cut")
 * im Men-Editor.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: MenueEditorCutHandler.java 46953 2013-11-05 10:05:57Z peuker $
 */
public class MenueEditorCutHandler extends AbstractHandler {

	@Override
	public Object execute(final ExecutionEvent event) throws ExecutionException {
		final ISelection currentSelection = HandlerUtil
				.getCurrentSelection(event);

		if (currentSelection instanceof IStructuredSelection) {

			final List<Object> selectionList = ((IStructuredSelection) currentSelection)
					.toList();

			final Collection<MenueElement> cutElements = new HashSet<MenueElement>();

			for (final Object obj : selectionList) {
				if (obj instanceof MenueElement) {
					if (((MenueElement) obj).getMenue() == null) {
						continue;
					}
					if (!isParentInSelection((MenueElement) obj, selectionList)) {
						cutElements.add((MenueElement) obj);
					}
				}
			}

			for (final MenueElement element : cutElements) {
				final Menue menue = element.getMenue();
				if (menue != null) {
					menue.removeElement(element);
				}
			}

			LocalSelectionTransfer.getTransfer().setSelection(
					new StructuredSelection(cutElements.toArray()));
		}

		return null;
	}

	private boolean isParentInSelection(final MenueElement element,
			final List<Object> selectionList) {

		Menue menue = element.getMenue();
		while (menue != null) {
			if (selectionList.contains(menue)) {
				return true;
			}
			menue = menue.getMenue();
		}

		return false;
	}
}
