/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 **************************************************************************************/

package de.bsvrz.buv.rw.rw.perspective;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;
import org.osgi.framework.Bundle;

import de.bsvrz.buv.rw.rw.RahmenwerkActivator;

/**
 * Standardview der Standardperspektive, da leere Perspektiven beim zurcksetzen
 * zu Problemen fhren (Eclispe 4 Bug?).
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwInfoView.java 48351 2014-01-24 15:18:14Z peuker $
 */
public class RwInfoView extends ViewPart {

	/** die ID des Views. */
	public static final String VIEW_ID = "de.bsvrz.buv.rw.rw.view.rahmenwerkinfo";

	private static final String PLACEHOLDER = "___PLACEHOLDER____";

	/** letzte Rckfallebene fr den Ausgabetext. */
	private static final String ERROR_TEXT = "Oops, it happened something unexpected<br>Send a message to <a href=\"mailto:support@bitctrl.de\">support@bitctrl.de</a><p>"
			+ RwInfoView.PLACEHOLDER + "</p>";

	/** das Browserwidget zur Anzeige der gewnschten Informationen. */
	private Browser browser;

	@Override
	public void createPartControl(final Composite parent) {
		browser = new Browser(parent, SWT.NONE);

		final Bundle bundle = RahmenwerkActivator.getDefault().getBundle();
		final URL fileURL = bundle.getEntry("/assets/infotext.html");
		String errorMsg;
		if (fileURL == null) {
			errorMsg = RwInfoView.ERROR_TEXT.replace(RwInfoView.PLACEHOLDER,
					"File assets/infotext.html nicht gefunden!");
			browser.setText(errorMsg);
		} else {
			try {
				final URL url = FileLocator.resolve(fileURL);
				browser.setUrl(url.toString());
			} catch (final IOException e) {
				final StringWriter stringWriter = new StringWriter();
				final PrintWriter writer = new PrintWriter(stringWriter);
				e.printStackTrace(writer);
				errorMsg = RwInfoView.ERROR_TEXT.replace(
						RwInfoView.PLACEHOLDER, stringWriter.getBuffer());
				browser.setText(errorMsg);
				writer.close();
			}
		}
	}

	@Override
	public void setFocus() {
		if ((browser != null) && (!browser.isDisposed())) {
			browser.setFocus();
		}
	}
}
