/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.ComboFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import de.bsvrz.buv.rw.rw.RahmenwerkActivator;

/**
 * Einstellungsseite fr allgemeine Rahmenwerk-Parameter. Unter anderem werden
 * hier festgelegt:
 * 
 * <ul>
 * <li>die Anzeige und das Verhalten der INTRO-Seite</li>
 * <li>das Windows-Titelformat im Offline-Betrieb</li>
 * <li>das Windows-Titelformat im Online-Betrieb</li>
 * </ul>
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RahmenwerkVoreinstellungenSeite.java 46953 2013-11-05 10:05:57Z peuker $
 */
public class RahmenwerkVoreinstellungenSeite extends FieldEditorPreferencePage
		implements IWorkbenchPreferencePage {

	@Override
	public void init(final IWorkbench workbench) {
		setPreferenceStore(RahmenwerkActivator.getDefault()
				.getPreferenceStore());
		setDescription("Allgemeine Rahmenwerkseinstellungen");
	}

	@Override
	protected void createFieldEditors() {
		addField(new StringFieldEditor(RahmenwerkPreference.INTRO_URL.getId(),
				"Intro-Seite:", getFieldEditorParent()));
		addField(new BooleanFieldEditor(
				RahmenwerkPreference.INTRO_ALWAYS.getId(),
				"Intro-Seite immer anzeigen", getFieldEditorParent()));
		addField(new StringFieldEditor(
				RahmenwerkPreference.ONLINE_TITLE_FORMAT.getId(),
				"Titel (online):", getFieldEditorParent()));
		addField(new StringFieldEditor(
				RahmenwerkPreference.OFFLINE_TITLE_FORMAT.getId(),
				"Titel (offline):", getFieldEditorParent()));
		addField(new ComboFieldEditor(
				RahmenwerkPreference.START_PERSPEKTIVE.getId(),
				"Standardperspektive:", RahmenwerkActivator.getDefault()
						.getRwPerspectives().getPerspektiveId(),
				getFieldEditorParent()));
	}
}
