/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.preferences;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.RahmenwerkActivator;

/**
 * Liefert die {@link PreferencePage} zum Vornehmen der Seiteneinstellungen.
 * 
 * TODO berarbeiten!
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: Seiteneinstellungen.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class Seiteneinstellungen extends PreferencePage implements
		IWorkbenchPreferencePage {

	/** ID der PreferencePage. */
	public static final String ID = "de.bsvrz.buv.rw.basislib.pageSeiteneinstellungen";

	/** Composite zum Vornehmen der Seiteneinstellungen. */
	private Zusammenstellung zusammenstellung;

	/**
	 * Konstruktor, parameterlos.
	 */
	public Seiteneinstellungen() {
	}

	/**
	 * Konstruktor, setzt den Titel der Seiteneinstellungen.
	 * 
	 * @param title
	 *            Titel der Seiteneinstellungen.
	 */
	public Seiteneinstellungen(final String title) {
		super(title);
	}

	/**
	 * Konstruktor, setzt den Titel und das Bild der Seiteneinstellugnen.
	 * 
	 * @param title
	 *            Titel der Seiteneinstellungen.
	 * @param image
	 *            Bild der Seiteneinstellungen
	 */
	public Seiteneinstellungen(final String title, final ImageDescriptor image) {
		super(title, image);
	}

	@Override
	protected Control createContents(final Composite parent) {
		zusammenstellung = new Zusammenstellung(parent, SWT.NONE);
		final GridData gridDataBasisZusammenstellung = new GridData();
		gridDataBasisZusammenstellung.horizontalAlignment = GridData.FILL;
		gridDataBasisZusammenstellung.grabExcessHorizontalSpace = true;
		gridDataBasisZusammenstellung.grabExcessVerticalSpace = true;
		gridDataBasisZusammenstellung.verticalAlignment = GridData.FILL;
		zusammenstellung.setLayoutData(gridDataBasisZusammenstellung);

		laden();

		return null;
	}

	/**
	 * Speichert die Einstellungen.
	 * 
	 */
	private void sichern() {
		final String[] kopfFussZeilen = zusammenstellung.getKopfFussZeilen();
		getPreferenceStore().setValue(
				DefsSeiteneinstellungen.ID_KOPFZEILE_LINKS, kopfFussZeilen[0]);
		getPreferenceStore().setValue(
				DefsSeiteneinstellungen.ID_KOPFZEILE_MITTE, kopfFussZeilen[1]);
		getPreferenceStore().setValue(
				DefsSeiteneinstellungen.ID_KOPFZEILE_RECHTS, kopfFussZeilen[2]);
		getPreferenceStore().setValue(
				DefsSeiteneinstellungen.ID_FUSSZEILE_LINKS, kopfFussZeilen[3]);
		getPreferenceStore().setValue(
				DefsSeiteneinstellungen.ID_FUSSZEILE_MITTE, kopfFussZeilen[4]);
		getPreferenceStore().setValue(
				DefsSeiteneinstellungen.ID_FUSSZEILE_RECHTS, kopfFussZeilen[5]);

	}

	/**
	 * Ldt die Einstellungen.
	 */
	private void laden() {
		final String fusszeileLinks = getPreferenceStore().getString(
				DefsSeiteneinstellungen.ID_FUSSZEILE_LINKS);

		final String fusszeileMitte = getPreferenceStore().getString(
				DefsSeiteneinstellungen.ID_FUSSZEILE_MITTE);

		final String fusszeileRechts = getPreferenceStore().getString(
				DefsSeiteneinstellungen.ID_FUSSZEILE_RECHTS);

		final String kopfzeileLinks = getPreferenceStore().getString(
				DefsSeiteneinstellungen.ID_KOPFZEILE_LINKS);

		final String kopfzeileMitte = getPreferenceStore().getString(
				DefsSeiteneinstellungen.ID_KOPFZEILE_MITTE);

		final String kopfzeileRechts = getPreferenceStore().getString(
				DefsSeiteneinstellungen.ID_KOPFZEILE_RECHTS);

		zusammenstellung.setKopfFussZeilen(new String[] { kopfzeileLinks,
				kopfzeileMitte, kopfzeileRechts, fusszeileLinks,
				fusszeileMitte, fusszeileRechts });

	}

	@Override
	protected void performApply() {
		sichern();
	}

	@Override
	public boolean performOk() {
		performApply();
		return super.performOk();
	}

	@Override
	public void performHelp() {
		final String helpContextId = RahmenwerkActivator.PLUGIN_ID
				+ ".seiteneinstellungen";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	@Override
	public void init(final IWorkbench workbench) {
		final IPreferenceStore store = RahmenwerkActivator.getDefault()
				.getPreferenceStore();
		setPreferenceStore(store);
	}

	@Override
	protected void performDefaults() {
		zusammenstellung.setKopfFussZeilen(new String[] { null, null, null,
				null, null, null });
		super.performDefaults();
	}

}
