/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.ui.bildschirmfangen;

import org.eclipse.jface.dialogs.TrayDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;

/**
 * ber den Dialog wird der fr einen Schnappschuss zu verwendende Bereich
 * ausgewhlt.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BildschirmBereichAuswahlDialog.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class BildschirmBereichAuswahlDialog extends TrayDialog {

	/** Bereich des Bildschirms, der in eine Datei gespeichert werden soll. */
	private BildschirmSicherungsBereich sicherungsBereich = BildschirmSicherungsBereich.fenster;

	/**
	 * Konstruktor, erzeugt einen Dialog bezglich der bergebenen Parent-Shell.
	 * 
	 * @param parent
	 *            die Parent-Shell
	 */
	public BildschirmBereichAuswahlDialog(final Shell parent) {
		super(parent);
	}

	@Override
	protected Control createDialogArea(final Composite parent) {

		getShell().setText("Sicherungsbereich");

		final Composite panel = new Composite(parent, SWT.NONE);
		final FillLayout layout = new FillLayout();
		layout.marginHeight = 10;
		layout.marginWidth = 10;
		panel.setLayout(layout);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(panel);

		final Group group = new Group(panel, SWT.NONE);
		group.setLayout(new GridLayout());
		group.setText("Bereichswahl:");

		final Button radioButtonFenster = new Button(group, SWT.RADIO);
		radioButtonFenster.setText("Fenster");
		radioButtonFenster.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				sicherungsBereich = BildschirmSicherungsBereich.fenster;
			}
		});

		final Button radioButtonGesamterBildschirm = new Button(group,
				SWT.RADIO);
		radioButtonGesamterBildschirm.setText("Gesamter Bildschirmbereich");
		radioButtonGesamterBildschirm
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {
						sicherungsBereich = BildschirmSicherungsBereich.bildschirm;
					}
				});

		return group;
	}

	/**
	 * Enumerator zur Auswahl, welcher der Bildschirmbereiche via Screenshot
	 * gesichert werden soll.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum BildschirmSicherungsBereich {

		/** Sichern des aktuellen Fensters. */
		fenster("Fenster"),

		/** Sichern des gesamten Bildschirmbereiches. */
		bildschirm("Bildschirm");

		private final String name;

		private BildschirmSicherungsBereich(final String name) {
			this.name = name;
		}

		@Override
		public String toString() {
			return name;
		}
	}

	/**
	 * liefert den ausgewhlten Sicherungsbereich.
	 * 
	 * @return den Bereich
	 */
	public BildschirmSicherungsBereich getSicherungsBereich() {
		return sicherungsBereich;
	}

}
