/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.ui.internal;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.ToolBarContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.progress.UIJob;

import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;

/**
 * Advisor zur Initialisierung der Toolbars und Mens der
 * Rahmenwerk-Applikation.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: ApplicationActionBarAdvisor.java 46952 2013-11-05 08:41:56Z peuker $
 */
public class ApplicationActionBarAdvisor extends ActionBarAdvisor {

	/**
	 * Konstruktor, bernimmt die Konfigurationsdaten fr die Toolbars und
	 * Mens.
	 * 
	 * @param configurer
	 *            die Konfigurationsdaten
	 */
	public ApplicationActionBarAdvisor(final IActionBarConfigurer configurer) {
		super(configurer);

	}

	@Override
	protected void fillMenuBar(final IMenuManager menuBar) {
		RahmenwerkImpl.getInstanz().getRwToolBarManager()
				.setMenuManager(menuBar);
	}

	/**
	 * die Funjtion entfernt Toolbar-Eintrge, die im Rahmenwerk nicht gewnscht
	 * sind, aber durch Kombatibilitts-Layer der Eclipse-Workbench eingebracht
	 * werden.
	 * 
	 * @param window
	 *            das WorkbechWindow, in dem die Toolbars gescuht werden
	 * @param coolBar
	 *            der Coolbar-Manager mit den potentiellen unerwnschten
	 *            Elementen
	 * @param toolBarManager
	 *            der Toolbar-Manager, der nach dem Enfernen von Elementen
	 *            aktualisiert werden muss
	 */
	public static void removeUnwantedToolbars(final IWorkbenchWindow window,
			final ICoolBarManager coolBar, final IToolBarManager toolBarManager) {

		if ((window == null) || window.getWorkbench().isClosing()) {
			return;
		}

		for (final IContributionItem item : coolBar.getItems()) {
			if (item.isVisible()) {
				// TODO HACK beseitigen
				final String id = item.getId();
				if (id == null) {
					continue;
				}

				if ("undoredo.toolbar".equals(id) || "editorToolbar".equals(id)
						|| id.startsWith("org.eclipse")) {
					item.setVisible(false);
					toolBarManager.markDirty();
					coolBar.markDirty();
					coolBar.update(true);
				}
			}
		}
	}

	@Override
	protected void fillCoolBar(final ICoolBarManager coolBar) {

		final ToolBarManager toolBarManager = new ToolBarManager(SWT.HORIZONTAL
				| SWT.FLAT);
		final ToolBarContributionItem toolbar = new ToolBarContributionItem(
				toolBarManager, "rahmenwerk.toolbar");
		coolBar.add(toolbar);

		coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_ADD));
		coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_APP));
		coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_EDITOR));
		coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_FILE));
		coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_HELP));
		coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_MANAGING));
		coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_REORGANIZE));
		coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_SHOW_IN));

		RahmenwerkImpl.getInstanz().getRwToolBarManager()
				.setSymbolLeisteManager(coolBar, toolBarManager);

		final UIJob uiJob = new UIJob("ToolBarHack") {

			@Override
			public IStatus runInUIThread(final IProgressMonitor monitor) {
				ApplicationActionBarAdvisor.removeUnwantedToolbars(
						getActionBarConfigurer().getWindowConfigurer()
								.getWindow(), coolBar, toolBarManager);
				return Status.OK_STATUS;
			}
		};
		uiJob.schedule();
	}

	@Override
	protected void fillStatusLine(final IStatusLineManager statusLine) {
		RahmenwerkImpl.getInstanz().getRwToolBarManager()
				.setStatusLeisteManager(statusLine);
	}
}
