/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.ui.internal;

import javax.inject.Inject;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.e4.ui.css.swt.theme.IThemeEngine;
import org.eclipse.e4.ui.css.swt.theme.IThemeManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.application.IWorkbenchConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchAdvisor;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;

import de.bsvrz.buv.rw.basislib.dav.DavConnectionException;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;
import de.bsvrz.buv.rw.rw.internal.RahmenwerkTray;
import de.bsvrz.buv.rw.rw.login.LoginDialog;
import de.bsvrz.buv.rw.rw.login.LoginDialog.AnmeldeDaten;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Workbench-Advisor fr die Rahmenwerk-Applikation, initialisiert die
 * Eigenschaften des Workbench.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RahmenwerkWorkbenchAdvisor.java 49941 2014-04-02 11:16:18Z peuker $
 */
@SuppressWarnings("restriction")
public class RahmenwerkWorkbenchAdvisor extends WorkbenchAdvisor {

	private static final Debug LOGGER = Debug.getLogger();
	private RahmenwerkTray tray;

	@Inject
	private IThemeManager themeManager;

	private boolean onShutDown;

	@Override
	public WorkbenchWindowAdvisor createWorkbenchWindowAdvisor(
			final IWorkbenchWindowConfigurer configurer) {
		return new RahmenwerkWorkbenchWindowAdvisor(configurer);
	}

	@Override
	public void initialize(final IWorkbenchConfigurer configurer) {
		super.initialize(configurer);
		configurer.setSaveAndRestore(true);
	}

	@Override
	public String getInitialWindowPerspectiveId() {
		return RahmenwerkActivator.getDefault().getDefaultPerspectiveId();
	}

	@Override
	public IAdaptable getDefaultPageInput() {
		return ResourcesPlugin.getWorkspace().getRoot();
	}

	@Override
	public void preStartup() {

		tray = new RahmenwerkTray(getWorkbenchConfigurer().getWorkbench()
				.getDisplay().getSystemTray());

		boolean runLogin = true;
		String errorText = null;

		LoginDialog loginDialog = new LoginDialog(null);
		while (runLogin) {
			if (loginDialog.open() == Window.OK) {
				final AnmeldeDaten anmeldeDaten = loginDialog.getAnmeldeDaten();
				if (anmeldeDaten.isOnline()) {
					try {
						RahmenwerkImpl.getInstanz().login(anmeldeDaten);
						runLogin = false;
					} catch (final DavConnectionException e) {
						RahmenwerkWorkbenchAdvisor.LOGGER.fine(
								e.getLocalizedMessage(), e);
						if (e.getCause() != null) {
							errorText = e.getCause().getLocalizedMessage();
						}
						if ((errorText == null) || errorText.isEmpty()) {
							errorText = e.getLocalizedMessage();
						}
						loginDialog = new LoginDialog(errorText);
					}
				} else {
					RahmenwerkImpl.getInstanz().setBenutzerName(
							anmeldeDaten.getName());
					runLogin = false;
				}
			} else {
				tray.dispose();
				System.exit(0);
			}
		}

		updateTheme();

		RahmenwerkImpl.getInstanz().addDavVerbindungsListener(
				new DavVerbindungsListener() {

					@Override
					public void verbindungHergestellt(
							final DavVerbindungsEvent event) {
						updateTheme();
					}

					@Override
					public boolean verbindungHalten(
							final DavVerbindungsEvent event) {
						return false;
					}

					@Override
					public void verbindungGetrennt(
							final DavVerbindungsEvent event) {
						updateTheme();
					}
				});

		super.preStartup();
	}

	private void updateTheme() {
		if (onShutDown) {
			return;
		}

		if (themeManager != null) {
			getWorkbenchConfigurer().getWorkbench().getDisplay()
			.asyncExec(new Runnable() {

				@Override
				public void run() {
					final IThemeEngine engine = themeManager
							.getEngineForDisplay(getWorkbenchConfigurer()
									.getWorkbench().getDisplay());
					if (engine != null) {
						if (RahmenwerkImpl.getInstanz().isOnline()
								&& (RahmenwerkImpl.getInstanz()
										.getDavVerbindung()
										.getClientDavParameters()
										.getSimulationVariant() > 0)) {
							engine.setTheme(
									"de.bsvrz.buv.rw.rw.theme.simulation",
									true);
						} else {
							engine.setTheme(
									"de.bsvrz.buv.rw.rw.theme.default",
									true);
						}
					}
				}
			});
		}
	}

	@Override
	public boolean preShutdown() {
		final boolean result = MessageDialog.openConfirm(null,
				"Rahmenwerk beenden",
				"Soll die Anwendung wirklich beendet werden?");

		// TODO DavVerbindungsListener#verbindungHalten() hier behandeln

		if (result) {
			onShutDown = true;
		}

		return result;
	}

	@Override
	public void postShutdown() {
		final RahmenwerkImpl rahmenwerk = RahmenwerkImpl.getInstanz();
		if (rahmenwerk != null) {
			rahmenwerk.davVerbindungBeenden();
		}

		if (tray != null) {
			tray.dispose();
		}

		super.postShutdown();
	}
}
