/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.ui.internal;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IContributionManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.graphics.Point;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPreferenceConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PerspectiveAdapter;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;
import org.eclipse.ui.progress.UIJob;
import org.xml.sax.SAXException;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungChangeListener;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungLocation;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungOwnerType;
import de.bsvrz.buv.rw.basislib.einstellungen.Einstellungen;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsAdresse;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsEvent;
import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.internal.InternalServices;
import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;
import de.bsvrz.buv.rw.rw.menu.MenuGenerator;
import de.bsvrz.buv.rw.rw.menu.MenueLeiste;
import de.bsvrz.buv.rw.rw.menu.StatusLeiste;
import de.bsvrz.buv.rw.rw.menu.SymbolLeiste;
import de.bsvrz.buv.rw.rw.menu.editoren.MenueEditor;
import de.bsvrz.buv.rw.rw.menu.editoren.StatusleisteEditor;
import de.bsvrz.buv.rw.rw.menu.editoren.ToolbarEditor;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Advisor fr die Konfiguration des Workbecnh-Windows der
 * Rahmenwerk-Applikation.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RahmenwerkWorkbenchWindowAdvisor.java 49941 2014-04-02 11:16:18Z peuker $
 */
public class RahmenwerkWorkbenchWindowAdvisor extends WorkbenchWindowAdvisor {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Konstruktor.
	 *
	 * @param configurer
	 *            die Konfigurationsdaten fr das WorkbenchWindow.
	 */
	public RahmenwerkWorkbenchWindowAdvisor(
			final IWorkbenchWindowConfigurer configurer) {
		super(configurer);
	}

	@Override
	public ActionBarAdvisor createActionBarAdvisor(
			final IActionBarConfigurer configurer) {
		return new ApplicationActionBarAdvisor(configurer);
	}

	@Override
	public void preWindowOpen() {

		final IWorkbenchWindowConfigurer configurer = getWindowConfigurer();
		configurer.setInitialSize(new Point(1024, 768));

		final WorkBenchWindowTitleProvider titleProvider = new WorkBenchWindowTitleProvider(
				configurer);

		PlatformUI.getPreferenceStore().setValue(
				IWorkbenchPreferenceConstants.CLOSE_EDITORS_ON_EXIT, true);
		PlatformUI.getPreferenceStore().setValue(
				IWorkbenchPreferenceConstants.ENABLE_ANIMATIONS, true);
		PlatformUI.getPreferenceStore().setValue(
				IWorkbenchPreferenceConstants.SHOW_PROGRESS_ON_STARTUP, true);
		PlatformUI.getPreferenceStore().setValue(
				IWorkbenchPreferenceConstants.DOCK_PERSPECTIVE_BAR,
				IWorkbenchPreferenceConstants.TOP_RIGHT);
		PlatformUI.getPreferenceStore().setValue(
				IWorkbenchPreferenceConstants.SHOW_TRADITIONAL_STYLE_TABS,
				false);
		PlatformUI.getPreferenceStore().setValue(
				IWorkbenchPreferenceConstants.SHOW_MEMORY_MONITOR, false);

		configurer.setShowProgressIndicator(true);
		configurer.setShowCoolBar(false);
		configurer.setShowStatusLine(true);
		configurer.setShowPerspectiveBar(true);
		configurer.setShowFastViewBars(false);
		configurer.setTitle(titleProvider.getTitel());
	}

	@Override
	public void postWindowOpen() {
		final IWorkbenchWindow window = getWindowConfigurer().getWindow();

		window.addPerspectiveListener(new PerspectiveAdapter() {

			@Override
			public void perspectiveActivated(final IWorkbenchPage page,
					final IPerspectiveDescriptor perspective) {

				ApplicationActionBarAdvisor.removeUnwantedToolbars(
						page.getWorkbenchWindow(),
						RahmenwerkImpl.getInstanz().getRwToolBarManager()
								.getSymbolLeisteManager(), RahmenwerkImpl
								.getInstanz().getRwToolBarManager()
								.getSymbolLeisteManagerToolbar());
			}

			@Override
			public void perspectiveChanged(final IWorkbenchPage page,
					final IPerspectiveDescriptor perspective,
					final String changeId) {
				if ("resetComplete".equals(changeId)) {
					ApplicationActionBarAdvisor.removeUnwantedToolbars(
							page.getWorkbenchWindow(), RahmenwerkImpl
									.getInstanz().getRwToolBarManager()
									.getSymbolLeisteManager(), RahmenwerkImpl
									.getInstanz().getRwToolBarManager()
									.getSymbolLeisteManagerToolbar());
				}
			}

			@Override
			public void perspectiveClosed(final IWorkbenchPage page,
					final IPerspectiveDescriptor perspective) {

				final IPerspectiveDescriptor[] openPerspectives = page
						.getOpenPerspectives();
				if (openPerspectives.length <= 0) {
					try {
						final IWorkbench workbench = page.getWorkbenchWindow()
								.getWorkbench();
						if (!workbench.isClosing()) {
							workbench.showPerspective(RahmenwerkActivator
									.getDefault().getDefaultPerspectiveId(),
									page.getWorkbenchWindow());
						}
					} catch (final WorkbenchException e) {
						RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(e
								.getLocalizedMessage());
					}
				}

				super.perspectiveClosed(page, perspective);
			}
		});

		ladeMenues(window);
		RahmenwerkImpl.getInstanz().addDavVerbindungsListener(
				new DavVerbindungsListener() {

					@Override
					public void verbindungHergestellt(
							final DavVerbindungsEvent event) {
						ladeMenues(window);
					}

					@Override
					public boolean verbindungHalten(
							final DavVerbindungsEvent event) {
						return false;
					}

					@Override
					public void verbindungGetrennt(
							final DavVerbindungsEvent event) {
						ladeMenues(window);

						if (!window.getWorkbench().isClosing()) {

							final UIJob uiJob = new UIJob("Reconnect Dialog") {

								@Override
								public IStatus runInUIThread(
										final IProgressMonitor monitor) {
									final Dialog dialog = new ReconnectDialog(
											window.getShell());
									dialog.open();
									return Status.OK_STATUS;
								}
							};
							uiJob.schedule();
						}
					}
				});

		final Einstellungen einstellungen = InternalServices.getInstance()
				.getEinstellungen();
		einstellungen.addEinstellungsListener(new EinstellungChangeListener() {

			@Override
			public void einstellungEntfernt(final EinstellungsEvent event) {
				update(window, event);
			}

			@Override
			public void einstellungAngelegt(final EinstellungsEvent event) {
				update(window, event);
			}

			@Override
			public void einstellungAktualisiert(final EinstellungsEvent event) {
				update(window, event);
			}

			private void update(final IWorkbenchWindow workbenchWndow,
					final EinstellungsEvent event) {

				final UIJob job = new UIJob("Aktualisiere Mens") {

					@Override
					public IStatus runInUIThread(final IProgressMonitor monitor) {
						if (!workbenchWndow.getWorkbench().isClosing()) {
							if (MenueEditor.PARAM_STRUCT_MENULEISTE
									.equals(event.getAdresse().getId())) {
								ladeMenu(workbenchWndow);
							} else if (ToolbarEditor.PARAM_STRUCT_SYMBOLLEISTE
									.equals(event.getAdresse().getId())) {
								ladeToolbar(workbenchWndow);
							} else if (StatusleisteEditor.PARAM_STRUCT_STATUSLEISTE
									.equals(event.getAdresse().getId())) {
								ladeStatusLeiste(workbenchWndow);
							}
						}
						return Status.OK_STATUS;

					}

					@Override
					public boolean belongsTo(final Object family) {
						return Rahmenwerk.JOB_FAMILY.equals(family)
								|| super.belongsTo(family);
					}
				};
				job.schedule();
			}
		});
	}

	private void ladeMenues(final IWorkbenchWindow window) {
		if (!window.getWorkbench().isClosing()) {
			final UIJob job = new UIJob("Mens laden") {
				@Override
				public IStatus runInUIThread(final IProgressMonitor monitor) {
					if (!window.getWorkbench().isClosing()) {
						ladeMenu(window);
						ladeToolbar(window);
						ladeStatusLeiste(window);
					}
					return Status.OK_STATUS;
				}
			};
			job.schedule();
		}
	}

	private void ladeStatusLeiste(final IWorkbenchWindow window) {
		try {
			final String parameter = getMenuParameter(
					StatusleisteEditor.PARAM_STRUCT_STATUSLEISTE,
					"-statusXmlDatei");
			final IStatusLineManager statusLeisteManager = RahmenwerkImpl
					.getInstanz().getRwToolBarManager()
					.getStatusLeisteManager();
			if (parameter == null) {
				statusLeisteManager.removeAll();
				statusLeisteManager.update(true);
			} else {
				final StatusLeiste statusLeiste = new StatusLeiste(parameter);
				final MenuGenerator menuGenerator = new MenuGenerator(
						statusLeisteManager, statusLeiste);
				menuGenerator.generiereMenu(window);
			}
		} catch (final SAXException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		} catch (final IOException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		} catch (final ParserConfigurationException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		}
	}

	private void ladeToolbar(final IWorkbenchWindow window) {
		try {
			final IContributionManager toolBarManager = RahmenwerkImpl
					.getInstanz().getRwToolBarManager()
					.getSymbolLeisteManagerToolbar();
			final String parameter = getMenuParameter(
					ToolbarEditor.PARAM_STRUCT_SYMBOLLEISTE, "-symbolXmlDatei");
			if (parameter == null) {
				toolBarManager.removeAll();
				toolBarManager.update(true);
			} else {
				final SymbolLeiste symbolLeiste = new SymbolLeiste(parameter);
				final MenuGenerator menuGenerator = new MenuGenerator(
						toolBarManager, symbolLeiste);
				menuGenerator.generiereMenu(window);
			}
		} catch (final SAXException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		} catch (final IOException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		} catch (final ParserConfigurationException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		}
	}

	private void ladeMenu(final IWorkbenchWindow window) {
		try {
			final IMenuManager menuManager = RahmenwerkImpl.getInstanz()
					.getRwToolBarManager().getMenuManager();
			final String parameter = getMenuParameter(
					MenueEditor.PARAM_STRUCT_MENULEISTE, "-menueXmlDatei");
			if (parameter == null) {
				menuManager.removeAll();
				menuManager.update(true);
			} else {
				final MenueLeiste menueLeiste = new MenueLeiste(parameter);
				final MenuGenerator menuGenerator = new MenuGenerator(
						menuManager, menueLeiste);
				menuGenerator.generiereMenu(window);
			}
		} catch (final SAXException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		} catch (final IOException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		} catch (final ParserConfigurationException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		}
	}

	private String getMenuFromXmlFile(final String argument) {
		String result = null;
		final ArgumentList startParameter = RahmenwerkImpl.getInstanz()
				.getStartParameter();
		if (startParameter.hasArgument(argument)) {
			final String fileName = startParameter.fetchArgument(argument)
					.getValue();
			final File menuFile = new File(fileName);
			if (menuFile.exists() && menuFile.isFile()) {
				BufferedReader reader = null;
				try {
					final StringBuffer buffer = new StringBuffer();
					reader = new BufferedReader(new FileReader(menuFile));
					String line = null;
					do {
						line = reader.readLine();
						if (line != null) {
							buffer.append(line);
						}
					} while (line != null);
					result = buffer.toString();
				} catch (final IOException e) {
					RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
							"Fehler beim Lesen der Menu-Datei: " + fileName, e);
				} finally {
					if (reader != null) {
						try {
							reader.close();
						} catch (final IOException e) {
							RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
									"Fehler beim Schlieen der Menu-Datei: "
											+ fileName, e);
						}
					}
				}
			}
		}

		return result;
	}

	private String getMenuParameter(final String id, final String fileArgument) {
		final Einstellungen einstellungen = InternalServices.getInstance()
				.getEinstellungen();

		String result = getMenuFromXmlFile(fileArgument);

		if (result == null) {
			if (RahmenwerkImpl.getInstanz().isOnline()) {
				result = getParameterUserLokal(einstellungen, id);
				if ((result == null) || (result.trim().isEmpty())) {
					result = getParameterUserGlobal(einstellungen, id);
					if ((result == null) || (result.trim().isEmpty())) {
						result = getParameterSystemGlobal(einstellungen, id);
						if ((result == null) || (result.trim().isEmpty())) {
							result = getParameterSystemLokal(einstellungen, id);
						}
					}
				}
			} else {
				result = getParameterSystemLokal(einstellungen, id);
			}
		}

		return result;
	}

	private String getParameterSystemLokal(final Einstellungen einstellungen,
			final String id) {
		try {
			final EinstellungsAdresse adresse = new EinstellungsAdresse(null,
					id, EinstellungOwnerType.SYSTEM, null,
					EinstellungLocation.LOKAL);
			return (String) einstellungen.getValue(adresse);
		} catch (final IOException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		}
		return null;
	}

	private String getParameterSystemGlobal(final Einstellungen einstellungen,
			final String id) {
		try {
			final EinstellungsAdresse adresse = new EinstellungsAdresse(null,
					id, EinstellungOwnerType.SYSTEM, null,
					EinstellungLocation.NETZWERKWEIT);
			return (String) einstellungen.getValue(adresse);
		} catch (final IOException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		}

		return null;
	}

	private String getParameterUserLokal(final Einstellungen einstellungen,
			final String id) {
		try {
			final EinstellungsAdresse adresse = new EinstellungsAdresse(null,
					id, EinstellungOwnerType.BENUTZER, RahmenwerkImpl
							.getInstanz().getBenutzer().getPid(),
					EinstellungLocation.LOKAL);
			return (String) einstellungen.getValue(adresse);
		} catch (final IOException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		}

		return null;
	}

	private String getParameterUserGlobal(final Einstellungen einstellungen,
			final String id) {
		try {
			final EinstellungsAdresse adresse = new EinstellungsAdresse(null,
					id, EinstellungOwnerType.BENUTZER, RahmenwerkImpl
							.getInstanz().getBenutzer().getPid(),
					EinstellungLocation.NETZWERKWEIT);
			return (String) einstellungen.getValue(adresse);
		} catch (final IOException e) {
			RahmenwerkWorkbenchWindowAdvisor.LOGGER.error(
					e.getLocalizedMessage(), e);
		}

		return null;
	}
}
