/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.ui.internal;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimBar;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimElement;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;

import de.bsvrz.buv.rw.rw.RahmenwerkActivator;

/**
 * Prozessor fr die initiale Manipulation des Applikations-Modells.
 *
 * Es werden Bereinigungsarbeiten an den Toolbars vorgenommen.
 *
 * Es werden alle Standardelemente entfernt bis auf den
 * "minimierten PerspektiveStack", der angezeigt wird, wenn die Welcome-Page
 * aktiv ist.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwProcessor.java 50222 2014-04-22 11:55:26Z peuker $
 */
public class RwProcessor {

	@Inject
	private MApplication application;

	/** fhrt den Prozessor aus. */
	@Execute
	public void execute() {
		RahmenwerkActivator.getDefault().setApplication(application);
		final MTrimmedWindow window = (MTrimmedWindow) application
				.getChildren().get(0);
		if (window != null) {
			for (final MTrimBar trimBar : window.getTrimBars()) {
				final List<MTrimElement> oldChildren = new ArrayList<MTrimElement>(
						trimBar.getChildren());
				for (final MTrimElement element : oldChildren) {
					if (!element.getElementId().startsWith("PerspectiveStack")) {
						trimBar.getChildren().remove(element);
					}
				}
			}
		}
	}
}
