/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.ui.internal;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;

import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.basislib.util.RwInfoDef;
import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;
import de.bsvrz.buv.rw.rw.preferences.RahmenwerkPreference;

/**
 * Provider fr die Befllung des Titels des Windows der Rahmenwerk-Applikation.
 * 
 * Im Titel knnen Informationen aus dem Rahmenwerk-Anwendungsumfeld dargestellt
 * werden. Die Darstellung hngt vom Verbindungszustand der Applikation ab.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: WorkBenchWindowTitleProvider.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class WorkBenchWindowTitleProvider implements IPropertyChangeListener {

	private final IWorkbenchWindowConfigurer configurer;

	/**
	 * Konstruktor, verarbeitet die Konfifgurationsdaten des Workbench-Windows.
	 * 
	 * @param configurer
	 *            die Konfigurationsdaten
	 */
	public WorkBenchWindowTitleProvider(
			final IWorkbenchWindowConfigurer configurer) {
		this.configurer = configurer;
		RahmenwerkActivator.getDefault().getPreferenceStore()
				.addPropertyChangeListener(this);
		RahmenwerkImpl.getInstanz().addDavVerbindungsListener(
				new DavVerbindungsListener() {

					@Override
					public void verbindungHergestellt(
							final DavVerbindungsEvent event) {
						setTitleInUI(configurer);
					}

					@Override
					public boolean verbindungHalten(
							final DavVerbindungsEvent event) {
						return false;
					}

					@Override
					public void verbindungGetrennt(
							final DavVerbindungsEvent event) {
						setTitleInUI(configurer);
					}
				});
	}

	/**
	 * liefert den formatierten Titel fr das Workbecnh-Window.
	 * 
	 * @return den Titel
	 */
	public String getTitel() {

		String result;
		if (RahmenwerkImpl.getInstanz().isOnline()) {
			result = RwInfoDef
					.format(RahmenwerkActivator
							.getDefault()
							.getPreferenceStore()
							.getString(
									RahmenwerkPreference.ONLINE_TITLE_FORMAT
											.getId()));
		} else {
			result = RwInfoDef.format(RahmenwerkActivator
					.getDefault()
					.getPreferenceStore()
					.getString(
							RahmenwerkPreference.OFFLINE_TITLE_FORMAT.getId()));
		}

		return result;
	}

	@Override
	public void propertyChange(final PropertyChangeEvent event) {
		if (event.getSource().equals(
				RahmenwerkActivator.getDefault().getPreferenceStore())) {
			if (RahmenwerkPreference.ONLINE_TITLE_FORMAT.getId().equals(
					event.getProperty())) {
				if (RahmenwerkImpl.getInstanz().isOnline()) {
					configurer
							.getWindow()
							.getShell()
							.setText(
									RwInfoDef.format((String) event
											.getNewValue()));
				}
			} else if (RahmenwerkPreference.OFFLINE_TITLE_FORMAT.getId()
					.equals(event.getProperty())
					&& (!RahmenwerkImpl.getInstanz().isOnline())) {
				configurer
						.getWindow()
						.getShell()
						.setText(RwInfoDef.format((String) event.getNewValue()));
			}
		}
	}

	private void setTitleInUI(final IWorkbenchWindowConfigurer windowConfigurer) {
		final IWorkbench workbench = windowConfigurer.getWindow()
				.getWorkbench();
		if (workbench.isClosing()) {
			return;
		}

		final Display display = workbench.getDisplay();
		if ((!workbench.isClosing()) && (!display.isDisposed())) {
			display.asyncExec(new Runnable() {
				@Override
				public void run() {
					if ((!workbench.isClosing()) && (!display.isDisposed())) {
						windowConfigurer.getWindow().getShell()
								.setText(getTitel());
					}
				}
			});
		}
	}
}
