/*
 * Copyright (C) 2004 by Friederich Kupzog Elektronik & Software
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
    Author: Friederich Kupzog  
    fkmk@kupzog.de
    www.kupzog.de/fkmk
 */
package de.kupzog.examples;

import org.eclipse.swt.graphics.Point;

import de.kupzog.ktable.KTableCellEditor;
import de.kupzog.ktable.KTableCellRenderer;
import de.kupzog.ktable.KTableModel;
import de.kupzog.ktable.editors.KTableCellEditorMultilineText;

/**
 * @author Friederich Kupzog
 */
public class TownExampleModel implements KTableModel {

	private final int[] colWidths;

	private final TownExampleContent[] content;

	public TownExampleModel() {
		colWidths = new int[getColumnCount()];
		colWidths[0] = 120;
		colWidths[1] = 100;
		colWidths[2] = 180;

		content = new TownExampleContent[3];
		content[0] = new TownExampleContent("Aachen", "Germany");
		content[1] = new TownExampleContent("Cologne", "Germany");
		content[2] = new TownExampleContent("Edinburgh", "Scotland");

	}

	/*
	 * overridden from superclass
	 */
	@Override
	public Object getContentAt(final int col, final int row) {
		if (row == 0) // Header
		{
			if (col == 0) {
				return "Town";
			} else if (col == 1) {
				return "Country";
			} else {
				return "Notes";
			}
		} else {
			return content[row - 1];
		}
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public KTableCellEditor getCellEditor(final int col, final int row) {
		if ((row > 0) && (col == 2)) {
			return new KTableCellEditorMultilineText();
		}
		return null;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public void setContentAt(final int col, final int row, final Object value) {
		content[row - 1].notes = (String) value;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getRowCount() {
		return 4;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getFixedHeaderRowCount() {
		return 1;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getColumnCount() {
		return 3;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getFixedHeaderColumnCount() {
		return 0;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getColumnWidth(final int col) {
		return colWidths[col];
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public boolean isColumnResizable(final int col) {
		return (col != 0);
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public void setColumnWidth(final int col, final int value) {
		if (value > 120) {
			colWidths[col] = value;
		}
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getRowHeight(final int row) {
		if (row == 0) {
			return 20;
		}
		return 140;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public boolean isRowResizable(final int row) {
		return false;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getRowHeightMinimum() {
		return 20;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public void setRowHeight(final int row, final int value) {
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public KTableCellRenderer getCellRenderer(final int col, final int row) {
		if (row > 0) {
			return new TownExampleRenderer();
		}
		return KTableCellRenderer.defaultRenderer;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#belongsToCell(int, int)
	 */
	@Override
	public Point belongsToCell(final int col, final int row) {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#getTooltipAt(int, int)
	 */
	@Override
	public String getTooltipAt(final int col, final int row) {
		// no tooltip
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#getFixedSelectableRowCount()
	 */
	@Override
	public int getFixedSelectableRowCount() {
		// all fixed rows are non-selectable.
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#getFixedSelectableColumnCount()
	 */
	@Override
	public int getFixedSelectableColumnCount() {
		// all fixed columns are non-selctable.
		return 0;
	}
}
